@@ -1,17 +1,29 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-
 #include "afs/sysincludes.h"
 #include "afsincludes.h"
+
+#define MYBUNDLEID "org.openafs.filesystems.afs"
+extern struct vfsops afs_vfsops;
+
 #ifdef AFS_DARWIN80_ENV
 static vfstable_t afs_vfstable;
-static struct vfs_fsentry afs_vfsentry;
 extern struct vnodeopv_desc afs_vnodeop_opv_desc;
 extern struct vnodeopv_desc afs_dead_vnodeop_opv_desc;
 static struct vnodeopv_desc *afs_vnodeop_opv_desc_list[2] =
    { &afs_vnodeop_opv_desc, &afs_dead_vnodeop_opv_desc };
 
+struct vfs_fsentry afs_vfsentry = {
+  &afs_vfsops,
+  2,
+  afs_vnodeop_opv_desc_list,
+  0,
+  "afs",
+  VFS_TBLNOTYPENUM|VFS_TBLTHREADSAFE|VFS_TBL64BITREADY,
+  NULL,
+  NULL,
+};
 
 #include <sys/conf.h>
 #include <miscfs/devfs/devfs.h>
@@ -23,42 +35,30 @@ extern open_close_fcn_t afs_cdev_nop_openclose;
 extern ioctl_fcn_t afs_cdev_ioctl;
 static void *afs_cdev_devfs_handle;
 #else
-#ifdef AFS_DARWIN60_ENV		/* not in Kernel.framework anymore !?! */
 #include <sys/syscall.h>
-#else
-#include "sys/syscall.h"
-#endif
 struct vfsconf afs_vfsconf;
 #endif
 #include <mach/kmod.h>
 
-extern struct vfsops afs_vfsops;
 extern struct mount *afs_globalVFS;
 extern int Afs_xsetgroups();
-extern int afs_xioctl();
-extern int afs3_syscall();
 
 extern int ioctl();
 extern int setgroups();
 extern int maxvfsconf;
 kern_return_t
-afs_modload(struct kmod_info *ki, void *data)
+afs_modload(struct kmod_info *kmod_info, void *data)
 {
+    int ret;
     osi_Init();
 #ifdef AFS_DARWIN80_ENV
     MUTEX_SETUP();
     afs_global_lock = lck_mtx_alloc_init(openafs_lck_grp, 0);
-    memset(&afs_vfsentry, 0, sizeof(struct vfs_fsentry));
-    strcpy(afs_vfsentry.vfe_fsname, "afs");
-    afs_vfsentry.vfe_vfsops = &afs_vfsops;
-    afs_vfsentry.vfe_vopcnt = 2;
-    afs_vfsentry.vfe_opvdescs = afs_vnodeop_opv_desc_list;
-    afs_vfsentry.vfe_flags = VFS_TBLTHREADSAFE|VFS_TBLNOTYPENUM|VFS_TBL64BITREADY;
-    if (vfs_fsadd(&afs_vfsentry, &afs_vfstable)) {
-	printf("AFS: vfs_fsadd failed. aborting\n");
-	MUTEX_FINISH();
-	lck_mtx_free(afs_global_lock, openafs_lck_grp);
-	return KERN_FAILURE;
+
+    if (ret = vfs_fsadd(&afs_vfsentry, &afs_vfstable)) {
+	printf("AFS: vfs_fsadd failed. aborting: %d\n", ret);
+	afs_vfstable = NULL;
+	goto fsadd_out;
     }
     afs_cdev.d_open = &afs_cdev_nop_openclose;
     afs_cdev.d_close = &afs_cdev_nop_openclose;
@@ -66,14 +66,21 @@ afs_modload(struct kmod_info *ki, void *data)
     afs_cdev_major = cdevsw_add(-1, &afs_cdev);
     if (afs_cdev_major == -1) {
 	printf("AFS: cdevsw_add failed. aborting\n");
-        vfs_fsremove(afs_vfstable);
-	MUTEX_FINISH();
-	lck_mtx_free(afs_global_lock, openafs_lck_grp);
-	return KERN_FAILURE;
+	goto cdevsw_out;
     }
     afs_cdev_devfs_handle = devfs_make_node(makedev(afs_cdev_major, 0),
                                             DEVFS_CHAR, UID_ROOT, GID_WHEEL,
                                             0666, "openafs_ioctl", 0);
+    if (!afs_cdev_devfs_handle) {
+	printf("AFS: devfs_make_node failed. aborting\n");
+	cdevsw_remove(afs_cdev_major, &afs_cdev);
+    cdevsw_out:
+	vfs_fsremove(afs_vfstable);
+    fsadd_out:
+	MUTEX_FINISH();
+	lck_mtx_free(afs_global_lock, openafs_lck_grp);
+	return KERN_FAILURE;
+    }
 #else
     memset(&afs_vfsconf, 0, sizeof(struct vfsconf));
     strcpy(afs_vfsconf.vfc_name, "afs");
@@ -89,9 +96,6 @@ afs_modload(struct kmod_info *ki, void *data)
 	return KERN_FAILURE;
     }
     sysent[SYS_setgroups].sy_call = Afs_xsetgroups;
-#if 0
-    sysent[SYS_ioctl].sy_call = afs_xioctl;
-#endif
     sysent[AFS_SYSCALL].sy_call = afs3_syscall;
     sysent[AFS_SYSCALL].sy_narg = 5;
     sysent[AFS_SYSCALL].sy_parallel = 0;
@@ -99,11 +103,15 @@ afs_modload(struct kmod_info *ki, void *data)
     sysent[AFS_SYSCALL].sy_funnel = KERNEL_FUNNEL;
 #endif
 #endif
+    printf("%s kext loaded; %u pages at 0x%lx (load tag %u).\n",
+	   kmod_info->name, (unsigned)kmod_info->size / PAGE_SIZE,
+	   (unsigned long)kmod_info->address, (unsigned)kmod_info->id);
+
     return KERN_SUCCESS;
 }
 
 kern_return_t
-afs_modunload(struct kmod_info * ki, void *data)
+afs_modunload(struct kmod_info * kmod_info, void *data)
 {
     if (afs_globalVFS)
 	return KERN_FAILURE;
@@ -116,12 +124,7 @@ afs_modunload(struct kmod_info * ki, void *data)
     if (vfsconf_del("afs"))
 	return KERN_FAILURE;
     /* give up syscall entries for ioctl & setgroups, which we've stolen */
-#if 0
-    sysent[SYS_ioctl].sy_call = ioctl;
-#endif
-#ifndef AFS_DARWIN80_ENV
     sysent[SYS_setgroups].sy_call = setgroups;
-#endif
     /* give up the stolen syscall entry */
     sysent[AFS_SYSCALL].sy_narg = 0;
     sysent[AFS_SYSCALL].sy_call = nosys;
@@ -130,8 +133,10 @@ afs_modunload(struct kmod_info * ki, void *data)
     MUTEX_FINISH();
     lck_mtx_free(afs_global_lock, openafs_lck_grp);
 #endif
+    printf("%s kext unloaded; (load tag %u).\n",
+	   kmod_info->name, (unsigned)kmod_info->id);
     return KERN_SUCCESS;
 }
 
-KMOD_EXPLICIT_DECL(org.openafs.filesystems.afs, VERSION, afs_modload,
+KMOD_EXPLICIT_DECL(MYBUNDLEID, VERSION, afs_modload,
 		   afs_modunload)
