@@ -15,13 +15,8 @@
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"	/* afs statistics */
 
-
-
-static int osi_TimedSleep(char *event, afs_int32 ams, int aintok);
-
 static char waitV;
 
-
 void
 afs_osi_InitWaitHandle(struct afs_osi_WaitHandle *achandle)
 {
@@ -60,7 +55,7 @@ afs_osi_Wait(afs_int32 ams, struct afs_osi_WaitHandle *ahandle, int aintok)
     do {
 	AFS_ASSERT_GLOCK();
 	code = 0;
-	code = osi_TimedSleep(&waitV, ams, aintok);
+	code = afs_osi_TimedSleep(&waitV, ams, aintok);
 
 	if (code)
 	    break;		/* if something happened, quit now */
@@ -154,7 +149,7 @@ afs_osi_SleepSig(void *event)
     return 0;
 }
 
-/* osi_TimedSleep
+/* afs_osi_TimedSleep
  * 
  * Arguments:
  * event - event to sleep on
@@ -163,8 +158,8 @@ afs_osi_SleepSig(void *event)
  *
  * Returns 0 if timeout and EINTR if signalled.
  */
-static int
-osi_TimedSleep(char *event, afs_int32 ams, int aintok)
+int
+afs_osi_TimedSleep(void *event, afs_int32 ams, int aintok)
 {
     int code = 0;
     struct afs_event *evp;
@@ -173,7 +168,6 @@ osi_TimedSleep(char *event, afs_int32 ams, int aintok)
     ticks.tv_sec = ams / 1000;
     ticks.tv_nsec = (ams - (ticks.tv_sec * 1000)) * 1000000;
 
-
     evp = afs_getevent(event);
 
     AFS_ASSERT_GLOCK();
