@@ -55,11 +55,7 @@ extern void afs_mpservice(void *);
 extern struct vfsops Afs_vfsops, *afs_vfsopsp;
 extern struct vnodeops Afs_vnodeops, *afs_vnodeopsp;
 extern void (*afsidestroyp) (struct inode *);
-extern void (*afsdptoipp) (struct efs_dinode *, struct inode *);
-extern void (*afsiptodpp) (struct inode *, struct efs_dinode *);
 extern void afsidestroy(struct inode *);
-extern void afsdptoip(struct efs_dinode *, struct inode *);
-extern void afsiptodp(struct inode *, struct efs_dinode *);
 extern int (*idbg_prafsnodep) (vnode_t *);
 extern int (*idbg_afsvfslistp) (void);
 extern int idbg_prafsnode(vnode_t *);
@@ -91,8 +87,6 @@ Afs_init(struct vfssw *vswp, int fstype)
     afs_vnodeopsp = &Afs_vnodeops;
     afs_vfsopsp = &Afs_vfsops;
     afsidestroyp = afsidestroy;
-    afsiptodpp = afsiptodp;
-    afsdptoipp = afsdptoip;
     idbg_prafsnodep = idbg_prafsnode;
     idbg_afsvfslistp = idbg_afsvfslist;
     NewSystemCall(AFS_SYSCALL, Afs_syscall, 6);
@@ -207,7 +201,7 @@ afs_unmount(OSI_VFS_ARG(afsp), flags, cr)
 {
     struct vcache *tvc;
     vnode_t *vp, *rootvp = NULL;
-    register struct afs_q *tq;
+    struct afs_q *tq;
     struct afs_q *uq;
     int error, fv_slept;
     OSI_VFS_CONVERT(afsp);
@@ -222,11 +216,12 @@ afs_unmount(OSI_VFS_ARG(afsp), flags, cr)
      * EBUSY if any still in use
      */
     ObtainWriteLock(&afs_xvcache, 172);
+ retry:
     for (tq = VLRU.prev; tq != &VLRU; tq = uq) {
 	tvc = QTOV(tq);
 	uq = QPrev(tq);
 	vp = (vnode_t *) tvc;
-	if (error = afs_FlushVCache(tvc, &fv_slept))
+	if (error = afs_FlushVCache(tvc, &fv_slept)) {
 	    if (vp->v_flag & VROOT) {
 		rootvp = vp;
 		continue;
@@ -234,6 +229,10 @@ afs_unmount(OSI_VFS_ARG(afsp), flags, cr)
 		ReleaseWriteLock(&afs_xvcache);
 		return error;
 	    }
+	}
+	if (fv_slept) {
+	    goto retry;
+	}
     }
 
     /*
@@ -266,13 +265,13 @@ afs_root(OSI_VFS_ARG(afsp), avpp)
     OSI_VFS_DECL(afsp);
      struct vnode **avpp;
 {
-    register afs_int32 code = 0;
+    afs_int32 code = 0;
     struct vrequest treq;
-    register struct vcache *tvp = 0;
+    struct vcache *tvp = 0;
     OSI_VFS_CONVERT(afsp);
 
     AFS_STATCNT(afs_root);
-    if (afs_globalVp && (afs_globalVp->states & CStatd)) {
+    if (afs_globalVp && (afs_globalVp->f.states & CStatd)) {
 	tvp = afs_globalVp;
     } else {
 	if (afs_globalVp) {
@@ -373,7 +372,7 @@ afs_sync(OSI_VFS_DECL(afsp),
     struct vcache *tvc;
     struct vnode *vp;
     afs_uint32 lvcachegen;
-    register struct afs_q *tq;
+    struct afs_q *tq;
     struct afs_q *uq;
     int s;
     OSI_VFS_CONVERT(afsp);
@@ -457,7 +456,7 @@ afs_sync(OSI_VFS_DECL(afsp),
 
 	AFS_GUNLOCK();
 	if (flags & SYNC_CLOSE) {
-	    PFLUSHINVALVP(vp, (off_t) 0, (off_t) tvc->m.Length);
+	    PFLUSHINVALVP(vp, (off_t) 0, (off_t) tvc->f.m.Length);
 	}
 #ifdef AFS_SGI61_ENV
 	else if (flags & SYNC_PDFLUSH) {
@@ -470,18 +469,18 @@ afs_sync(OSI_VFS_DECL(afsp),
 
 	if ((flags & SYNC_DELWRI) && AFS_VN_DIRTY(vp)) {
 #ifdef AFS_SGI61_ENV
-	    PFLUSHVP(vp, (off_t) tvc->m.Length,
+	    PFLUSHVP(vp, (off_t) tvc->f.m.Length,
 		     (flags & SYNC_WAIT) ? 0 : B_ASYNC, error);
 #else /* AFS_SGI61_ENV */
 	    if (flags & SYNC_WAIT)
 		/* push all and wait */
-		PFLUSHVP(vp, (off_t) tvc->m.Length, (off_t) 0, error);
+		PFLUSHVP(vp, (off_t) tvc->f.m.Length, (off_t) 0, error);
 	    else if (flags & SYNC_BDFLUSH) {
 		/* push oldest */
 		error = pdflush(vp, B_ASYNC);
 	    } else {
 		/* push all but don't wait */
-		PFLUSHVP(vp, (off_t) tvc->m.Length, (off_t) B_ASYNC, error);
+		PFLUSHVP(vp, (off_t) tvc->f.m.Length, (off_t) B_ASYNC, error);
 	    }
 #endif /* AFS_SGI61_ENV */
 	}
@@ -517,8 +516,8 @@ afs_vget(OSI_VFS_DECL(afsp), vnode_t ** avcp, struct fid * fidp)
 {
     struct VenusFid vfid;
     struct vrequest treq;
-    register struct cell *tcell;
-    register afs_int32 code = 0;
+    struct cell *tcell;
+    afs_int32 code = 0;
     afs_int32 ret;
 
 #if defined(AFS_SGI64_ENV) && defined(CKPT) && !defined(_R5000_CVT_WAR)
