@@ -15,7 +15,7 @@
  * afs_readdir_move
  * afs_bulkstat_send
  * afs_readdir/afs_readdir2(HP)
- * afs_readdir1 - HP and DUX NFS versions
+ * afs_readdir1 - HP NFS version
  * 
  */
 
@@ -29,19 +29,18 @@
 #include "afs/afs_cbqueue.h"
 #include "afs/nfsclient.h"
 #include "afs/afs_osidnlc.h"
+#if defined(AFS_NBSD40_ENV)
+#include <ufs/ufs/ufs_extern.h> /* direct_pool */
+#endif
 
-
-#if	defined(AFS_HPUX1122_ENV)
+#if defined(AFS_HPUX1122_ENV)
 #define DIRPAD 7
+#elif defined(AFS_NBSD40_ENV)
+#define DIRPAD 4
 #else
 #define DIRPAD 3
 #endif
-/**
- * A few definitions. This is until we have a proper header file
- * which ahs prototypes for all functions
- */
 
-extern struct DirEntry *afs_dir_GetBlob();
 /*
  * AFS readdir vnodeop and bulk stat support.
  */
@@ -67,10 +66,10 @@ extern struct DirEntry *afs_dir_GetBlob();
 int
 BlobScan(struct dcache * afile, afs_int32 ablob)
 {
-    register afs_int32 relativeBlob;
+    afs_int32 relativeBlob;
     afs_int32 pageBlob;
-    register struct PageHeader *tpe;
-    register afs_int32 i;
+    struct PageHeader *tpe;
+    afs_int32 i;
 
     AFS_STATCNT(BlobScan);
     /* advance ablob over free and header blobs */
@@ -96,7 +95,7 @@ BlobScan(struct dcache * afile, afs_int32 ablob)
 	}
 	/* now relativeBlob is the page-relative first allocated blob,
 	 * or EPP (if there are none in this page). */
-	DRelease((struct buffer *)tpe, 0);
+	DRelease(tpe, 0);
 	if (i != EPP)
 	    return i + pageBlob;
 	ablob = pageBlob + EPP;	/* go around again */
@@ -104,6 +103,7 @@ BlobScan(struct dcache * afile, afs_int32 ablob)
     /* never get here */
 }
 
+
 #if !defined(AFS_LINUX20_ENV)
 /* Changes to afs_readdir which affect dcache or vcache handling or use of
  * bulk stat data should also be reflected in the Linux specific verison of
@@ -146,6 +146,11 @@ struct min_direct {		/* miniature direct structure */
     u_short d_reclen;
     u_char d_type;
     u_char d_namlen;
+#elif defined(AFS_NBSD40_ENV)
+    ino_t d_fileno;		/* file number of entry */
+    uint16_t d_reclen;		/* length of this record */
+    uint16_t d_namlen;		/* length of string in d_name */
+    uint8_t  d_type; 		/* file type, see below */
 #elif defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
     afs_uint32 d_fileno;
     u_short d_reclen;
@@ -168,7 +173,7 @@ struct min_direct {		/* miniature direct structure */
 };
 #endif /* AFS_SGI_ENV */
 
-#if	defined(AFS_HPUX_ENV) || defined(AFS_OSF_ENV)
+#if	defined(AFS_HPUX_ENV)
 struct minnfs_direct {
     afs_int32 d_off;		/* XXX */
     afs_uint32 d_fileno;
@@ -232,18 +237,16 @@ int afs_rd_stash_i = 0;
 
 #if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 int
-afs_readdir_type(avc, ade)
-     struct DirEntry *ade;
-     struct vcache *avc;
+afs_readdir_type(struct vcache *avc, struct DirEntry *ade)
 {
     struct VenusFid tfid;
     struct vcache *tvc;
     int vtype;
-    tfid.Cell = avc->fid.Cell;
-    tfid.Fid.Volume = avc->fid.Fid.Volume;
+    tfid.Cell = avc->f.fid.Cell;
+    tfid.Fid.Volume = avc->f.fid.Fid.Volume;
     tfid.Fid.Vnode = ntohl(ade->fid.vnode);
     tfid.Fid.Unique = ntohl(ade->fid.vunique);
-    if ((avc->states & CForeign) == 0 && (ntohl(ade->fid.vnode) & 1)) {
+    if ((avc->f.states & CForeign) == 0 && (ntohl(ade->fid.vnode) & 1)) {
 	return DT_DIR;
     }
     ObtainReadLock(&afs_xvcache);
@@ -252,7 +255,7 @@ afs_readdir_type(avc, ade)
 	if (tvc->mvstat) {
 	    afs_PutVCache(tvc);
 	    return DT_DIR;
-	} else if (((tvc->states) & (CStatd | CTruth))) {
+	} else if (((tvc->f.states) & (CStatd | CTruth))) {
 	    /* CTruth will be set if the object has
 	     *ever* been statd */
 	    vtype = vType(tvc);
@@ -262,8 +265,9 @@ afs_readdir_type(avc, ade)
 	    else if (vtype == VREG)
 		return DT_REG;
 	    /* Don't do this until we're sure it can't be a mtpt */
-	    /* else if (vtype == VLNK)
-	     * type=DT_LNK; */
+	    /* if we're CStatd and CTruth and mvstat==0, it's a link */
+	    else if (vtype == VLNK)
+		return DT_LNK;
 	    /* what other types does AFS support? */
 	} else
 	    afs_PutVCache(tvc);
@@ -282,22 +286,19 @@ afs_readdir_type(avc, ade)
 #endif
 char bufofzeros[64];		/* gotta fill with something */
 
-int
-afs_readdir_move(de, vc, auio, slen, rlen, off)
-     struct DirEntry *de;
-     struct vcache *vc;
-     struct uio *auio;
-     int slen;
 #ifdef AFS_SGI65_ENV
-     ssize_t rlen;
+int
+afs_readdir_move(struct DirEntry *de, struct vcache *vc, struct uio *auio, 
+		 int slen, ssize_t rlen, afs_size_t off)
 #else
-     int rlen;
+int
+afs_readdir_move(struct DirEntry *de, struct vcache *vc, struct uio *auio, 
+		 int slen, int rlen, afs_size_t off)
 #endif
-     afs_size_t off;
 {
     int code = 0;
     struct volume *tvp;
-    afs_uint32 Volume = vc->fid.Fid.Volume;
+    afs_uint32 Volume = vc->f.fid.Fid.Volume;
     afs_uint32 Vnode  = de->fid.vnode;
 #if	defined(AFS_SUN56_ENV)
     struct dirent64 *direntp;
@@ -318,11 +319,11 @@ afs_readdir_move(de, vc, auio, slen, rlen, off)
 	/* This is the '.' entry; if we are a volume root, we need to
 	 * ignore the directory and use the inum for the mount point.
 	 */
-	if (!FidCmp(&afs_rootFid, &vc->fid)) {
+	if (!FidCmp(&afs_rootFid, &vc->f.fid)) {
 	    Volume = 0;
 	    Vnode  = 2;
 	} else if (vc->mvstat == 2) {
-	    tvp = afs_GetVolume(&vc->fid, 0, READ_LOCK);
+	    tvp = afs_GetVolume(&vc->f.fid, 0, READ_LOCK);
 	    if (tvp) {
 		Volume = tvp->mtpoint.Fid.Volume;
 		Vnode  = tvp->mtpoint.Fid.Vnode;
@@ -335,7 +336,7 @@ afs_readdir_move(de, vc, auio, slen, rlen, off)
 	 * because we might be a volume root (so our parent is in a
 	 * different volume), or our parent might be a volume root
 	 * (so we actually want the mount point) or BOTH! */
-	if (!FidCmp(&afs_rootFid, &vc->fid)) {
+	if (!FidCmp(&afs_rootFid, &vc->f.fid)) {
 	    /* We are the root of the AFS root, and thus our own parent */
 	    Volume = 0;
 	    Vnode  = 2;
@@ -366,7 +367,7 @@ afs_readdir_move(de, vc, auio, slen, rlen, off)
 	} else if (de->fid.vnode == 1 && de->fid.vunique == 1) {
 	    /* XXX The above test is evil and probably breaks DFS */
 	    /* Parent directory is a volume root; use the right inum */
-	    tvp = afs_GetVolume(&vc->fid, 0, READ_LOCK);
+	    tvp = afs_GetVolume(&vc->f.fid, 0, READ_LOCK);
 	    if (tvp) {
 		if (tvp->cell == afs_rootFid.Cell
 		    && tvp->volume == afs_rootFid.Fid.Volume) {
@@ -499,20 +500,30 @@ afs_readdir_move(de, vc, auio, slen, rlen, off)
 		    auio, code);
 #else /* AFS_SGI_ENV */
     AFS_MOVE_UNLOCK();
-    AFS_UIOMOVE((char *)&sdirEntry, sizeof(sdirEntry), UIO_READ, auio, code);
-
+#if defined(AFS_NBSD40_ENV)
+    {
+	struct dirent *dp;
+	dp = (struct dirent *) pool_get(&ufs_direct_pool, PR_WAITOK);
+	dp->d_ino =  (Volume << 16) + ntohl(Vnode);
+	FIXUPSTUPIDINODE(dp->d_ino);
+	dp->d_reclen = rlen;
+	strcpy(dp->d_name, de->name);
+	AFS_UIOMOVE((char*) dp, sizeof(struct dirent), UIO_READ, auio, code);
+	pool_put(&ufs_direct_pool, dp);
+    }
+#else
+    AFS_UIOMOVE((char *) &sdirEntry, sizeof(sdirEntry), UIO_READ, auio, code);
     if (code == 0) {
 	AFS_UIOMOVE(de->name, slen, UIO_READ, auio, code);
     }
-
     /* pad out the remaining characters with zeros */
     if (code == 0) {
 	AFS_UIOMOVE(bufofzeros, ((slen + 1 + DIRPAD) & ~DIRPAD) - slen,
 		    UIO_READ, auio, code);
     }
+#endif
     AFS_MOVE_LOCK();
 #endif /* AFS_SGI_ENV */
-
     /* pad out the difference between rlen and slen... */
     if (DIRSIZ_LEN(slen) < rlen) {
 	AFS_MOVE_UNLOCK();
@@ -548,9 +559,7 @@ afs_readdir_move(de, vc, auio, slen, rlen, off)
  */
 
 void
-afs_bulkstat_send(avc, req)
-     struct vcache *avc;
-     struct vrequest *req;
+afs_bulkstat_send(struct vcache *avc, struct vrequest *req)
 {
     afs_rd_stash_i = 0;
 }
@@ -561,22 +570,19 @@ afs_bulkstat_send(avc, req)
 */
 
 int
-#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV) || defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
-afs_readdir(OSI_VC_ARG(avc), auio, acred, eofp)
-     int *eofp;
+#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+afs_readdir(OSI_VC_DECL(avc), struct uio *auio, afs_ucred_t *acred, 
+	    int *eofp)
 #else
 #if defined(AFS_HPUX100_ENV)
-afs_readdir2(OSI_VC_ARG(avc), auio, acred)
+afs_readdir2(OSI_VC_DECL(avc), struct uio *auio, afs_ucred_t *acred)
 #else
-afs_readdir(OSI_VC_ARG(avc), auio, acred)
+afs_readdir(OSI_VC_DECL(avc), struct uio *auio, afs_ucred_t *acred)
 #endif
 #endif
-     OSI_VC_DECL(avc);
-     struct uio *auio;
-     struct AFS_UCRED *acred;
 {
     struct vrequest treq;
-    register struct dcache *tdc;
+    struct dcache *tdc;
     afs_size_t origOffset, tlen;
     afs_int32 len;
     int code = 0;
@@ -627,14 +633,16 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 #endif /* AFS_SGI61_ENV */
 #endif /* defined(AFS_SGI53_ENV) */
 
-#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV) || defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
     /* Not really used by the callee so we ignore it for now */
     if (eofp)
 	*eofp = 0;
 #endif
+#ifndef AFS_64BIT_CLIENT
     if (AfsLargeFileUio(auio)	/* file is large than 2 GB */
 	||AfsLargeFileSize(AFS_UIO_OFFSET(auio), AFS_UIO_RESID(auio)))
 	return EFBIG;
+#endif
 
     if ((code = afs_InitReq(&treq, acred))) {
 #ifdef	AFS_HPUX_ENV
@@ -644,6 +652,9 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
     }
     /* update the cache entry */
     afs_InitFakeStat(&fakestate);
+
+    AFS_DISCON_LOCK();
+
     code = afs_EvalFakeStat(&avc, &fakestate, &treq);
     if (code)
 	goto done;
@@ -667,9 +678,9 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
      * 1. The cache data is being fetched by another process.
      * 2. The cache data is no longer valid
      */
-    while ((avc->states & CStatd)
+    while ((avc->f.states & CStatd)
 	   && (tdc->dflags & DFFetching)
-	   && hsame(avc->m.DataVersion, tdc->f.versionNo)) {
+	   && hsame(avc->f.m.DataVersion, tdc->f.versionNo)) {
 	afs_Trace4(afs_iclSetp, CM_TRACE_DCACHEWAIT, ICL_TYPE_STRING,
 		   __FILE__, ICL_TYPE_INT32, __LINE__, ICL_TYPE_POINTER, tdc,
 		   ICL_TYPE_INT32, tdc->dflags);
@@ -679,8 +690,8 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 	ObtainReadLock(&avc->lock);
 	ObtainReadLock(&tdc->lock);
     }
-    if (!(avc->states & CStatd)
-	|| !hsame(avc->m.DataVersion, tdc->f.versionNo)) {
+    if (!(avc->f.states & CStatd)
+	|| !hsame(avc->f.m.DataVersion, tdc->f.versionNo)) {
 	ReleaseReadLock(&tdc->lock);
 	ReleaseReadLock(&avc->lock);
 	afs_PutDCache(tdc);
@@ -720,14 +731,18 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 	origOffset = AFS_UIO_OFFSET(auio);
 	/* scan for the next interesting entry scan for in-use blob otherwise up point at
 	 * this blob note that ode, if non-zero, also represents a held dir page */
-	if (!(us = BlobScan(tdc, (origOffset >> 5)))
-	    || !(nde = (struct DirEntry *)afs_dir_GetBlob(tdc, us))) {
+	us = BlobScan(tdc, (origOffset >> 5));
+
+	if (us)
+	   afs_dir_GetVerifiedBlob(tdc, us, &nde);
+
+	if (us == 0 || nde == NULL) {
 	    /* failed to setup nde, return what we've got, and release ode */
 	    if (len) {
 		/* something to hand over. */
 #ifdef	AFS_HPUX_ENV
 		sdirEntry->d_fileno =
-		    (avc->fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
+		    (avc->f.fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
 		FIXUPSTUPIDINODE(sdirEntry->d_fileno);
 		sdirEntry->d_reclen = rlen = AFS_UIO_RESID(auio);
 		sdirEntry->d_namlen = o_slen;
@@ -768,16 +783,15 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 	    } else {
 		/* nothin to hand over */
 	    }
-#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV) || defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 	    if (eofp)
 		*eofp = 1;	/* Set it properly */
 #endif
 	    if (ode)
-		DRelease((struct buffer *)ode, 0);
+		DRelease(ode, 0);
 	    goto dirend;
 	}
 	/* by here nde is set */
-
 	/* Do we have enough user space to carry out our mission? */
 #if defined(AFS_SGI_ENV)
 	n_slen = strlen(nde->name) + 1;	/* NULL terminate */
@@ -792,11 +806,11 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 	if (DIRSIZ_LEN(n_slen) >= (AFS_UIO_RESID(auio) - len)) {
 #endif /* AFS_SGI53_ENV */
 	    /* No can do no more now; ya know... at this time */
-	    DRelease((struct buffer *)nde, 0);	/* can't use this one. */
+	    DRelease(nde, 0);	/* can't use this one. */
 	    if (len) {
 #ifdef	AFS_HPUX_ENV
 		sdirEntry->d_fileno =
-		    (avc->fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
+		    (avc->f.fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
 		FIXUPSTUPIDINODE(sdirEntry->d_fileno);
 		sdirEntry->d_reclen = rlen = AFS_UIO_RESID(auio);
 		sdirEntry->d_namlen = o_slen;
@@ -840,7 +854,7 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 		code = EINVAL;
 	    }
 	    if (ode)
-		DRelease((struct buffer *)ode, 0);
+		DRelease(ode, 0);
 	    goto dirend;
 	}
 
@@ -851,7 +865,7 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 	if (len) {
 #ifdef	AFS_HPUX_ENV
 	    sdirEntry->d_fileno =
-		(avc->fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
+		(avc->f.fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
 	    FIXUPSTUPIDINODE(sdirEntry->d_fileno);
 	    sdirEntry->d_reclen = rlen = len;
 	    sdirEntry->d_namlen = o_slen;
@@ -890,12 +904,12 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 	len = DIRSIZ_LEN(o_slen = n_slen);
 #endif /* AFS_SGI53_ENV */
 	if (ode)
-	    DRelease((struct buffer *)ode, 0);
+	    DRelease(ode, 0);
 	ode = nde;
 	AFS_UIO_SETOFFSET(auio, (afs_int32) ((us + afs_dir_NameBlobs(nde->name)) << 5));
     }
     if (ode)
-	DRelease((struct buffer *)ode, 0);
+	DRelease(ode, 0);
 
   dirend:
     ReleaseReadLock(&tdc->lock);
@@ -906,58 +920,46 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 #ifdef	AFS_HPUX_ENV
     osi_FreeSmallSpace((char *)sdirEntry);
 #endif
+    AFS_DISCON_UNLOCK();
     afs_PutFakeStat(&fakestate);
     code = afs_CheckCode(code, &treq, 28);
     return code;
 }
 
-#if	defined(AFS_HPUX_ENV) || defined(AFS_OSF_ENV)
-#ifdef	AFS_OSF_ENV
-afs1_readdir(avc, auio, acred, eofp)
-     int *eofp;
-#else
-afs1_readdir(avc, auio, acred)
-#endif
-     struct vcache *avc;
-     struct uio *auio;
-     struct AFS_UCRED *acred;
+#if defined(AFS_HPUX_ENV)
+int
+afs1_readdir(struct vcache *avc, struct uio *auio, afs_ucred_t *acred)
 {
     struct vrequest treq;
-    register struct dcache *tdc;
+    struct dcache *tdc;
     afs_size_t origOffset, len;
     int code = 0;
     struct DirEntry *ode = 0, *nde = 0;
     int o_slen = 0, n_slen = 0;
     afs_uint32 us;
-#if	defined(AFS_HPUX_ENV) || defined(AFS_OSF_ENV)
     /*
-     * XXX All the hacks for alloced sdirEntry and inlining of afs_readdir_move instead of calling
-     * it is necessary for hpux due to stack problems that seem to occur when coming thru the nfs
+     * XXX All the hacks for alloced sdirEntry and inlining of
+     * afs_readdir_move instead of calling it is necessary for hpux due to
+     * stack problems that seem to occur when coming thru the nfs
      * translator side XXX
      */
     struct minnfs_direct *sdirEntry = (struct minnfs_direct *)
 	osi_AllocSmallSpace(sizeof(struct min_direct));
     afs_int32 rlen;
-#endif
+
     struct afs_fakestat_state fakestate;
 
     AFS_STATCNT(afs_readdir);
-#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV) || defined(AFS_OSF_ENV)
-    if (eofp)
-	*eofp = 0;
-#endif
     if (code = afs_InitReq(&treq, acred)) {
-#ifdef	AFS_HPUX_ENV
 	osi_FreeSmallSpace((char *)sdirEntry);
-#endif
 	return code;
     }
     afs_InitFakeStat(&fakestate);
+    AFS_DISCON_LOCK();
     code = afs_EvalFakeStat(&avc, &fakestate, &treq);
     if (code) {
-#ifdef	AFS_HPUX_ENV
 	osi_FreeSmallSpace((char *)sdirEntry);
-#endif
+	AFS_DISCON_UNLOCK();
 	afs_PutFakeStat(&fakestate);
 	return code;
     }
@@ -981,9 +983,9 @@ afs1_readdir(avc, auio, acred)
      * 1. The cache data is being fetched by another process.
      * 2. The cache data is no longer valid
      */
-    while ((avc->states & CStatd)
+    while ((avc->f.states & CStatd)
 	   && (tdc->dflags & DFFetching)
-	   && hsame(avc->m.DataVersion, tdc->f.versionNo)) {
+	   && hsame(avc->f.m.DataVersion, tdc->f.versionNo)) {
 	afs_Trace4(afs_iclSetp, CM_TRACE_DCACHEWAIT, ICL_TYPE_STRING,
 		   __FILE__, ICL_TYPE_INT32, __LINE__, ICL_TYPE_POINTER, tdc,
 		   ICL_TYPE_INT32, tdc->dflags);
@@ -993,8 +995,8 @@ afs1_readdir(avc, auio, acred)
 	ObtainReadLock(&avc->lock);
 	ObtainReadLock(&tdc->lock);
     }
-    if (!(avc->states & CStatd)
-	|| !hsame(avc->m.DataVersion, tdc->f.versionNo)) {
+    if (!(avc->f.states & CStatd)
+	|| !hsame(avc->f.m.DataVersion, tdc->f.versionNo)) {
 	ReleaseReadLock(&tdc->lock);
 	ReleaseReadLock(&avc->lock);
 	afs_PutDCache(tdc);
@@ -1002,22 +1004,24 @@ afs1_readdir(avc, auio, acred)
     }
 
     len = 0;
-#ifdef AFS_HPUX_ENV
     auio->uio_fpflags = 0;
-#endif
     while (code == 0) {
 	origOffset = AFS_UIO_OFFSET(auio);
 
-	/* scan for the next interesting entry scan for in-use blob otherwise up point at
-	 * this blob note that ode, if non-zero, also represents a held dir page */
-	if (!(us = BlobScan(tdc, (origOffset >> 5)))
-	    || !(nde = (struct DirEntry *)afs_dir_GetBlob(tdc, us))) {
+	/* scan for the next interesting entry scan for in-use blob
+	 * otherwise up point at this blob note that ode, if non-zero,
+	 * also represents a held dir page */
+	us = BlobScan(tdc, (orginOffset >> 5));
+
+	if (us)
+	    afs_dir_GetVerifiedBlob(tdc, us, &nde);
+
+	if (us == 0 || nde == NULL) {
 	    /* failed to setup nde, return what we've got, and release ode */
 	    if (len) {
 		/* something to hand over. */
-#if	defined(AFS_HPUX_ENV) || defined(AFS_OSF_ENV)
 		sdirEntry->d_fileno =
-		    (avc->fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
+		    (avc->f.fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
 		FIXUPSTUPIDINODE(sdirEntry->d_fileno);
 		sdirEntry->d_reclen = rlen = AFS_UIO_RESID(auio);
 		sdirEntry->d_namlen = o_slen;
@@ -1043,19 +1047,10 @@ afs1_readdir(avc, auio, acred)
 			rlen -= minLen;
 		    }
 		}
-#else
-		code =
-		    afs_readdir_move(ode, avc, auio, o_slen,
-				     AFS_UIO_RESID(auio), origOffset);
-#endif /* AFS_HPUX_ENV */
 		AFS_UIO_SETRESID(auio, 0);
 	    } else {
 		/* nothin to hand over */
 	    }
-#if defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV) || defined(AFS_OSF_ENV)
-	    if (eofp)
-		*eofp = 1;
-#endif
 	    if (ode)
 		DRelease(ode, 0);
 	    goto dirend;
@@ -1063,18 +1058,13 @@ afs1_readdir(avc, auio, acred)
 	/* by here nde is set */
 
 	/* Do we have enough user space to carry out our mission? */
-#if defined(AFS_SGI_ENV)
-	n_slen = strlen(nde->name) + 1;	/* NULL terminate */
-#else
 	n_slen = strlen(nde->name);
-#endif
 	if (NDIRSIZ_LEN(n_slen) >= (AFS_UIO_RESID(auio) - len)) {
 	    /* No can do no more now; ya know... at this time */
 	    DRelease(nde, 0);	/* can't use this one. */
 	    if (len) {
-#if	defined(AFS_HPUX_ENV) || defined(AFS_OSF_ENV)
 		sdirEntry->d_fileno =
-		    (avc->fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
+		    (avc->f.fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
 		FIXUPSTUPIDINODE(sdirEntry->d_fileno);
 		sdirEntry->d_reclen = rlen = AFS_UIO_RESID(auio);
 		sdirEntry->d_namlen = o_slen;
@@ -1099,11 +1089,6 @@ afs1_readdir(avc, auio, acred)
 			rlen -= minLen;
 		    }
 		}
-#else
-		code =
-		    afs_readdir_move(ode, avc, auio, o_slen,
-				     AFS_UIO_RESID(auio), origOffset);
-#endif /* AFS_HPUX_ENV */
 		/* this next line used to be AFSVFS40 or AIX 3.1, but is really generic */
 		AFS_UIO_SETOFFSET(auio, origOffset);
 		AFS_UIO_SETRESID(auio, 0);
@@ -1124,9 +1109,8 @@ afs1_readdir(avc, auio, acred)
 	 * to set up for the next one.
 	 */
 	if (len) {
-#if	defined(AFS_HPUX_ENV) || defined(AFS_OSF_ENV)
 	    sdirEntry->d_fileno =
-		(avc->fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
+		(avc->f.fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
 	    FIXUPSTUPIDINODE(sdirEntry->d_fileno);
 	    sdirEntry->d_reclen = rlen = len;
 	    sdirEntry->d_namlen = o_slen;
@@ -1151,9 +1135,6 @@ afs1_readdir(avc, auio, acred)
 		    rlen -= minLen;
 		}
 	    }
-#else
-	    code = afs_readdir_move(ode, avc, auio, o_slen, len, origOffset);
-#endif /* AFS_HPUX_ENV */
 	}
 	len = NDIRSIZ_LEN(o_slen = n_slen);
 	if (ode)
@@ -1170,9 +1151,8 @@ afs1_readdir(avc, auio, acred)
     ReleaseReadLock(&avc->lock);
 
   done:
-#if	defined(AFS_HPUX_ENV) || defined(AFS_OSF_ENV)
     osi_FreeSmallSpace((char *)sdirEntry);
-#endif
+    AFS_DISCON_UNLOCK();
     afs_PutFakeStat(&fakestate);
     code = afs_CheckCode(code, &treq, 29);
     return code;
