@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -26,7 +26,7 @@
 #ifdef AFS_SGI62_ENV
 #include "h/hashing.h"
 #endif
-#if !defined(AFS_HPUX110_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_FBSD_ENV) && !defined(AFS_DARWIN60_ENV)
+#if !defined(AFS_HPUX110_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_FBSD_ENV) && !defined(AFS_DARWIN_ENV)
 #include <netinet/in_var.h>
 #endif
 #endif /* !UKERNEL */
@@ -69,236 +69,6 @@ afs_int32 hm_retry_RO = 0;	/* don't wait */
 afs_int32 hm_retry_RW = 0;	/* don't wait */
 afs_int32 hm_retry_int = 0;	/* don't wait */
 
-static int et2sys[512];
-
-void
-init_et_to_sys_error(void)
-{
-    memset(&et2sys, 0, sizeof(et2sys));
-    et2sys[(UAEPERM - ERROR_TABLE_BASE_uae)] = EPERM;
-    et2sys[(UAENOENT - ERROR_TABLE_BASE_uae)] = ENOENT;
-    et2sys[(UAESRCH - ERROR_TABLE_BASE_uae)] = ESRCH;
-    et2sys[(UAEINTR - ERROR_TABLE_BASE_uae)] = EINTR;
-    et2sys[(UAEIO - ERROR_TABLE_BASE_uae)] = EIO;
-    et2sys[(UAENXIO - ERROR_TABLE_BASE_uae)] = ENXIO;
-    et2sys[(UAE2BIG - ERROR_TABLE_BASE_uae)] = E2BIG;
-    et2sys[(UAENOEXEC - ERROR_TABLE_BASE_uae)] = ENOEXEC;
-    et2sys[(UAEBADF - ERROR_TABLE_BASE_uae)] = EBADF;
-    et2sys[(UAECHILD - ERROR_TABLE_BASE_uae)] = ECHILD;
-    et2sys[(UAEAGAIN - ERROR_TABLE_BASE_uae)] = EAGAIN;
-    et2sys[(UAENOMEM - ERROR_TABLE_BASE_uae)] = ENOMEM;
-    et2sys[(UAEACCES - ERROR_TABLE_BASE_uae)] = EACCES;
-    et2sys[(UAEFAULT - ERROR_TABLE_BASE_uae)] = EFAULT;
-    et2sys[(UAENOTBLK - ERROR_TABLE_BASE_uae)] = ENOTBLK;
-    et2sys[(UAEBUSY - ERROR_TABLE_BASE_uae)] = EBUSY;
-    et2sys[(UAEEXIST - ERROR_TABLE_BASE_uae)] = EEXIST;
-    et2sys[(UAEXDEV - ERROR_TABLE_BASE_uae)] = EXDEV;
-    et2sys[(UAENODEV - ERROR_TABLE_BASE_uae)] = ENODEV;
-    et2sys[(UAENOTDIR - ERROR_TABLE_BASE_uae)] = ENOTDIR;
-    et2sys[(UAEISDIR - ERROR_TABLE_BASE_uae)] = EISDIR;
-    et2sys[(UAEINVAL - ERROR_TABLE_BASE_uae)] = EINVAL;
-    et2sys[(UAENFILE - ERROR_TABLE_BASE_uae)] = ENFILE;
-    et2sys[(UAEMFILE - ERROR_TABLE_BASE_uae)] = EMFILE;
-    et2sys[(UAENOTTY - ERROR_TABLE_BASE_uae)] = ENOTTY;
-    et2sys[(UAETXTBSY - ERROR_TABLE_BASE_uae)] = ETXTBSY;
-    et2sys[(UAEFBIG - ERROR_TABLE_BASE_uae)] = EFBIG;
-    et2sys[(UAENOSPC - ERROR_TABLE_BASE_uae)] = ENOSPC;
-    et2sys[(UAESPIPE - ERROR_TABLE_BASE_uae)] = ESPIPE;
-    et2sys[(UAEROFS - ERROR_TABLE_BASE_uae)] = EROFS;
-    et2sys[(UAEMLINK - ERROR_TABLE_BASE_uae)] = EMLINK;
-    et2sys[(UAEPIPE - ERROR_TABLE_BASE_uae)] = EPIPE;
-    et2sys[(UAEDOM - ERROR_TABLE_BASE_uae)] = EDOM;
-    et2sys[(UAERANGE - ERROR_TABLE_BASE_uae)] = ERANGE;
-    et2sys[(UAEDEADLK - ERROR_TABLE_BASE_uae)] = EDEADLK;
-    et2sys[(UAENAMETOOLONG - ERROR_TABLE_BASE_uae)] = ENAMETOOLONG;
-    et2sys[(UAENOLCK - ERROR_TABLE_BASE_uae)] = ENOLCK;
-    et2sys[(UAENOSYS - ERROR_TABLE_BASE_uae)] = ENOSYS;
-    et2sys[(UAENOTEMPTY - ERROR_TABLE_BASE_uae)] = ENOTEMPTY;
-    et2sys[(UAELOOP - ERROR_TABLE_BASE_uae)] = ELOOP;
-    et2sys[(UAEWOULDBLOCK - ERROR_TABLE_BASE_uae)] = EWOULDBLOCK;
-    et2sys[(UAENOMSG - ERROR_TABLE_BASE_uae)] = ENOMSG;
-    et2sys[(UAEIDRM - ERROR_TABLE_BASE_uae)] = EIDRM;
-    et2sys[(UAECHRNG - ERROR_TABLE_BASE_uae)] = ECHRNG;
-    et2sys[(UAEL2NSYNC - ERROR_TABLE_BASE_uae)] = EL2NSYNC;
-    et2sys[(UAEL3HLT - ERROR_TABLE_BASE_uae)] = EL3HLT;
-    et2sys[(UAEL3RST - ERROR_TABLE_BASE_uae)] = EL3RST;
-    et2sys[(UAELNRNG - ERROR_TABLE_BASE_uae)] = ELNRNG;
-    et2sys[(UAEUNATCH - ERROR_TABLE_BASE_uae)] = EUNATCH;
-    et2sys[(UAENOCSI - ERROR_TABLE_BASE_uae)] = ENOCSI;
-    et2sys[(UAEL2HLT - ERROR_TABLE_BASE_uae)] = EL2HLT;
-    et2sys[(UAEBADE - ERROR_TABLE_BASE_uae)] = EBADE;
-    et2sys[(UAEBADR - ERROR_TABLE_BASE_uae)] = EBADR;
-    et2sys[(UAEXFULL - ERROR_TABLE_BASE_uae)] = EXFULL;
-    et2sys[(UAENOANO - ERROR_TABLE_BASE_uae)] = ENOANO;
-    et2sys[(UAEBADRQC - ERROR_TABLE_BASE_uae)] = EBADRQC;
-    et2sys[(UAEBADSLT - ERROR_TABLE_BASE_uae)] = EBADSLT;
-    et2sys[(UAEBFONT - ERROR_TABLE_BASE_uae)] = EBFONT;
-    et2sys[(UAENOSTR - ERROR_TABLE_BASE_uae)] = ENOSTR;
-    et2sys[(UAENODATA - ERROR_TABLE_BASE_uae)] = ENODATA;
-    et2sys[(UAETIME - ERROR_TABLE_BASE_uae)] = ETIME;
-    et2sys[(UAENOSR - ERROR_TABLE_BASE_uae)] = ENOSR;
-    et2sys[(UAENONET - ERROR_TABLE_BASE_uae)] = ENONET;
-    et2sys[(UAENOPKG - ERROR_TABLE_BASE_uae)] = ENOPKG;
-    et2sys[(UAEREMOTE - ERROR_TABLE_BASE_uae)] = EREMOTE;
-    et2sys[(UAENOLINK - ERROR_TABLE_BASE_uae)] = ENOLINK;
-    et2sys[(UAEADV - ERROR_TABLE_BASE_uae)] = EADV;
-    et2sys[(UAESRMNT - ERROR_TABLE_BASE_uae)] = ESRMNT;
-    et2sys[(UAECOMM - ERROR_TABLE_BASE_uae)] = ECOMM;
-    et2sys[(UAEPROTO - ERROR_TABLE_BASE_uae)] = EPROTO;
-    et2sys[(UAEMULTIHOP - ERROR_TABLE_BASE_uae)] = EMULTIHOP;
-    et2sys[(UAEDOTDOT - ERROR_TABLE_BASE_uae)] = EDOTDOT;
-    et2sys[(UAEBADMSG - ERROR_TABLE_BASE_uae)] = EBADMSG;
-    et2sys[(UAEOVERFLOW - ERROR_TABLE_BASE_uae)] = EOVERFLOW;
-    et2sys[(UAENOTUNIQ - ERROR_TABLE_BASE_uae)] = ENOTUNIQ;
-    et2sys[(UAEBADFD - ERROR_TABLE_BASE_uae)] = EBADFD;
-    et2sys[(UAEREMCHG - ERROR_TABLE_BASE_uae)] = EREMCHG;
-    et2sys[(UAELIBACC - ERROR_TABLE_BASE_uae)] = ELIBACC;
-    et2sys[(UAELIBBAD - ERROR_TABLE_BASE_uae)] = ELIBBAD;
-    et2sys[(UAELIBSCN - ERROR_TABLE_BASE_uae)] = ELIBSCN;
-    et2sys[(UAELIBMAX - ERROR_TABLE_BASE_uae)] = ELIBMAX;
-    et2sys[(UAELIBEXEC - ERROR_TABLE_BASE_uae)] = ELIBEXEC;
-    et2sys[(UAEILSEQ - ERROR_TABLE_BASE_uae)] = EILSEQ;
-    et2sys[(UAERESTART - ERROR_TABLE_BASE_uae)] = ERESTART;
-    et2sys[(UAESTRPIPE - ERROR_TABLE_BASE_uae)] = ESTRPIPE;
-    et2sys[(UAEUSERS - ERROR_TABLE_BASE_uae)] = EUSERS;
-    et2sys[(UAENOTSOCK - ERROR_TABLE_BASE_uae)] = ENOTSOCK;
-    et2sys[(UAEDESTADDRREQ - ERROR_TABLE_BASE_uae)] = EDESTADDRREQ;
-    et2sys[(UAEMSGSIZE - ERROR_TABLE_BASE_uae)] = EMSGSIZE;
-    et2sys[(UAEPROTOTYPE - ERROR_TABLE_BASE_uae)] = EPROTOTYPE;
-    et2sys[(UAENOPROTOOPT - ERROR_TABLE_BASE_uae)] = ENOPROTOOPT;
-    et2sys[(UAEPROTONOSUPPORT - ERROR_TABLE_BASE_uae)] = EPROTONOSUPPORT;
-    et2sys[(UAESOCKTNOSUPPORT - ERROR_TABLE_BASE_uae)] = ESOCKTNOSUPPORT;
-    et2sys[(UAEOPNOTSUPP - ERROR_TABLE_BASE_uae)] = EOPNOTSUPP;
-    et2sys[(UAEPFNOSUPPORT - ERROR_TABLE_BASE_uae)] = EPFNOSUPPORT;
-    et2sys[(UAEAFNOSUPPORT - ERROR_TABLE_BASE_uae)] = EAFNOSUPPORT;
-    et2sys[(UAEADDRINUSE - ERROR_TABLE_BASE_uae)] = EADDRINUSE;
-    et2sys[(UAEADDRNOTAVAIL - ERROR_TABLE_BASE_uae)] = EADDRNOTAVAIL;
-    et2sys[(UAENETDOWN - ERROR_TABLE_BASE_uae)] = ENETDOWN;
-    et2sys[(UAENETUNREACH - ERROR_TABLE_BASE_uae)] = ENETUNREACH;
-    et2sys[(UAENETRESET - ERROR_TABLE_BASE_uae)] = ENETRESET;
-    et2sys[(UAECONNABORTED - ERROR_TABLE_BASE_uae)] = ECONNABORTED;
-    et2sys[(UAECONNRESET - ERROR_TABLE_BASE_uae)] = ECONNRESET;
-    et2sys[(UAENOBUFS - ERROR_TABLE_BASE_uae)] = ENOBUFS;
-    et2sys[(UAEISCONN - ERROR_TABLE_BASE_uae)] = EISCONN;
-    et2sys[(UAENOTCONN - ERROR_TABLE_BASE_uae)] = ENOTCONN;
-    et2sys[(UAESHUTDOWN - ERROR_TABLE_BASE_uae)] = ESHUTDOWN;
-    et2sys[(UAETOOMANYREFS - ERROR_TABLE_BASE_uae)] = ETOOMANYREFS;
-    et2sys[(UAETIMEDOUT - ERROR_TABLE_BASE_uae)] = ETIMEDOUT;
-    et2sys[(UAECONNREFUSED - ERROR_TABLE_BASE_uae)] = ECONNREFUSED;
-    et2sys[(UAEHOSTDOWN - ERROR_TABLE_BASE_uae)] = EHOSTDOWN;
-    et2sys[(UAEHOSTUNREACH - ERROR_TABLE_BASE_uae)] = EHOSTUNREACH;
-    et2sys[(UAEALREADY - ERROR_TABLE_BASE_uae)] = EALREADY;
-    et2sys[(UAEINPROGRESS - ERROR_TABLE_BASE_uae)] = EINPROGRESS;
-    et2sys[(UAESTALE - ERROR_TABLE_BASE_uae)] = ESTALE;
-    et2sys[(UAEUCLEAN - ERROR_TABLE_BASE_uae)] = EUCLEAN;
-    et2sys[(UAENOTNAM - ERROR_TABLE_BASE_uae)] = ENOTNAM;
-    et2sys[(UAENAVAIL - ERROR_TABLE_BASE_uae)] = ENAVAIL;
-    et2sys[(UAEISNAM - ERROR_TABLE_BASE_uae)] = EISNAM;
-    et2sys[(UAEREMOTEIO - ERROR_TABLE_BASE_uae)] = EREMOTEIO;
-    et2sys[(UAEDQUOT - ERROR_TABLE_BASE_uae)] = EDQUOT;
-    et2sys[(UAENOMEDIUM - ERROR_TABLE_BASE_uae)] = ENOMEDIUM;
-    et2sys[(UAEMEDIUMTYPE - ERROR_TABLE_BASE_uae)] = EMEDIUMTYPE;
-}
-
-static afs_int32
-et_to_sys_error(afs_int32 in)
-{
-    if (in < ERROR_TABLE_BASE_uae || in >= ERROR_TABLE_BASE_uae + 512)
-	return in;
-    if (et2sys[in - ERROR_TABLE_BASE_uae] != 0)
-	return et2sys[in - ERROR_TABLE_BASE_uae];
-    return in;
-}
-
-void
-afs_CopyError(register struct vrequest *afrom, register struct vrequest *ato)
-{
-    int i = 0;
-    AFS_STATCNT(afs_CopyError);
-    if (!afrom->initd)
-	return;
-    afs_FinalizeReq(ato);
-    while (i < MAXHOSTS) {
-	ato->skipserver[i] = afrom->skipserver[i];
-	i++;
-    }
-    if (afrom->tokenError)
-	ato->tokenError = afrom->tokenError;
-    if (afrom->idleError)
-	ato->idleError = afrom->idleError;
-    if (afrom->accessError)
-	ato->accessError = 1;
-    if (afrom->volumeError)
-	ato->volumeError = 1;
-    if (afrom->networkError)
-	ato->networkError = 1;
-    if (afrom->permWriteError)
-	ato->permWriteError = 1;
-
-}
-
-void
-afs_FinalizeReq(register struct vrequest *areq)
-{
-    int i = 0;
-    AFS_STATCNT(afs_FinalizeReq);
-    if (areq->initd)
-	return;
-    while (i < MAXHOSTS) {
-	areq->skipserver[i] = 0;
-	i++;
-    }
-    areq->busyCount = 0;
-    areq->idleError = 0;
-    areq->tokenError = 0;
-    areq->accessError = 0;
-    areq->volumeError = 0;
-    areq->networkError = 0;
-    areq->permWriteError = 0;
-    areq->initd = 1;
-
-}
-
-int
-afs_CheckCode(afs_int32 acode, struct vrequest *areq, int where)
-{
-    AFS_STATCNT(afs_CheckCode);
-    if (acode) {
-	afs_Trace2(afs_iclSetp, CM_TRACE_CHECKCODE, ICL_TYPE_INT32, acode,
-		   ICL_TYPE_INT32, where);
-    }
-    if ((acode & ~0xff) == ERROR_TABLE_BASE_uae)
-	acode = et_to_sys_error(acode);
-    if (!areq || !areq->initd)
-	return acode;
-    if (areq->networkError)
-	return ETIMEDOUT;
-    if (acode == 0)
-	return 0;
-    if (areq->accessError)
-	return EACCES;
-    if (areq->volumeError == VOLMISSING)
-	return ENODEV;
-    if (areq->volumeError == VOLBUSY)
-	return EWOULDBLOCK;
-    if (acode == VNOVNODE)
-	return ENOENT;
-    if (acode == VDISKFULL)
-	return ENOSPC;
-    if (acode == VOVERQUOTA)
-	return
-#ifdef EDQUOT
-	    EDQUOT
-#else
-	    ENOSPC
-#endif
-	    ;
-
-    return acode;
-
-}				/*afs_CheckCode */
-
-
 #define	VSleep(at)	afs_osi_Wait((at)*1000, 0, 0)
 
 
@@ -307,7 +77,7 @@ int lastcode;
  * 0   if the vldb record for a specific volume is different from what
  *     we have cached -- perhaps the volume has moved.
  * 1   if the vldb record is the same
- * 2   if we can't tell if it's the same or not. 
+ * 2   if we can't tell if it's the same or not.
  *
  * If 0, the caller will probably start over at the beginning of our
  * list of servers for this volume and try to find one that is up.  If
@@ -333,6 +103,7 @@ VLDB_Same(struct VenusFid *afid, struct vrequest *areq)
     char *bp, tbuf[CVBS];	/* biggest volume id is 2^32, ~ 4*10^9 */
     unsigned int changed;
     struct server *(oldhosts[NMAXNSERVERS]);
+    struct rx_connection *rxconn;
 
     AFS_STATCNT(CheckVLDB);
     afs_FinalizeReq(areq);
@@ -346,34 +117,34 @@ VLDB_Same(struct VenusFid *afid, struct vrequest *areq)
 	VSleep(2);		/* Better safe than sorry. */
 	tconn =
 	    afs_ConnByMHosts(tcell->cellHosts, tcell->vlport, tcell->cellNum,
-			     &treq, SHARED_LOCK);
+			     &treq, SHARED_LOCK, &rxconn);
 	if (tconn) {
 	    if (tconn->srvr->server->flags & SNO_LHOSTS) {
 		type = 0;
 		RX_AFS_GUNLOCK();
-		i = VL_GetEntryByNameO(tconn->id, bp, &v->tve);
+		i = VL_GetEntryByNameO(rxconn, bp, &v->tve);
 		RX_AFS_GLOCK();
 	    } else if (tconn->srvr->server->flags & SYES_LHOSTS) {
 		type = 1;
 		RX_AFS_GUNLOCK();
-		i = VL_GetEntryByNameN(tconn->id, bp, &v->ntve);
+		i = VL_GetEntryByNameN(rxconn, bp, &v->ntve);
 		RX_AFS_GLOCK();
 	    } else {
 		type = 2;
 		RX_AFS_GUNLOCK();
-		i = VL_GetEntryByNameU(tconn->id, bp, &v->utve);
+		i = VL_GetEntryByNameU(rxconn, bp, &v->utve);
 		RX_AFS_GLOCK();
 		if (!(tconn->srvr->server->flags & SVLSRV_UUID)) {
 		    if (i == RXGEN_OPCODE) {
 			type = 1;
 			RX_AFS_GUNLOCK();
-			i = VL_GetEntryByNameN(tconn->id, bp, &v->ntve);
+			i = VL_GetEntryByNameN(rxconn, bp, &v->ntve);
 			RX_AFS_GLOCK();
 			if (i == RXGEN_OPCODE) {
 			    type = 0;
 			    tconn->srvr->server->flags |= SNO_LHOSTS;
 			    RX_AFS_GUNLOCK();
-			    i = VL_GetEntryByNameO(tconn->id, bp, &v->tve);
+			    i = VL_GetEntryByNameO(rxconn, bp, &v->tve);
 			    RX_AFS_GLOCK();
 			} else if (!i)
 			    tconn->srvr->server->flags |= SYES_LHOSTS;
@@ -384,7 +155,7 @@ VLDB_Same(struct VenusFid *afid, struct vrequest *areq)
 	    }
 	} else
 	    i = -1;
-    } while (afs_Analyze(tconn, i, NULL, &treq, -1,	/* no op code for this */
+    } while (afs_Analyze(tconn, rxconn, i, NULL, &treq, -1,	/* no op code for this */
 			 SHARED_LOCK, tcell));
 
     afs_PutCell(tcell, READ_LOCK);
@@ -449,8 +220,8 @@ VLDB_Same(struct VenusFid *afid, struct vrequest *areq)
  *	areq  : The request record associated with this operation.
  *	afid  : The FID of the file involved in the action.  This argument
  *		may be null if none was involved.
- *      tsp   : pointer to a server struct for the server we wish to 
- *              blacklist. 
+ *      tsp   : pointer to a server struct for the server we wish to
+ *              blacklist.
  *
  * Returns:
  *	Non-zero value if further servers are available to try,
@@ -464,11 +235,11 @@ VLDB_Same(struct VenusFid *afid, struct vrequest *areq)
  *	As advertised.
  *
  * NOTE:
- *	The afs_Conn* routines use the list of invalidated servers to 
+ *	The afs_Conn* routines use the list of invalidated servers to
  *      avoid reusing a server marked as invalid for this request.
  *------------------------------------------------------------------------*/
-static afs_int32 
-afs_BlackListOnce(struct vrequest *areq, struct VenusFid *afid, 
+static afs_int32
+afs_BlackListOnce(struct vrequest *areq, struct VenusFid *afid,
 		  struct server *tsp)
 {
     struct volume *tvp;
@@ -478,18 +249,18 @@ afs_BlackListOnce(struct vrequest *areq, struct VenusFid *afid,
     if (afid) {
 	tvp = afs_FindVolume(afid, READ_LOCK);
 	if (tvp) {
-	    for (i = 0; i < MAXHOSTS; i++) {
+	    for (i = 0; i < AFS_MAXHOSTS; i++) {
 		if (tvp->serverHost[i] == tsp) {
 		    areq->skipserver[i] = 1;
 		}
 		if (tvp->serverHost[i] &&
-		    (tvp->serverHost[i]->addr->sa_flags & 
+		    (tvp->serverHost[i]->addr->sa_flags &
 		      SRVR_ISDOWN)) {
 		    areq->skipserver[i] = 1;
 		}
 	    }
 	    afs_PutVolume(tvp, READ_LOCK);
-	    for (i = 0; i < MAXHOSTS; i++) {
+	    for (i = 0; i < AFS_MAXHOSTS; i++) {
 	        if (tvp->serverHost[i] && areq->skipserver[i] == 0) {
 		    serversleft = 1;
 		    break;
@@ -498,7 +269,7 @@ afs_BlackListOnce(struct vrequest *areq, struct VenusFid *afid,
 	    return serversleft;
 	}
     }
-    return 1;
+    return serversleft;
 }
 
 
@@ -535,8 +306,9 @@ afs_BlackListOnce(struct vrequest *areq, struct VenusFid *afid,
  *	if this is a temporary or permanent error.
  *------------------------------------------------------------------------*/
 int
-afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
-	    struct VenusFid *afid, register struct vrequest *areq, int op,
+afs_Analyze(struct afs_conn *aconn, struct rx_connection *rxconn,
+	    afs_int32 acode,
+	    struct VenusFid *afid, struct vrequest *areq, int op,
 	    afs_int32 locktype, struct cell *cellp)
 {
     afs_int32 i;
@@ -547,6 +319,21 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
     afs_int32 serversleft = 1;
     struct afs_stats_RPCErrors *aerrP;
     afs_int32 markeddown;
+    afs_uint32 address;
+
+    if (AFS_IS_DISCONNECTED && !AFS_IN_SYNC) {
+	/* On reconnection, act as connected. XXX: for now.... */
+        /* SXW - This may get very tired after a while. We should try and
+	 *       intercept all RPCs before they get here ... */
+	/*printf("afs_Analyze: disconnected\n");*/
+	afs_FinalizeReq(areq);
+	if (aconn) {
+	    /* SXW - I suspect that this will _never_ happen - we shouldn't
+	     *       get a connection because we're disconnected !!!*/
+	    afs_PutConn(aconn, rxconn, locktype);
+	}
+	return 0;
+    }
 
     AFS_STATCNT(afs_Analyze);
     afs_Trace4(afs_iclSetp, CM_TRACE_ANALYZE, ICL_TYPE_INT32, op,
@@ -570,7 +357,7 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
 			   && tvp->serverHost[0]->cell) ? tvp->serverHost[0]->
 			  cell->cellName : ""));
 
-	    for (i = 0; i < MAXHOSTS; i++) {
+	    for (i = 0; i < AFS_MAXHOSTS; i++) {
 		if (tvp->status[i] != not_busy && tvp->status[i] != offline) {
 		    tvp->status[i] = not_busy;
 		}
@@ -629,7 +416,10 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
 		}
 	    } /* if (hm_retry_int ... */
 	    else {
-		areq->networkError = 1;
+		if (acode == RX_MSGSIZE)
+		    shouldRetry = 1;
+		else
+		    areq->networkError = 1;
 	    }
 	}
 	return shouldRetry;
@@ -638,6 +428,7 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
     /* Find server associated with this connection. */
     sa = aconn->srvr;
     tsp = sa->server;
+    address = ntohl(sa->sa_ip);
 
     /* Before we do anything with acode, make sure we translate it back to
      * a system error */
@@ -649,7 +440,7 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
 	if (areq->volumeError) {
 	    tvp = afs_FindVolume(afid, READ_LOCK);
 	    if (tvp) {
-		for (i = 0; i < MAXHOSTS; i++) {
+		for (i = 0; i < AFS_MAXHOSTS; i++) {
 		    if (tvp->serverHost[i] == tsp) {
 			tvp->status[i] = not_busy;
 		    }
@@ -658,18 +449,22 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
 	    }
 	}
 
-	afs_PutConn(aconn, locktype);
+	afs_PutConn(aconn, rxconn, locktype);
 	return 0;
     }
 
     /* If network troubles, mark server as having bogued out again. */
-    /* VRESTARTING is < 0 because of backward compatibility issues 
+    /* VRESTARTING is < 0 because of backward compatibility issues
      * with 3.4 file servers and older cache managers */
 #ifdef AFS_64BIT_CLIENT
     if (acode == -455)
 	acode = 455;
 #endif /* AFS_64BIT_CLIENT */
     if ((acode < 0) && (acode != VRESTARTING)) {
+	if (acode == RX_MSGSIZE) {
+	    shouldRetry = 1;
+	    goto out;
+	}
 	if (acode == RX_CALL_TIMEOUT) {
 	    serversleft = afs_BlackListOnce(areq, afid, tsp);
 	    if (afid)
@@ -685,13 +480,13 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
 	    if (tvp)
 		afs_PutVolume(tvp, READ_LOCK);
 	    /* By doing this, we avoid ever marking a server down
-	     * in an idle timeout case. That's because the server is 
+	     * in an idle timeout case. That's because the server is
 	     * still responding and may only be letting a single vnode
 	     * time out. We otherwise risk having the server continually
-	     * be marked down, then up, then down again... 
+	     * be marked down, then up, then down again...
 	     */
 	    goto out;
-	} 
+	}
 	markeddown = afs_ServerDown(sa);
 	ForceNewConnections(sa); /**multi homed clients lock:afs_xsrvAddr? */
 	if (aerrP)
@@ -715,7 +510,7 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
 
 	tvp = afs_FindVolume(afid, READ_LOCK);
 	if (tvp) {
-	    for (i = 0; i < MAXHOSTS; i++) {
+	    for (i = 0; i < AFS_MAXHOSTS; i++) {
 		if (tvp->serverHost[i] == tsp) {
 		    tvp->status[i] = rdwr_busy;	/* can't tell which yet */
 		    /* to tell which, have to look at the op code. */
@@ -723,8 +518,10 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
 	    }
 	    afs_PutVolume(tvp, READ_LOCK);
 	} else {
-	    afs_warnuser("afs: Waiting for busy volume %u in cell %s\n",
-			 (afid ? afid->Fid.Volume : 0), tsp->cell->cellName);
+	    afs_warnuser("afs: Waiting for busy volume %u in cell %s (server %d.%d.%d.%d)\n",
+			 (afid ? afid->Fid.Volume : 0), tsp->cell->cellName,
+			 (address >> 24), (address >> 16) & 0xff,
+			 (address >> 8) & 0xff, (address) & 0xff);
 	    VSleep(afs_BusyWaitPeriod);	/* poll periodically */
 	}
 	shouldRetry = 1;
@@ -747,25 +544,33 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
 	    } else if (acode == RXKADEXPIRED) {
 		aconn->forceConnectFS = 0;	/* don't check until new tokens set */
 		aconn->user->states |= UTokensBad;
+		afs_NotifyUser(tu, UTokensDropped);
 		afs_warnuser
-		    ("afs: Tokens for user of AFS id %d for cell %s have expired\n",
-		     tu->vid, aconn->srvr->server->cell->cellName);
+		    ("afs: Tokens for user of AFS id %d for cell %s have expired (server %d.%d.%d.%d)\n",
+		     tu->vid, aconn->srvr->server->cell->cellName,
+		     (address >> 24), (address >> 16) & 0xff,
+		     (address >> 8) & 0xff, (address) & 0xff);
 	    } else {
 		serversleft = afs_BlackListOnce(areq, afid, tsp);
 		areq->tokenError++;
 
 		if (serversleft) {
 		    afs_warnuser
-			("afs: Tokens for user of AFS id %d for cell %s: rxkad error=%d\n",
-			 tu->vid, aconn->srvr->server->cell->cellName, acode);
+			("afs: Tokens for user of AFS id %d for cell %s: rxkad error=%d (server %d.%d.%d.%d)\n",
+			 tu->vid, aconn->srvr->server->cell->cellName, acode,
+			 (address >> 24), (address >> 16) & 0xff,
+			 (address >> 8) & 0xff, (address) & 0xff);
 		    shouldRetry = 1;
 		} else {
 		    areq->tokenError = 0;
 		    aconn->forceConnectFS = 0;	/* don't check until new tokens set */
 		    aconn->user->states |= UTokensBad;
+		    afs_NotifyUser(tu, UTokensDropped);
 		    afs_warnuser
-			("afs: Tokens for user of AFS id %d for cell %s are discarded (rxkad error=%d)\n",
-			 tu->vid, aconn->srvr->server->cell->cellName, acode);
+			("afs: Tokens for user of AFS id %d for cell %s are discarded (rxkad error=%d, server %d.%d.%d.%d)\n",
+			 tu->vid, aconn->srvr->server->cell->cellName, acode,
+			 (address >> 24), (address >> 16) & 0xff,
+			 (address >> 8) & 0xff, (address) & 0xff);
 		}
 	    }
 	    afs_PutUser(tu, READ_LOCK);
@@ -776,15 +581,22 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
 	    } else if (acode == RXKADEXPIRED) {
 		aconn->forceConnectFS = 0;	/* don't check until new tokens set */
 		aconn->user->states |= UTokensBad;
+		afs_NotifyUser(tu, UTokensDropped);
 		afs_warnuser
-		    ("afs: Tokens for user %d for cell %s have expired\n",
-		     areq->uid, aconn->srvr->server->cell->cellName);
+		    ("afs: Tokens for user %d for cell %s have expired (server %d.%d.%d.%d)\n",
+		     areq->uid, aconn->srvr->server->cell->cellName,
+		     (address >> 24), (address >> 16) & 0xff,
+		     (address >> 8) & 0xff, (address) & 0xff);
 	    } else {
 		aconn->forceConnectFS = 0;	/* don't check until new tokens set */
 		aconn->user->states |= UTokensBad;
+		afs_NotifyUser(tu, UTokensDropped);
 		afs_warnuser
-		    ("afs: Tokens for user %d for cell %s are discarded (rxkad error = %d)\n",
-		     areq->uid, aconn->srvr->server->cell->cellName, acode);
+		    ("afs: Tokens for user %d for cell %s are discarded (rxkad error = %d, server %d.%d.%d.%d)\n",
+		     areq->uid, aconn->srvr->server->cell->cellName, acode,
+		     (address >> 24), (address >> 16) & 0xff,
+		     (address >> 8) & 0xff, (address) & 0xff);
+
 	    }
 	}
 	shouldRetry = 1;	/* Try again (as root). */
@@ -821,7 +633,7 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
 	    same = VLDB_Same(afid, areq);
 	    tvp = afs_FindVolume(afid, READ_LOCK);
 	    if (tvp) {
-		for (i = 0; i < MAXHOSTS && tvp->serverHost[i]; i++) {
+		for (i = 0; i < AFS_MAXHOSTS && tvp->serverHost[i]; i++) {
 		    if (tvp->serverHost[i] == tsp) {
 			if (tvp->status[i] == end_not_busy)
 			    tvp->status[i] = offline;
@@ -844,6 +656,11 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
 	    areq->permWriteError = 1;
 	shouldRetry = 0;	/* Other random Vice error. */
     } else if (acode == RX_MSGSIZE) {	/* same meaning as EMSGSIZE... */
+	afs_warnuser
+	    ("afs: Path MTU may have been exceeded, retrying (server %d.%d.%d.%d)\n",
+	     (address >> 24), (address >> 16) & 0xff,
+	     (address >> 8) & 0xff, (address) & 0xff);
+
 	VSleep(1);		/* Just a hack for desperate times. */
 	if (aerrP)
 	    (aerrP->err_Other)++;
@@ -864,6 +681,6 @@ afs_Analyze(register struct afs_conn *aconn, afs_int32 acode,
     }
 out:
     /* now unlock the connection and return */
-    afs_PutConn(aconn, locktype);
+    afs_PutConn(aconn, rxconn, locktype);
     return (shouldRetry);
 }				/*afs_Analyze */
