@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -31,13 +31,14 @@
 #ifdef AFS_SGI62_ENV
 #include "h/hashing.h"
 #endif
-#if !defined(AFS_HPUX110_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_DARWIN60_ENV)
+#if !defined(AFS_HPUX110_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_DARWIN_ENV)
 #include <netinet/in_var.h>
 #endif /* ! AFS_HPUX110_ENV */
 #endif /* !defined(UKERNEL) */
 
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"	/* afs statistics */
+#include "afs/afs_dynroot.h"
 
 #if	defined(AFS_SUN56_ENV)
 #include <inet/led.h>
@@ -55,11 +56,11 @@
 #endif /* vlserver error base define */
 
 /* Exported variables */
-ino_t volumeInode;		/*Inode for VolumeItems file */
-afs_rwlock_t afs_xvolume;	/* allocation lock for volumes */
+afs_dcache_id_t volumeInode;	/* Inode for VolumeItems file */
+afs_rwlock_t afs_xvolume;	/** allocation lock for volumes */
 struct volume *afs_freeVolList;
 struct volume *afs_volumes[NVOLS];
-afs_int32 afs_volCounter = 1;	/* for allocating volume indices */
+afs_int32 afs_volCounter = 1;	/** for allocating volume indices */
 afs_int32 fvTable[NFENTRIES];
 
 /* Forward declarations */
@@ -71,12 +72,17 @@ static int inVolList(struct VenusFid *fid, afs_int32 nvols, afs_int32 * vID,
 		     afs_int32 * cID);
 
 
-/* Convert a volume name to a #; return 0 if can't parse as a number */
+
+/**
+ * Convert a volume name to a number;
+ * @param aname Volume name.
+ * @return return 0 if can't parse as a number.
+ */
 static int
-afs_vtoi(register char *aname)
+afs_vtoi(char *aname)
 {
-    register afs_int32 temp;
-    register int tc;
+    afs_int32 temp;
+    int tc;
     temp = 0;
     AFS_STATCNT(afs_vtoi);
     while ((tc = *aname++)) {
@@ -90,7 +96,7 @@ afs_vtoi(register char *aname)
 }				/*afs_vtoi */
 
 
-/*
+/**
  * All of the vol cache routines must be called with the afs_xvolume
  * lock held in exclusive mode, since they use static variables.
  * In addition, we don't want two people adding the same volume
@@ -100,14 +106,16 @@ afs_vtoi(register char *aname)
 static struct fvolume staticFVolume;
 afs_int32 afs_FVIndex = -1;
 
-/* UFS specific version of afs_GetVolSlot */
-
+/**
+ * UFS specific version of afs_GetVolSlot
+ * @return
+ */
 struct volume *
 afs_UFSGetVolSlot(void)
 {
-    register struct volume *tv = NULL, **lv;
+    struct volume *tv = NULL, **lv;
     struct osi_file *tfile;
-    register afs_int32 i = -1, code;
+    afs_int32 i = -1, code;
     afs_int32 bestTime;
     struct volume *bestVp, *oldLp = NULL, **bestLp = NULL;
     char *oldname = NULL;
@@ -157,7 +165,7 @@ afs_UFSGetVolSlot(void)
 	     * next chain
 	     */
 	    if (afs_FVIndex != tv->vtix) {
-		tfile = osi_UFSOpen(volumeInode);
+		tfile = osi_UFSOpen(&volumeInode);
 		code =
 		    afs_osi_Read(tfile, sizeof(struct fvolume) * tv->vtix,
 				 &staticFVolume, sizeof(struct fvolume));
@@ -177,7 +185,7 @@ afs_UFSGetVolSlot(void)
 	staticFVolume.dotdot = tv->dotdot;
 	staticFVolume.rootVnode = tv->rootVnode;
 	staticFVolume.rootUnique = tv->rootUnique;
-	tfile = osi_UFSOpen(volumeInode);
+	tfile = osi_UFSOpen(&volumeInode);
 	code =
 	    afs_osi_Write(tfile, sizeof(struct fvolume) * afs_FVIndex,
 			  &staticFVolume, sizeof(struct fvolume));
@@ -223,10 +231,15 @@ afs_UFSGetVolSlot(void)
 }				/*afs_UFSGetVolSlot */
 
 
+/**
+ *   Get an available volume list slot. If the list does not exist,
+ * create one containing a single element.
+ * @return
+ */
 struct volume *
 afs_MemGetVolSlot(void)
 {
-    register struct volume *tv;
+    struct volume *tv;
 
     AFS_STATCNT(afs_MemGetVolSlot);
     if (!afs_freeVolList) {
@@ -243,9 +256,10 @@ afs_MemGetVolSlot(void)
 
 }				/*afs_MemGetVolSlot */
 
-/* afs_ResetVolumes()
- * Reset volume inforamation for all volume structs that
+/**
+ *   Reset volume information for all volume structs that
  * point to a speicific server.
+ * @param srvp
  */
 void
 afs_ResetVolumes(struct server *srvp)
@@ -256,7 +270,7 @@ afs_ResetVolumes(struct server *srvp)
     /* Find any volumes residing on this server and flush their state */
     for (j = 0; j < NVOLS; j++) {
 	for (vp = afs_volumes[j]; vp; vp = vp->next) {
-	    for (k = 0; k < MAXHOSTS; k++) {
+	    for (k = 0; k < AFS_MAXHOSTS; k++) {
 		if (!srvp || (vp->serverHost[k] == srvp)) {
 		    vp->serverHost[k] = 0;
 		    afs_ResetVolumeInfo(vp);
@@ -268,7 +282,10 @@ afs_ResetVolumes(struct server *srvp)
 }
 
 
-/* reset volume name to volume id mapping  cache */
+/**
+ *   Reset volume name to volume id mapping cache.
+ * @param flags
+ */
 void
 afs_CheckVolumeNames(int flags)
 {
@@ -314,8 +331,9 @@ afs_CheckVolumeNames(int flags)
 		    continue;
 		}
 	    }
+	    /* ??? */
 	    if (flags & (AFS_VOLCHECK_BUSY | AFS_VOLCHECK_FORCE)) {
-		for (j = 0; j < MAXHOSTS; j++)
+		for (j = 0; j < AFS_MAXHOSTS; j++)
 		    tv->status[j] = not_busy;
 	    }
 
@@ -339,25 +357,25 @@ loop:
 		if ((flags & (AFS_VOLCHECK_FORCE | AFS_VOLCHECK_MTPTS))
 		    || (tvc->mvid
 			&& inVolList(tvc->mvid, nvols, volumeID, cellID)))
-		    tvc->states &= ~CMValid;
+		    tvc->f.states &= ~CMValid;
 
 		/* If the volume that this file belongs to was reset earlier,
 		 * then we should remove its callback.
 		 * Again, if forced, always do it.
 		 */
-		if ((tvc->states & CRO)
-		    && (inVolList(&tvc->fid, nvols, volumeID, cellID)
+		if ((tvc->f.states & CRO)
+		    && (inVolList(&tvc->f.fid, nvols, volumeID, cellID)
 			|| (flags & AFS_VOLCHECK_FORCE))) {
 
-                    if (tvc->states & CVInit) {
+                    if (tvc->f.states & CVInit) {
                         ReleaseReadLock(&afs_xvcache);
-			afs_osi_Sleep(&tvc->states);
+			afs_osi_Sleep(&tvc->f.states);
                         goto loop;
                     }
 #ifdef AFS_DARWIN80_ENV
-                    if (tvc->states & CDeadVnode) {
+                    if (tvc->f.states & CDeadVnode) {
                         ReleaseReadLock(&afs_xvcache);
-			afs_osi_Sleep(&tvc->states);
+			afs_osi_Sleep(&tvc->f.states);
                         goto loop;
                     }
 		    tvp = AFSTOV(tvc);
@@ -378,9 +396,9 @@ loop:
 		    ObtainWriteLock(&afs_xcbhash, 485);
 		    /* LOCKXXX: We aren't holding tvc write lock? */
 		    afs_DequeueCallback(tvc);
-		    tvc->states &= ~CStatd;
+		    tvc->f.states &= ~CStatd;
 		    ReleaseWriteLock(&afs_xcbhash);
-		    if (tvc->fid.Fid.Vnode & 1 || (vType(tvc) == VDIR))
+		    if (tvc->f.fid.Fid.Vnode & 1 || (vType(tvc) == VDIR))
 			osi_dnlc_purgedp(tvc);
 
 #ifdef AFS_DARWIN80_ENV
@@ -407,6 +425,14 @@ loop:
 }				/*afs_CheckVolumeNames */
 
 
+/**
+ * Check if volume is in the specified list.
+ * @param fid File FID.
+ * @param nvols Nomber of volumes???
+ * @param vID Array of volume IDs.
+ * @param cID Array of cache IDs.
+ * @return 1 - true, 0 - false.
+ */
 static int
 inVolList(struct VenusFid *fid, afs_int32 nvols, afs_int32 * vID,
 	  afs_int32 * cID)
@@ -428,10 +454,12 @@ inVolList(struct VenusFid *fid, afs_int32 nvols, afs_int32 * vID,
 /* afs_PutVolume is now a macro in afs.h */
 
 
-/* afs_FindVolume()
- *  return volume struct if we have it cached and it's up-to-date
- *
- *  Environment:  Must be called with afs_xvolume unlocked.
+/**
+ *    Return volume struct if we have it cached and it's up-to-date.
+ *  Environment: Must be called with afs_xvolume unlocked.
+ *  @param afid Volume FID.
+ *  @param locktype
+ *  @return Volume or NULL if no result.
  */
 struct volume *
 afs_FindVolume(struct VenusFid *afid, afs_int32 locktype)
@@ -457,9 +485,13 @@ afs_FindVolume(struct VenusFid *afid, afs_int32 locktype)
 
 
 
-/*
- * Note that areq may be null, in which case we don't bother to set any
+/**
+ *   Note that areq may be null, in which case we don't bother to set any
  * request status information.
+ * @param afid Volume FID.
+ * @param areq Request type.
+ * @param locktype Lock to be used.
+ * @return Volume or NULL if no result.
  */
 struct volume *
 afs_GetVolume(struct VenusFid *afid, struct vrequest *areq,
@@ -471,6 +503,7 @@ afs_GetVolume(struct VenusFid *afid, struct vrequest *areq,
 
     tv = afs_FindVolume(afid, locktype);
     if (!tv) {
+	/* Do a dynroot check and add dynroot volume if found. */
 	if (afs_IsDynrootAnyFid(afid)) {
 	    tv = afs_NewDynrootVolume(afid);
 	} else {
@@ -483,6 +516,17 @@ afs_GetVolume(struct VenusFid *afid, struct vrequest *areq,
 
 
 
+/**
+ *
+ * @param volid Volume ID. If it's 0, get it from the name.
+ * @param aname Volume name.
+ * @param ve Volume entry.
+ * @param tcell The cell containing this volume.
+ * @param agood
+ * @param type Type of volume.
+ * @param areq Request.
+ * @return Volume or NULL if failure.
+ */
 static struct volume *
 afs_SetupVolume(afs_int32 volid, char *aname, void *ve, struct cell *tcell,
 		afs_int32 agood, afs_int32 type, struct vrequest *areq)
@@ -516,8 +560,8 @@ afs_SetupVolume(afs_int32 volid, char *aname, void *ve, struct cell *tcell,
 	    } else {
 		volid = ove->volumeId[whichType];
 	    }
-	}
-    }
+	} /* end of if (volid == 0) */
+    } /* end of if (!volid) */
 
 
     ObtainWriteLock(&afs_xvolume, 108);
@@ -535,16 +579,16 @@ afs_SetupVolume(afs_int32 volid, char *aname, void *ve, struct cell *tcell,
 	    ReleaseWriteLock(&afs_xvolume);
 	    return NULL;
 	}
-	memset((char *)tv, 0, sizeof(struct volume));
+	memset(tv, 0, sizeof(struct volume));
 	tv->cell = tcell->cellNum;
-	RWLOCK_INIT(&tv->lock, "volume lock");
+	AFS_RWLOCK_INIT(&tv->lock, "volume lock");
 	tv->next = afs_volumes[i];	/* thread into list */
 	afs_volumes[i] = tv;
 	tv->volume = volid;
 	for (j = fvTable[FVHash(tv->cell, volid)]; j != 0; j = tf->next) {
 	    if (afs_FVIndex != j) {
 		struct osi_file *tfile;
-		tfile = osi_UFSOpen(volumeInode);
+	        tfile = osi_UFSOpen(&volumeInode);
 		err =
 		    afs_osi_Read(tfile, sizeof(struct fvolume) * j,
 				 &staticFVolume, sizeof(struct fvolume));
@@ -566,6 +610,11 @@ afs_SetupVolume(afs_int32 volid, char *aname, void *ve, struct cell *tcell,
 	} else {
 	    tv->vtix = -1;
 	    tv->rootVnode = tv->rootUnique = 0;
+            afs_GetDynrootMountFid(&tv->dotdot);
+            afs_GetDynrootMountFid(&tv->mtpoint);
+            tv->mtpoint.Fid.Vnode =
+              VNUM_FROM_TYPEID(VN_TYPE_MOUNT, tcell->cellIndex << 2);
+            tv->mtpoint.Fid.Unique = volid;
 	}
     }
     tv->refCount++;
@@ -593,8 +642,18 @@ afs_SetupVolume(afs_int32 volid, char *aname, void *ve, struct cell *tcell,
 }
 
 
+/**
+ * Seek volume by it's name and attributes.
+ * If volume not found, try to add one.
+ * @param aname Volume name.
+ * @param acell Cell
+ * @param agood
+ * @param areq
+ * @param locktype Type of lock to be used.
+ * @return
+ */
 struct volume *
-afs_GetVolumeByName(register char *aname, afs_int32 acell, int agood,
+afs_GetVolumeByName(char *aname, afs_int32 acell, int agood,
 		    struct vrequest *areq, afs_int32 locktype)
 {
     afs_int32 i;
@@ -615,10 +674,18 @@ afs_GetVolumeByName(register char *aname, afs_int32 acell, int agood,
 
     ReleaseWriteLock(&afs_xvolume);
 
+    if (AFS_IS_DISCONNECTED)
+        return NULL;
+
     tv = afs_NewVolumeByName(aname, acell, agood, areq, locktype);
     return (tv);
 }
 
+/**
+ *   Init a new dynroot volume.
+ * @param Volume FID.
+ * @return Volume or NULL if not found.
+ */
 static struct volume *
 afs_NewDynrootVolume(struct VenusFid *fid)
 {
@@ -647,6 +714,15 @@ afs_NewDynrootVolume(struct VenusFid *fid)
 }
 
 int lastnvcode;
+
+/**
+ * @param aname Volume name.
+ * @param acell Cell id.
+ * @param agood
+ * @param areq Request type.
+ * @param locktype Type of lock to be used.
+ * @return Volume or NULL if failure.
+ */
 static struct volume *
 afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 		    struct vrequest *areq, afs_int32 locktype)
@@ -660,6 +736,7 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
     char *tbuffer, *ve;
     struct afs_conn *tconn;
     struct vrequest treq;
+    struct rx_connection *rxconn;
 
     if (strlen(aname) > VL_MAXNAMELEN)	/* Invalid volume name */
 	return NULL;
@@ -684,34 +761,34 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
     do {
 	tconn =
 	    afs_ConnByMHosts(tcell->cellHosts, tcell->vlport, tcell->cellNum,
-			     &treq, SHARED_LOCK);
+			     &treq, SHARED_LOCK, &rxconn);
 	if (tconn) {
 	    if (tconn->srvr->server->flags & SNO_LHOSTS) {
 		type = 0;
 		RX_AFS_GUNLOCK();
-		code = VL_GetEntryByNameO(tconn->id, aname, tve);
+		code = VL_GetEntryByNameO(rxconn, aname, tve);
 		RX_AFS_GLOCK();
 	    } else if (tconn->srvr->server->flags & SYES_LHOSTS) {
 		type = 1;
 		RX_AFS_GUNLOCK();
-		code = VL_GetEntryByNameN(tconn->id, aname, ntve);
+		code = VL_GetEntryByNameN(rxconn, aname, ntve);
 		RX_AFS_GLOCK();
 	    } else {
 		type = 2;
 		RX_AFS_GUNLOCK();
-		code = VL_GetEntryByNameU(tconn->id, aname, utve);
+		code = VL_GetEntryByNameU(rxconn, aname, utve);
 		RX_AFS_GLOCK();
 		if (!(tconn->srvr->server->flags & SVLSRV_UUID)) {
 		    if (code == RXGEN_OPCODE) {
 			type = 1;
 			RX_AFS_GUNLOCK();
-			code = VL_GetEntryByNameN(tconn->id, aname, ntve);
+			code = VL_GetEntryByNameN(rxconn, aname, ntve);
 			RX_AFS_GLOCK();
 			if (code == RXGEN_OPCODE) {
 			    type = 0;
 			    tconn->srvr->server->flags |= SNO_LHOSTS;
 			    RX_AFS_GUNLOCK();
-			    code = VL_GetEntryByNameO(tconn->id, aname, tve);
+			    code = VL_GetEntryByNameO(rxconn, aname, tve);
 			    RX_AFS_GLOCK();
 			} else if (!code)
 			    tconn->srvr->server->flags |= SYES_LHOSTS;
@@ -722,7 +799,7 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 	    }
 	} else
 	    code = -1;
-    } while (afs_Analyze(tconn, code, NULL, &treq, -1,	/* no op code for this */
+    } while (afs_Analyze(tconn, rxconn, code, NULL, &treq, -1,	/* no op code for this */
 			 SHARED_LOCK, tcell));
 
     if (code) {
@@ -737,7 +814,7 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 	    int i;
 	    struct server *sp;
 	    struct srvAddr *sap;
-	    for (i = 0; i < MAXCELLHOSTS; i++) {
+	    for (i = 0; i < AFS_MAXCELLHOSTS; i++) {
 		if ((sp = tcell->cellHosts[i]) == NULL)
 		    break;
 		for (sap = sp->addr; sap; sap = sap->next_sa)
@@ -797,13 +874,18 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 
 
 
-/* call this with the volume structure locked; used for new-style vldb requests */
+/**
+ *   Call this with the volume structure locked; used for new-style vldb requests.
+ * @param av Volume
+ * @param ve
+ * @param acell
+ */
 void
 InstallVolumeEntry(struct volume *av, struct vldbentry *ve, int acell)
 {
-    register struct server *ts;
+    struct server *ts;
     struct cell *cellp;
-    register int i, j;
+    int i, j;
     afs_int32 mask;
     afs_uint32 temp;
 
@@ -835,12 +917,12 @@ InstallVolumeEntry(struct volume *av, struct vldbentry *ve, int acell)
 
     cellp = afs_GetCell(acell, 0);
 
-    /* This volume, av, is locked. Zero out the serverHosts[] array 
-     * so that if afs_GetServer() decides to replace the server 
+    /* This volume, av, is locked. Zero out the serverHosts[] array
+     * so that if afs_GetServer() decides to replace the server
      * struct, we don't deadlock trying to afs_ResetVolumeInfo()
      * this volume.
      */
-    for (j = 0; j < MAXHOSTS; j++) {
+    for (j = 0; j < AFS_MAXHOSTS; j++) {
 	av->serverHost[j] = 0;
     }
 
@@ -869,19 +951,19 @@ InstallVolumeEntry(struct volume *av, struct vldbentry *ve, int acell)
 	afs_PutServer(ts, WRITE_LOCK);
 	j++;
     }
-    if (j < MAXHOSTS) {
+    if (j < AFS_MAXHOSTS) {
 	av->serverHost[j++] = 0;
     }
-    afs_SortServers(av->serverHost, MAXHOSTS);
+    afs_SortServers(av->serverHost, AFS_MAXHOSTS);
 }				/*InstallVolumeEntry */
 
 
 void
 InstallNVolumeEntry(struct volume *av, struct nvldbentry *ve, int acell)
 {
-    register struct server *ts;
+    struct server *ts;
     struct cell *cellp;
-    register int i, j;
+    int i, j;
     afs_int32 mask;
     afs_uint32 temp;
 
@@ -913,12 +995,12 @@ InstallNVolumeEntry(struct volume *av, struct nvldbentry *ve, int acell)
 
     cellp = afs_GetCell(acell, 0);
 
-    /* This volume, av, is locked. Zero out the serverHosts[] array 
-     * so that if afs_GetServer() decides to replace the server 
+    /* This volume, av, is locked. Zero out the serverHosts[] array
+     * so that if afs_GetServer() decides to replace the server
      * struct, we don't deadlock trying to afs_ResetVolumeInfo()
      * this volume.
      */
-    for (j = 0; j < MAXHOSTS; j++) {
+    for (j = 0; j < AFS_MAXHOSTS; j++) {
 	av->serverHost[j] = 0;
     }
 
@@ -946,10 +1028,10 @@ InstallNVolumeEntry(struct volume *av, struct nvldbentry *ve, int acell)
 	afs_PutServer(ts, WRITE_LOCK);
 	j++;
     }
-    if (j < MAXHOSTS) {
+    if (j < AFS_MAXHOSTS) {
 	av->serverHost[j++] = 0;
     }
-    afs_SortServers(av->serverHost, MAXHOSTS);
+    afs_SortServers(av->serverHost, AFS_MAXHOSTS);
 }				/*InstallNVolumeEntry */
 
 
@@ -957,10 +1039,10 @@ void
 InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 		    struct cell *tcell, struct vrequest *areq)
 {
-    register struct server *ts;
+    struct server *ts;
     struct afs_conn *tconn;
     struct cell *cellp;
-    register int i, j;
+    int i, j;
     afs_uint32 serverid;
     afs_int32 mask;
     int k;
@@ -993,12 +1075,12 @@ InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 
     cellp = afs_GetCell(acell, 0);
 
-    /* This volume, av, is locked. Zero out the serverHosts[] array 
-     * so that if afs_GetServer() decides to replace the server 
+    /* This volume, av, is locked. Zero out the serverHosts[] array
+     * so that if afs_GetServer() decides to replace the server
      * struct, we don't deadlock trying to afs_ResetVolumeInfo()
      * this volume.
      */
-    for (j = 0; j < MAXHOSTS; j++) {
+    for (j = 0; j < AFS_MAXHOSTS; j++) {
 	av->serverHost[j] = 0;
     }
 
@@ -1024,24 +1106,27 @@ InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 		&& ts->addr) {
 		/* uuid, uniquifier, and portal are the same */
 	    } else {
-		afs_uint32 *addrp, nentries, code, unique;
+		afs_uint32 *addrp, code;
+		afs_int32 nentries, unique;
 		bulkaddrs addrs;
 		ListAddrByAttributes attrs;
 		afsUUID uuid;
+		struct rx_connection *rxconn;
 
-		memset((char *)&attrs, 0, sizeof(attrs));
+		memset(&attrs, 0, sizeof(attrs));
 		attrs.Mask = VLADDR_UUID;
 		attrs.uuid = ve->serverNumber[i];
-		memset((char *)&uuid, 0, sizeof(uuid));
-		memset((char *)&addrs, 0, sizeof(addrs));
+		memset(&uuid, 0, sizeof(uuid));
+		memset(&addrs, 0, sizeof(addrs));
 		do {
 		    tconn =
 			afs_ConnByMHosts(tcell->cellHosts, tcell->vlport,
-					 tcell->cellNum, areq, SHARED_LOCK);
+					 tcell->cellNum, areq, SHARED_LOCK,
+					 &rxconn);
 		    if (tconn) {
 			RX_AFS_GUNLOCK();
 			code =
-			    VL_GetAddrsU(tconn->id, &attrs, &uuid, &unique,
+			    VL_GetAddrsU(rxconn, &attrs, &uuid, &unique,
 					 &nentries, &addrs);
 			RX_AFS_GLOCK();
 		    } else {
@@ -1053,7 +1138,7 @@ InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 			code = VL_NOENT;
 
 		} while (afs_Analyze
-			 (tconn, code, NULL, areq, -1, SHARED_LOCK, tcell));
+			 (tconn, rxconn, code, NULL, areq, -1, SHARED_LOCK, tcell));
 		if (code) {
 		    /* Better handing of such failures; for now we'll simply retry this call */
 		    areq->volumeError = 1;
@@ -1067,8 +1152,7 @@ InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 		ts = afs_GetServer(addrp, nentries, acell, cellp->fsport,
 				   WRITE_LOCK, &ve->serverNumber[i],
 				   ve->serverUnique[i]);
-		afs_osi_Free(addrs.bulkaddrs_val,
-			     addrs.bulkaddrs_len * sizeof(*addrp));
+		xdr_free((xdrproc_t) xdr_bulkaddrs, &addrs);
 	    }
 	}
 	av->serverHost[j] = ts;
@@ -1086,10 +1170,15 @@ InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 	j++;
     }
 
-    afs_SortServers(av->serverHost, MAXHOSTS);
+    afs_SortServers(av->serverHost, AFS_MAXHOSTS);
 }				/*InstallVolumeEntry */
 
 
+/**
+ *   Reset volume info for the specified volume strecture. Mark volume
+ * to be rechecked next time.
+ * @param tv
+ */
 void
 afs_ResetVolumeInfo(struct volume *tv)
 {
@@ -1098,7 +1187,7 @@ afs_ResetVolumeInfo(struct volume *tv)
     AFS_STATCNT(afs_ResetVolumeInfo);
     ObtainWriteLock(&tv->lock, 117);
     tv->states |= VRecheck;
-    for (i = 0; i < MAXHOSTS; i++)
+    for (i = 0; i < AFS_MAXHOSTS; i++)
 	tv->status[i] = not_busy;
     if (tv->name) {
 	afs_osi_Free(tv->name, strlen(tv->name) + 1);
