@@ -1,12 +1,16 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#ifndef AFS_INCLUDES_H
+#define AFS_INCLUDES_H 1
+
+
 #ifdef UKERNEL
 #include <UKERNEL/afsincludes.h>
 #else
@@ -17,7 +21,7 @@
 #include "afs/stds.h"
 #ifdef	AFS_AIX_ENV
 #include "osi_vfs.h"
-#elif defined(AFS_HPUX_ENV)
+#elif defined(AFS_HPUX_ENV) || defined(AFS_NBSD_ENV)
 #include "osi_vfs.h"
 #endif
 #if defined(AFS_SGI_ENV) || defined(AFS_LINUX20_ENV)
@@ -51,8 +55,11 @@
 #include "afs/icl.h"
 #include "afs/afs_stats.h"
 #include "afs/afs_prototypes.h"
+#include "afs/discon.h"
 #if defined(AFS_LINUX20_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
 #include "osi_machdep.h"
 #endif
 
 #endif
+
+#endif /* AFS_INCLUDES_H */
