@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -25,6 +25,7 @@
 #include <afs/xstat_fs.h>
 #include <afs/xstat_cm.h>
 
+#include "afsmonitor.h"
 
 
 /* Extern Variables */
@@ -32,8 +33,6 @@ extern int afsmon_debug;	/* debugging on ? */
 extern FILE *debugFD;		/* debug file FD */
 extern char errMsg[256];	/* error message buffer */
 
-extern int afsmon_Exit();	/* exit routine */
-
 static FILE *fs_outFD;		/* fs output file descriptor */
 static FILE *cm_outFD;		/* cm output file descriptor */
 
@@ -85,8 +84,27 @@ static char *xferOpNames[] = {
     "StoreData"
 };
 
+static char *CbCounterStrings[] = {
+    "DeleteFiles",
+    "DeleteCallBacks",
+    "BreakCallBacks",
+    "AddCallBack",
+    "GotSomeSpaces",
+    "DeleteAllCallBacks",
+    "nFEs",
+    "nCBs",
+    "nblks",
+    "CBsTimedOut",
+    "nbreakers",
+    "GSS1",
+    "GSS2",
+    "GSS3",
+    "GSS4",
+    "GSS5"
+};
+
 /*________________________________________________________________________
-				FS STATS ROUTINES 
+				FS STATS ROUTINES
  *_______________________________________________________________________*/
 
 /*------------------------------------------------------------------------
@@ -109,9 +127,7 @@ static char *xferOpNames[] = {
  *------------------------------------------------------------------------*/
 
 void
-Print_fs_OverallPerfInfo(a_ovP)
-     struct afs_PerfStats *a_ovP;
-
+Print_fs_OverallPerfInfo(struct afs_PerfStats *a_ovP)
 {				/*Print_fs_OverallPerfInfo */
 
     fprintf(fs_outFD, "\t%10d numPerfCalls\n\n", a_ovP->numPerfCalls);
@@ -207,8 +223,12 @@ Print_fs_OverallPerfInfo(a_ovP)
     fprintf(fs_outFD, "\t%10d rx_nClientConns\n", a_ovP->rx_nClientConns);
     fprintf(fs_outFD, "\t%10d rx_nPeerStructs\n", a_ovP->rx_nPeerStructs);
     fprintf(fs_outFD, "\t%10d rx_nCallStructs\n", a_ovP->rx_nCallStructs);
-    fprintf(fs_outFD, "\t%10d rx_nFreeCallStructs\n\n",
+    fprintf(fs_outFD, "\t%10d rx_nFreeCallStructs\n",
 	    a_ovP->rx_nFreeCallStructs);
+    fprintf(fs_outFD, "\t%10d rx_nBusies\n\n", a_ovP->rx_nBusies);
+
+    fprintf(fs_outFD, "\t%10d fs_nBusies\n", a_ovP->fs_nBusies);
+    fprintf(fs_outFD, "\t%10d fs_GetCapabilities\n\n", a_ovP->fs_nGetCaps);
 
     /*
      * Host module fields.
@@ -228,6 +248,7 @@ Print_fs_OverallPerfInfo(a_ovP)
     fprintf(fs_outFD, "\t%10d host_ClientBlocks\n\n",
 	    a_ovP->host_ClientBlocks);
 
+    fprintf(fs_outFD, "\t%10d sysname_ID\n", a_ovP->sysname_ID);
 }				/*Print_fs_OverallPerfInfo */
 
 
@@ -252,18 +273,15 @@ Print_fs_OverallPerfInfo(a_ovP)
  *------------------------------------------------------------------------*/
 
 void
-Print_fs_OpTiming(a_opIdx, a_opTimeP)
-     int a_opIdx;
-     struct fs_stats_opTimingData *a_opTimeP;
-
+Print_fs_OpTiming(int a_opIdx, struct fs_stats_opTimingData *a_opTimeP)
 {				/*Print_fs_OpTiming */
 
     fprintf(fs_outFD,
-	    "%15s: %d ops (%d OK); sum=%d.%06d, min=%d.%06d, max=%d.%06d\n",
+	    "%15s: %d ops (%d OK); sum=%ld.%06ld, min=%ld.%06ld, max=%ld.%06ld\n",
 	    fsOpNames[a_opIdx], a_opTimeP->numOps, a_opTimeP->numSuccesses,
-	    a_opTimeP->sumTime.tv_sec, a_opTimeP->sumTime.tv_usec,
-	    a_opTimeP->minTime.tv_sec, a_opTimeP->minTime.tv_usec,
-	    a_opTimeP->maxTime.tv_sec, a_opTimeP->maxTime.tv_usec);
+	    (long)a_opTimeP->sumTime.tv_sec, (long)a_opTimeP->sumTime.tv_usec,
+	    (long)a_opTimeP->minTime.tv_sec, (long)a_opTimeP->minTime.tv_usec,
+	    (long)a_opTimeP->maxTime.tv_sec, (long)a_opTimeP->maxTime.tv_usec);
 
 }				/*Print_fs_OpTiming */
 
@@ -289,18 +307,15 @@ Print_fs_OpTiming(a_opIdx, a_opTimeP)
  *------------------------------------------------------------------------*/
 
 void
-Print_fs_XferTiming(a_opIdx, a_xferP)
-     int a_opIdx;
-     struct fs_stats_xferData *a_xferP;
-
+Print_fs_XferTiming(int a_opIdx, struct fs_stats_xferData *a_xferP)
 {				/*Print_fs_XferTiming */
 
     fprintf(fs_outFD,
-	    "%s: %d xfers (%d OK), time sum=%d.%06d, min=%d.%06d, max=%d.%06d\n",
+	    "%s: %d xfers (%d OK), time sum=%ld.%06ld, min=%ld.%06ld, max=%ld.%06ld\n",
 	    xferOpNames[a_opIdx], a_xferP->numXfers, a_xferP->numSuccesses,
-	    a_xferP->sumTime.tv_sec, a_xferP->sumTime.tv_usec,
-	    a_xferP->minTime.tv_sec, a_xferP->minTime.tv_usec,
-	    a_xferP->maxTime.tv_sec, a_xferP->maxTime.tv_usec);
+	    (long)a_xferP->sumTime.tv_sec, (long)a_xferP->sumTime.tv_usec,
+	    (long)a_xferP->minTime.tv_sec, (long)a_xferP->minTime.tv_usec,
+	    (long)a_xferP->maxTime.tv_sec, (long)a_xferP->maxTime.tv_usec);
     fprintf(fs_outFD, "\t[bytes: sum=%d, min=%d, max=%d]\n",
 	    a_xferP->sumBytes, a_xferP->minBytes, a_xferP->maxBytes);
     fprintf(fs_outFD,
@@ -332,14 +347,12 @@ Print_fs_XferTiming(a_opIdx, a_xferP)
  *------------------------------------------------------------------------*/
 
 void
-Print_fs_DetailedPerfInfo(a_detP)
-     struct fs_stats_DetailedStats *a_detP;
-
+Print_fs_DetailedPerfInfo(struct fs_stats_DetailedStats *a_detP)
 {				/*Print_fs_DetailedPerfInfo */
 
     int currIdx;		/*Loop variable */
 
-    fprintf(fs_outFD, "\t%10d epoch\n", a_detP->epoch);
+    fprintf(fs_outFD, "\t%10ld epoch\n", (long)a_detP->epoch.tv_sec);
 
     for (currIdx = 0; currIdx < FS_STATS_NUM_RPC_OPS; currIdx++)
 	Print_fs_OpTiming(currIdx, &(a_detP->rpcOpTimes[currIdx]));
@@ -371,47 +384,91 @@ Print_fs_DetailedPerfInfo(a_detP)
  *------------------------------------------------------------------------*/
 
 void
-Print_fs_FullPerfInfo(a_fs_Results)
-     struct xstat_fs_ProbeResults *a_fs_Results;	/* ptr to fs results */
+Print_fs_FullPerfInfo(struct xstat_fs_ProbeResults *a_fs_Results)
 {				/*Print_fs_FullPerfInfo */
-
-    static char rn[] = "Print_fs_FullPerfInfo";	/*Routine name */
     static afs_int32 fullPerfLongs = (sizeof(struct fs_stats_FullPerfStats) >> 2);	/*Correct # longs to rcv */
     afs_int32 numLongs;		/*# longwords received */
     struct fs_stats_FullPerfStats *fullPerfP;	/*Ptr to full perf stats */
+    struct fs_stats_FullPerfStats buffer;
     char *printableTime;	/*Ptr to printable time string */
+    time_t probeTime;
+    int code;
 
 
-    numLongs = a_fs_Results->data.AFS_CollData_len;
-    if (numLongs != fullPerfLongs) {
-	fprintf(fs_outFD,
-		" ** Data size mismatch in full performance collection!\n");
-	fprintf(fs_outFD, " ** Expecting %d, got %d\n", fullPerfLongs,
-		numLongs);
-	return;
-    }
-
-    printableTime = ctime((time_t *) & (a_fs_Results->probeTime));
+    probeTime = a_fs_Results->probeTime;
+    printableTime = ctime(&probeTime);
     printableTime[strlen(printableTime) - 1] = '\0';
-    fullPerfP = (struct fs_stats_FullPerfStats *)
-	(a_fs_Results->data.AFS_CollData_val);
 
     fprintf(fs_outFD,
 	    "AFS_XSTATSCOLL_FULL_PERF_INFO (coll %d) for FS %s\n[Probe %d, %s]\n\n",
 	    a_fs_Results->collectionNumber, a_fs_Results->connP->hostName,
 	    a_fs_Results->probeNum, printableTime);
 
-    Print_fs_OverallPerfInfo(&(fullPerfP->overall));
-    Print_fs_DetailedPerfInfo(&(fullPerfP->det));
+    code = xstat_fs_DecodeFullPerfStats(&fullPerfP,
+					a_fs_Results->data.AFS_CollData_val,
+					a_fs_Results->data.AFS_CollData_len,
+					&buffer);
+    if (code) {
+	numLongs = a_fs_Results->data.AFS_CollData_len;
+	fprintf(fs_outFD,
+		" ** Data size mismatch in full performance collection!\n");
+	fprintf(fs_outFD, " ** Expecting %d, got %d\n", fullPerfLongs,
+		numLongs);
+
+    } else {
+	Print_fs_OverallPerfInfo(&(fullPerfP->overall));
+	Print_fs_DetailedPerfInfo(&(fullPerfP->det));
+    }
 
 }				/*Print_fs_FullPerfInfo */
 
+/*------------------------------------------------------------------------
+ * Print_fs_CallbackStats
+ *
+ * Description:
+ *	Print out the AFS_XSTATSCOLL_CBSTATS collection we just
+ *	received.
+ *
+ * Arguments:
+ *	None.
+ *
+ * Returns:
+ *	Nothing.
+ *
+ * Environment:
+ *	All the info we need is nestled into xstat_fs_Results.
+ *
+ * Side Effects:
+ *	As advertised.
+ *------------------------------------------------------------------------*/
+void
+Print_fs_CallBackStats(struct xstat_fs_ProbeResults *a_fs_Results)
+{
+    char *printableTime;
+    time_t probeTime;
+    int numInt32s = xstat_fs_Results.data.AFS_CollData_len;
+    afs_int32 *val = xstat_fs_Results.data.AFS_CollData_val;
+    int i;
+
+    probeTime = a_fs_Results->probeTime;
+    printableTime = ctime(&probeTime);
+    printableTime[strlen(printableTime) - 1] = '\0';
+    fprintf(fs_outFD,
+	    "AFS_XSTATSCOLL_CBSTATS (coll %d) for FS %s\n[Probe %d, %s]\n\n",
+	    a_fs_Results->collectionNumber, a_fs_Results->connP->hostName,
+	    a_fs_Results->probeNum, printableTime);
+
+    numInt32s = min(numInt32s, sizeof(CbCounterStrings)/sizeof(*CbCounterStrings));
+    for (i=0; i<numInt32s; i++) {
+	fprintf(fs_outFD, "\t%10u %s\n", val[i], CbCounterStrings[i]);
+    }
+}				/*Print_fs_CallbackStats */
 
 /*------------------------------------------------------------------------
  * afsmon_fsOutput()
  *
  * Description:
- *	Prints the contents of xstat_fs_Results to an output file. The 
+ *	Prints the contents of xstat_fs_Results to an output file. The
  *	output is either in a compact (longs only) format or a detailed
  *	format giving the names of each of the datums. Output is appended.
  *
@@ -429,9 +486,8 @@ Print_fs_FullPerfInfo(a_fs_Results)
  *	As advertised.
  *------------------------------------------------------------------------*/
 int
-afsmon_fsOutput(a_outfile, a_detOutput)
-     char *a_outfile;		/* ptr to output file name */
-     int a_detOutput;		/* detailed output ? */
+afsmon_fsOutput(char *a_outfile,	/* ptr to output file name */
+		int a_detOutput)	/* detailed output ? */
 {
 
     static char rn[] = "afsmon_fsOutput";	/* routine name */
@@ -440,6 +496,7 @@ afsmon_fsOutput(a_outfile, a_detOutput)
     afs_int32 numLongs;		/* longwords in result */
     afs_int32 *currLong;	/* ptr to longwords in result */
     int i;
+    time_t probeTime;
 
     if (afsmon_debug) {
 	fprintf(debugFD, "[ %s ] Called, a_outfile= %s, a_detOutput= %d\n",
@@ -455,7 +512,8 @@ afsmon_fsOutput(a_outfile, a_detOutput)
     }
 
     /* get the probe time and strip the \n at the end */
-    printTime = ctime((time_t *) & (xstat_fs_Results.probeTime));
+    probeTime = xstat_fs_Results.probeTime;
+    printTime = ctime(&probeTime);
     printTime[strlen(printTime) - 1] = '\0';
     hostname = xstat_fs_Results.connP->hostName;
 
@@ -480,8 +538,15 @@ afsmon_fsOutput(a_outfile, a_detOutput)
 
     /* print detailed information */
     if (a_detOutput) {
-	Print_fs_FullPerfInfo(&xstat_fs_Results);
-	fflush(fs_outFD);
+	if (xstat_fs_Results.collectionNumber ==
+	    AFS_XSTATSCOLL_FULL_PERF_INFO) {
+	    Print_fs_FullPerfInfo(&xstat_fs_Results);
+	    fflush(fs_outFD);
+	} else if (xstat_fs_Results.collectionNumber ==
+		   AFS_XSTATSCOLL_CBSTATS) {
+	    Print_fs_CallBackStats(&xstat_fs_Results);
+	    fflush(fs_outFD);
+	}
     }
 
     if (fclose(fs_outFD))
@@ -520,9 +585,7 @@ afsmon_fsOutput(a_outfile, a_detOutput)
  *------------------------------------------------------------------------*/
 
 void
-Print_cm_UpDownStats(a_upDownP)
-     struct afs_stats_SrvUpDownInfo *a_upDownP;	/*Ptr to server up/down info */
-
+Print_cm_UpDownStats(struct afs_stats_SrvUpDownInfo *a_upDownP)	/*Ptr to server up/down info */
 {				/*Print_cm_UpDownStats */
 
     /*
@@ -601,9 +664,7 @@ Print_cm_UpDownStats(a_upDownP)
  *------------------------------------------------------------------------*/
 
 void
-Print_cm_OverallPerfInfo(a_ovP)
-     struct afs_stats_CMPerf *a_ovP;
-
+Print_cm_OverallPerfInfo(struct afs_stats_CMPerf *a_ovP)
 {				/*Print_cm_OverallPerfInfo */
 
     fprintf(cm_outFD, "\t%10d numPerfCalls\n", a_ovP->numPerfCalls);
@@ -692,14 +753,13 @@ Print_cm_OverallPerfInfo(a_ovP)
  *------------------------------------------------------------------------*/
 
 void
-Print_cm_PerfInfo()
+Print_cm_PerfInfo(void)
 {				/*Print_cm_PerfInfo */
-
-    static char rn[] = "Print_cm_PerfInfo";	/*Routine name */
     static afs_int32 perfLongs = (sizeof(struct afs_stats_CMPerf) >> 2);	/*Correct # longs to rcv */
     afs_int32 numLongs;		/*# longwords received */
     struct afs_stats_CMPerf *perfP;	/*Ptr to performance stats */
     char *printableTime;	/*Ptr to printable time string */
+    time_t probeTime;
 
     numLongs = xstat_cm_Results.data.AFSCB_CollData_len;
     if (numLongs != perfLongs) {
@@ -709,7 +769,8 @@ Print_cm_PerfInfo()
 	return;
     }
 
-    printableTime = ctime((time_t *) & (xstat_cm_Results.probeTime));
+    probeTime = xstat_cm_Results.probeTime;
+    printableTime = ctime(&probeTime);
     printableTime[strlen(printableTime) - 1] = '\0';
     perfP = (struct afs_stats_CMPerf *)
 	(xstat_cm_Results.data.AFSCB_CollData_val);
@@ -747,19 +808,16 @@ Print_cm_PerfInfo()
  *------------------------------------------------------------------------*/
 
 void
-Print_cm_OpTiming(a_opIdx, a_opNames, a_opTimeP)
-     int a_opIdx;
-     char *a_opNames[];
-     struct afs_stats_opTimingData *a_opTimeP;
-
+Print_cm_OpTiming(int a_opIdx, char *a_opNames[],
+		  struct afs_stats_opTimingData *a_opTimeP)
 {				/*Print_cm_OpTiming */
 
     fprintf(cm_outFD,
-	    "%15s: %d ops (%d OK); sum=%d.%06d, min=%d.%06d, max=%d.%06d\n",
+	    "%15s: %d ops (%d OK); sum=%ld.%06ld, min=%ld.%06ld, max=%ld.%06ld\n",
 	    a_opNames[a_opIdx], a_opTimeP->numOps, a_opTimeP->numSuccesses,
-	    a_opTimeP->sumTime.tv_sec, a_opTimeP->sumTime.tv_usec,
-	    a_opTimeP->minTime.tv_sec, a_opTimeP->minTime.tv_usec,
-	    a_opTimeP->maxTime.tv_sec, a_opTimeP->maxTime.tv_usec);
+	    (long)a_opTimeP->sumTime.tv_sec, (long)a_opTimeP->sumTime.tv_usec,
+	    (long)a_opTimeP->minTime.tv_sec, (long)a_opTimeP->minTime.tv_usec,
+	    (long)a_opTimeP->maxTime.tv_sec, (long)a_opTimeP->maxTime.tv_usec);
 
 }				/*Print_cm_OpTiming */
 
@@ -785,19 +843,16 @@ Print_cm_OpTiming(a_opIdx, a_opNames, a_opTimeP)
  *------------------------------------------------------------------------*/
 
 void
-Print_cm_XferTiming(a_opIdx, a_opNames, a_xferP)
-     int a_opIdx;
-     char *a_opNames[];
-     struct afs_stats_xferData *a_xferP;
-
+Print_cm_XferTiming(int a_opIdx, char *a_opNames[],
+		    struct afs_stats_xferData *a_xferP)
 {				/*Print_cm_XferTiming */
 
     fprintf(cm_outFD,
-	    "%s: %d xfers (%d OK), time sum=%d.%06d, min=%d.%06d, max=%d.%06d\n",
+	    "%s: %d xfers (%d OK), time sum=%ld.%06ld, min=%ld.%06ld, max=%ld.%06ld\n",
 	    a_opNames[a_opIdx], a_xferP->numXfers, a_xferP->numSuccesses,
-	    a_xferP->sumTime.tv_sec, a_xferP->sumTime.tv_usec,
-	    a_xferP->minTime.tv_sec, a_xferP->minTime.tv_usec,
-	    a_xferP->maxTime.tv_sec, a_xferP->maxTime.tv_usec);
+	    (long)a_xferP->sumTime.tv_sec, (long)a_xferP->sumTime.tv_usec,
+	    (long)a_xferP->minTime.tv_sec, (long)a_xferP->minTime.tv_usec,
+	    (long)a_xferP->maxTime.tv_sec, (long)a_xferP->maxTime.tv_usec);
     fprintf(cm_outFD, "\t[bytes: sum=%d, min=%d, max=%d]\n",
 	    a_xferP->sumBytes, a_xferP->minBytes, a_xferP->maxBytes);
     fprintf(cm_outFD,
@@ -831,11 +886,8 @@ Print_cm_XferTiming(a_opIdx, a_opNames, a_xferP)
  *------------------------------------------------------------------------*/
 
 void
-Print_cm_ErrInfo(a_opIdx, a_opNames, a_opErrP)
-     int a_opIdx;
-     char *a_opNames[];
-     struct afs_stats_RPCErrors *a_opErrP;
-
+Print_cm_ErrInfo(int a_opIdx, char *a_opNames[],
+		 struct afs_stats_RPCErrors *a_opErrP)
 {				/*Print_cm_ErrInfo */
 
     fprintf(cm_outFD,
@@ -867,9 +919,7 @@ Print_cm_ErrInfo(a_opIdx, a_opNames, a_opErrP)
  *------------------------------------------------------------------------*/
 
 void
-Print_cm_RPCPerfInfo(a_rpcP)
-     struct afs_stats_RPCOpInfo *a_rpcP;
-
+Print_cm_RPCPerfInfo(struct afs_stats_RPCOpInfo *a_rpcP)
 {				/*Print_cm_RPCPerfInfo */
 
     int currIdx;		/*Loop variable */
@@ -917,16 +967,15 @@ Print_cm_RPCPerfInfo(a_rpcP)
  *------------------------------------------------------------------------*/
 
 void
-Print_cm_FullPerfInfo()
+Print_cm_FullPerfInfo(void)
 {				/*Print_cm_FullPerfInfo */
 
-    static char rn[] = "Print_cm_FullPerfInfo";	/* routine name */
     struct afs_stats_AuthentInfo *authentP;	/*Ptr to authentication stats */
     struct afs_stats_AccessInfo *accessinfP;	/*Ptr to access stats */
     static afs_int32 fullPerfLongs = (sizeof(struct afs_stats_CMFullPerf) >> 2);	/*Correct #longs */
     afs_int32 numLongs;		/*# longs actually received */
     struct afs_stats_CMFullPerf *fullP;	/*Ptr to full perf info */
-
+    time_t probeTime;
     char *printableTime;	/*Ptr to printable time string */
 
     numLongs = xstat_cm_Results.data.AFSCB_CollData_len;
@@ -938,7 +987,8 @@ Print_cm_FullPerfInfo()
 	return;
     }
 
-    printableTime = ctime((time_t *) & (xstat_cm_Results.probeTime));
+    probeTime = xstat_cm_Results.probeTime;
+    printableTime = ctime(&probeTime);
     printableTime[strlen(printableTime) - 1] = '\0';
     fullP = (struct afs_stats_CMFullPerf *)
 	(xstat_cm_Results.data.AFSCB_CollData_val);
@@ -991,7 +1041,7 @@ Print_cm_FullPerfInfo()
  * afsmon_cmOutput()
  *
  * Description:
- *	Prints the contents of xstat_cm_Results to an output file. The 
+ *	Prints the contents of xstat_cm_Results to an output file. The
  *	output is either in a compact (longs only) format or a detailed
  *	format giving the names of each of the datums. Output is appended.
  *
@@ -1009,9 +1059,8 @@ Print_cm_FullPerfInfo()
  *	As advertised.
  *------------------------------------------------------------------------*/
 int
-afsmon_cmOutput(a_outfile, a_detOutput)
-     char *a_outfile;		/* ptr to output file name */
-     int a_detOutput;		/* detailed output ? */
+afsmon_cmOutput(char *a_outfile,		/* ptr to output file name */
+		int a_detOutput)		/* detailed output ? */
 {
 
     static char rn[] = "afsmon_cmOutput";	/* routine name */
@@ -1020,6 +1069,7 @@ afsmon_cmOutput(a_outfile, a_detOutput)
     afs_int32 numLongs;		/* longwords in result */
     afs_int32 *currLong;	/* ptr to longwords in result */
     int i;
+    time_t probeTime;
 
     if (afsmon_debug) {
 	fprintf(debugFD, "[ %s ] Called, a_outfile= %s, a_detOutput= %d\n",
@@ -1036,7 +1086,8 @@ afsmon_cmOutput(a_outfile, a_detOutput)
     }
 
     /* get the probe time and strip the \n at the end */
-    printTime = ctime((time_t *) & (xstat_cm_Results.probeTime));
+    probeTime = xstat_cm_Results.probeTime;
+    printTime = ctime(&probeTime);
     printTime[strlen(printTime) - 1] = '\0';
     hostname = xstat_cm_Results.connP->hostName;
 
