@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -21,6 +21,9 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #endif
+#ifdef HAVE_STDINT_H
+# include <stdint.h>
+#endif
 
 #include <afs/procmgmt.h>
 #include <rx/xdr.h>
@@ -31,12 +34,16 @@
 #include <afs/tcdata.h>
 #include <afs/bubasics.h>	/* PA */
 #include <afs/budb_client.h>
+#include <afs/bucoord_prototypes.h>
+#include <afs/butm_prototypes.h>
 #include <afs/volser.h>
+#include <afs/volser_prototypes.h>
 #include <afs/com_err.h>
 #include "error_macros.h"
 #include <afs/afsutil.h>
 #include <errno.h>
 #include "butc_xbsa.h"
+#include "butc_internal.h"
 
 /* GLOBAL CONFIGURATION PARAMETERS */
 extern int queryoperator;
@@ -44,7 +51,7 @@ extern int tapemounted;
 extern char *opencallout;
 extern char *closecallout;
 extern char *whoami;
-extern char *extractDumpName();
+extern char *extractDumpName(char *);
 extern int BufferSize;		/* Size in B stored for header info */
 FILE *restoretofilefd;
 #ifdef xbsa
@@ -122,21 +129,25 @@ struct timeval tp;
 struct timezone tzp;
 
 /* forward declaration */
-afs_int32 readVolumeHeader( /*char *buffer,afs_int32 bufloc,(struct volumeHeader *)vhptr */ );
+afs_int32 readVolumeHeader(char *, afs_int32, struct volumeHeader *);
+int FindVolTrailer(char *, afs_int32, afs_int32 *, struct volumeHeader *);
+int FindVolTrailer2(char *, afs_int32, afs_int32 *, char *, afs_int32,
+		    afs_int32 *, struct volumeHeader *);
+int SkipVolume(struct tc_restoreDesc *, afs_int32, afs_int32, afs_int32,
+	       afs_int32);
+
+
 
 /* The on-disk volume header or trailer can differ in size from platform to platform */
 static struct TapeBlock tapeBlock;
 char tapeVolumeHT[sizeof(struct volumeHeader) + 2 * sizeof(char)];
 
 void
-PrintLog(log, error1, error2, str, a, b, c, d, e, f, g, h, i, j)
-     FILE *log;
-     afs_int32 error1, error2;
-     char *str, *a, *b, *c, *d, *e, *f, *g, *h, *i, *j;
+PrintLogStr(FILE *log, afs_int32 error1, afs_int32 error2, char *str)
 {
     char *err1, *err2;
 
-    fprintf(log, str, a, b, c, d, e, f, g, h, i, j);
+    fprintf(log, "%s", str);
     if (error1) {
 	err2 = "vols";
 	switch (error1) {
@@ -190,10 +201,8 @@ PrintLog(log, error1, error2, str, a, b, c, d, e, f, g, h, i, j)
 }
 
 void
-TapeLog(debug, task, error1, error2, str, a, b, c, d, e, f, g, h, i, j)
-     int debug;
-     afs_int32 task, error1, error2;
-     char *str, *a, *b, *c, *d, *e, *f, *g, *h, *i, *j;
+TapeLogStr(int debug, afs_int32 task, afs_int32 error1, afs_int32 error2,
+	   char *str)
 {
     time_t now;
     char tbuffer[32], *timestr;
@@ -205,35 +214,51 @@ TapeLog(debug, task, error1, error2, str, a, b, c, d, e, f, g, h, i, j)
     fprintf(logIO, "%s: ", timestr);
     if (task)
 	fprintf(logIO, "Task %u: ", task);
-    PrintLog(logIO, error1, error2, str, a, b, c, d, e, f, g, h, i, j);
+    PrintLogStr(logIO, error1, error2, str);
 
     if (lastPass && lastLogIO) {
 	fprintf(lastLogIO, "%s: ", timestr);
 	if (task)
 	    fprintf(lastLogIO, "Task %u: ", task);
-	PrintLog(lastLogIO, error1, error2, str, a, b, c, d, e, f, g, h, i,
-		 j);
+	PrintLogStr(lastLogIO, error1, error2, str);
     }
 
     /* Now print to the screen if debug level requires */
     if (debug <= debugLevel)
-	PrintLog(stdout, error1, error2, str, a, b, c, d, e, f, g, h, i, j);
+	PrintLogStr(stdout, error1, error2, str);
 }
 
 void
-TLog(task, str, a, b, c, d, e, f, g, h, i, j)
-     afs_int32 task;
-     char *str, *a, *b, *c, *d, *e, *f, *g, *h, *i, *j;
+TapeLog(int debug, afs_int32 task, afs_int32 error1, afs_int32 error2,
+	char *fmt, ...)
 {
-    /* Sends message to TapeLog and stdout */
-    TapeLog(0, task, 0, 0, str, a, b, c, d, e, f, g, h, i, j);
+    char tmp[1024];
+    va_list ap;
+
+    va_start(ap, fmt);
+    afs_vsnprintf(tmp, sizeof(tmp), fmt, ap);
+    va_end(ap);
+
+    TapeLogStr(debug, task, error1, error2, tmp);
 }
 
 void
-ErrorLog(debug, task, error1, error2, str, a, b, c, d, e, f, g, h, i, j)
-     int debug;
-     afs_int32 task, error1, error2;
-     char *str, *a, *b, *c, *d, *e, *f, *g, *h, *i, *j;
+TLog(afs_int32 task, char *fmt, ...)
+{
+    char tmp[1024];
+    va_list ap;
+
+    va_start(ap, fmt);
+    afs_vsnprintf(tmp, sizeof(tmp), fmt, ap);
+    va_end(ap);
+
+    /* Sends message to TapeLog and stdout */
+    TapeLogStr(0, task, 0, 0, tmp);
+}
+
+void
+ErrorLogStr(int debug, afs_int32 task, afs_int32 error1, afs_int32 error2,
+	    char *errStr)
 {
     time_t now;
     char tbuffer[32], *timestr;
@@ -246,24 +271,43 @@ ErrorLog(debug, task, error1, error2, str, a, b, c, d, e, f, g, h, i, j)
     /* Print the time and task number */
     if (task)
 	fprintf(ErrorlogIO, "Task %u: ", task);
-    PrintLog(ErrorlogIO, error1, error2, str, a, b, c, d, e, f, g, h, i, j);
 
-    TapeLog(debug, task, error1, error2, str, a, b, c, d, e, f, g, h, i, j);
+    PrintLogStr(ErrorlogIO, error1, error2, errStr);
+    TapeLogStr(debug, task, error1, error2, errStr);
 }
 
 void
-ELog(task, str, a, b, c, d, e, f, g, h, i, j)
-     afs_int32 task;
-     char *str, *a, *b, *c, *d, *e, *f, *g, *h, *i, *j;
+ErrorLog(int debug, afs_int32 task, afs_int32 error1, afs_int32 error2,
+	 char *fmt, ...)
 {
+    char tmp[1024];
+    va_list ap;
+
+    va_start(ap, fmt);
+    afs_vsnprintf(tmp, sizeof(tmp), fmt, ap);
+    va_end(ap);
+
+    ErrorLogStr(debug, task, error1, error2, tmp);
+
+}
+
+void
+ELog(afs_int32 task, char *fmt, ...)
+{
+    char tmp[1024];
+    va_list ap;
+
+    va_start(ap, fmt);
+    afs_vsnprintf(tmp, sizeof(tmp), fmt, ap);
+    va_end(ap);
+
     /* Sends message to ErrorLog, TapeLog and stdout */
-    ErrorLog(0, task, 0, 0, str, a, b, c, d, e, f, g, h, i, j);
+    ErrorLog(0, task, 0, 0, "%s", tmp);
 }
 
 /* first proc called by anybody who intends to use the device */
 void
-EnterDeviceQueue(devLatch)
-     struct deviceSyncNode *devLatch;
+EnterDeviceQueue(struct deviceSyncNode *devLatch)
 {
     ObtainWriteLock(&(devLatch->lock));
     devLatch->flags = TC_DEVICEINUSE;
@@ -271,8 +315,7 @@ EnterDeviceQueue(devLatch)
 
 /* last proc called by anybody finishing using the device */
 void
-LeaveDeviceQueue(devLatch)
-     struct deviceSyncNode *devLatch;
+LeaveDeviceQueue(struct deviceSyncNode *devLatch)
 {
     devLatch->flags = 0;
     ReleaseWriteLock(&(devLatch->lock));
@@ -320,7 +363,7 @@ WaitForKeystroke(int seconds)
 	 * process run some*/
 	select(0, 0, 0, 0, &twait);
 
-	if (seconds > 0) {	/* we only worry about elapsed time if 
+	if (seconds > 0) {	/* we only worry about elapsed time if
 				 * not looping forever (seconds < 0) */
 	    time(&nowTime);
 	    timeleft = seconds - difftime(nowTime, startTime);
@@ -366,7 +409,7 @@ WaitForKeystroke(int seconds)
  * Input:
  *    seconds - how long to wait for a key press.
  *    *key    - char entered by user
- * Return Values: 
+ * Return Values:
  *    0 - Time ran out before the user typed a key.
  *    1 - Valid char is being returned.
  *
@@ -395,15 +438,16 @@ GetResponseKey(int seconds, char *key)
 #endif
     return rc;
 }
-#endif /* AFS_PTHREAD_ENV
-        * 
-        * /* FFlushInput
-        * *     flush all input
-        * * notes:
-        * *     only external clients are in recoverDb.c. Was static. PA
-        */
+#endif /* AFS_PTHREAD_ENV */
+
+/*
+ * FFlushInput
+ *     flush all input
+ * notes:
+ *     only external clients are in recoverDb.c. Was static. PA
+ */
 void
-FFlushInput()
+FFlushInput(void)
 {
     int w;
 
@@ -429,15 +473,9 @@ FFlushInput()
 }
 
 int
-callOutRoutine(taskId, tapePath, flag, name, dbDumpId, tapecount)
-     afs_int32 taskId;
-     char *tapePath;
-     int flag;
-     char *name;
-     afs_uint32 dbDumpId;
-     int tapecount;
+callOutRoutine(afs_int32 taskId, char *tapePath, int flag, char *name,
+	       afs_uint32 dbDumpId, int tapecount)
 {
-    afs_int32 code = 0;
     int pid;
 
     char StapePath[256];
@@ -546,9 +584,7 @@ callOutRoutine(taskId, tapePath, flag, name, dbDumpId, tapecount)
  *     (unless a tape is not mounted in the first place).
  */
 void
-unmountTape(taskId, tapeInfoPtr)
-     afs_int32 taskId;
-     struct butm_tapeInfo *tapeInfoPtr;
+unmountTape(afs_int32 taskId, struct butm_tapeInfo *tapeInfoPtr)
 {
     afs_int32 code;
     int cpid, status, rcpid;
@@ -602,9 +638,7 @@ unmountTape(taskId, tapeInfoPtr)
  */
 
 void static
-PrintPrompt(flag, name, dumpid)
-     int flag;
-     char *name;
+PrintPrompt(int flag, char *name, int dumpid)
 {
     char tapename[BU_MAXTAPELEN + 32];
     char *dn;
@@ -671,21 +705,17 @@ PrintPrompt(flag, name, dumpid)
 }
 
 /* PromptForTape
- *	Prompt the operator to change the tape. 
+ *	Prompt the operator to change the tape.
  *      Use to be a void routine but now returns an error. Some calls
  *      don't use the error code.
  * notes:
  *	only external clients are in recoverDb.c. Was static PA
  */
 afs_int32
-PromptForTape(flag, name, dbDumpId, taskId, tapecount)
-     int flag;
-     char *name;
-     afs_uint32 dbDumpId;	/* Specific dump ID - If non-zero */
-     afs_uint32 taskId;
-     int tapecount;
+PromptForTape(int flag, char *name, afs_uint32 dbDumpId, afs_uint32 taskId,
+	      int tapecount)
 {
-    register afs_int32 code = 0;
+    afs_int32 code = 0;
     afs_int32 wcode;
     afs_int32 start = 0;
     char inchr;
@@ -861,8 +891,8 @@ PromptForTape(flag, name, dbDumpId, taskId, tapecount)
  */
 
 afs_int32
-VolHeaderToHost(hostVolHeader, tapeVolHeader)
-     struct volumeHeader *hostVolHeader, *tapeVolHeader;
+VolHeaderToHost(struct volumeHeader *hostVolHeader,
+		struct volumeHeader *tapeVolHeader)
 {
     switch (ntohl(tapeVolHeader->versionflags)) {
     case TAPE_VERSION_0:
@@ -903,10 +933,9 @@ VolHeaderToHost(hostVolHeader, tapeVolHeader)
 }
 
 afs_int32
-ReadVolHeader(taskId, tapeInfoPtr, volHeaderPtr)
-     afs_int32 taskId;
-     struct butm_tapeInfo *tapeInfoPtr;
-     struct volumeHeader *volHeaderPtr;
+ReadVolHeader(afs_int32 taskId,
+	      struct butm_tapeInfo *tapeInfoPtr,
+	      struct volumeHeader *volHeaderPtr)
 {
     afs_int32 code = 0;
     afs_int32 nbytes;
@@ -940,12 +969,8 @@ ReadVolHeader(taskId, tapeInfoPtr, volHeaderPtr)
 }
 
 afs_int32 static
-GetVolumeHead(taskId, tapeInfoPtr, position, volName, volId)
-     afs_int32 taskId;
-     struct butm_tapeInfo *tapeInfoPtr;
-     afs_int32 position;
-     char *volName;
-     afs_int32 volId;
+GetVolumeHead(afs_int32 taskId, struct butm_tapeInfo *tapeInfoPtr,
+	      afs_int32 position, char *volName, afs_int32 volId)
 {
     afs_int32 code = 0;
     struct volumeHeader tapeVolHeader;
@@ -1014,12 +1039,8 @@ GetVolumeHead(taskId, tapeInfoPtr, position, volName, volId)
 }
 
 afs_int32
-GetRestoreTape(taskId, tapeInfoPtr, tname, tapeID, prompt)
-     afs_int32 taskId;
-     struct butm_tapeInfo *tapeInfoPtr;
-     char *tname;
-     afs_int32 tapeID;
-     int prompt;
+GetRestoreTape(afs_int32 taskId, struct butm_tapeInfo *tapeInfoPtr,
+	       char *tname, afs_int32 tapeID, int prompt)
 {
     struct butm_tapeLabel tapeLabel;
     afs_int32 code = 0, rc;
@@ -1084,12 +1105,11 @@ GetRestoreTape(taskId, tapeInfoPtr, tname, tapeID, prompt)
 }
 
 afs_int32
-xbsaRestoreVolumeData(call, rparamsPtr)
-     register struct rx_call *call;
-     struct restoreParams *rparamsPtr;
+xbsaRestoreVolumeData(struct rx_call *call, void *rock)
 {
     afs_int32 code = 0;
 #ifdef xbsa
+    struct restoreParams *rparamsPtr = (struct restoreParams *)rock;
     afs_int32 curChunk, rc;
     afs_uint32 totalWritten;
     afs_int32 headBytes, tailBytes, w;
@@ -1237,13 +1257,12 @@ xbsaRestoreVolumeData(call, rparamsPtr)
  */
 
 afs_int32
-restoreVolumeData(call, rparamsPtr)
-     register struct rx_call *call;
-     struct restoreParams *rparamsPtr;
+restoreVolumeData(struct rx_call *call, void *rock)
 {
+    struct restoreParams *rparamsPtr = (struct restoreParams *)rock;
     afs_int32 curChunk;
     afs_uint32 totalWritten = 0;
-    afs_int32 code;
+    afs_int32 code = 0;
     afs_int32 headBytes, tailBytes, w;
     afs_int32 taskId;
     afs_int32 nbytes;		/* # bytes data in last tape block read */
@@ -1343,7 +1362,7 @@ restoreVolumeData(call, rparamsPtr)
 	}
 
 	/* lastbuf is last block read and it has nbytes of data
-	 * startWbuf is the 2nd to last block read 
+	 * startWbuf is the 2nd to last block read
 	 * Seach for the volume trailer in these two blocks.
 	 */
 	if (lastbuf == startWbuf)
@@ -1385,7 +1404,7 @@ restoreVolumeData(call, rparamsPtr)
 	if (!tapeVolTrailer.contd)
 	    break;		/* We've read the entire volume */
 
-	/* Volume is continued on next tape. 
+	/* Volume is continued on next tape.
 	 * Step to the next volume fragment and prompt for its tape.
 	 * When a volume has multiple frags, those frags are on different
 	 * tapes. So we know that we need to prompt for a tape.
@@ -1426,10 +1445,9 @@ restoreVolumeData(call, rparamsPtr)
 /* SkipTape
  *    Find all the volumes on a specific tape and mark them to skip.
  */
-SkipTape(Restore, size, index, tapename, tapeid, taskid)
-     struct tc_restoreDesc *Restore;
-     afs_int32 size, index, tapeid, taskid;
-     char *tapename;
+int
+SkipTape(struct tc_restoreDesc *Restore, afs_int32 size, afs_int32 index,
+	 char *tapename, afs_int32 tapeid, afs_int32 taskid)
 {
     afs_int32 i, tid;
 
@@ -1449,9 +1467,9 @@ SkipTape(Restore, size, index, tapename, tapeid, taskid)
 /* SkipVolume
  *    Find all the entries for a volume and mark them to skip.
  */
-SkipVolume(Restore, size, index, volid, taskid)
-     struct tc_restoreDesc *Restore;
-     afs_int32 size, index, volid, taskid;
+int
+SkipVolume(struct tc_restoreDesc *Restore, afs_int32 size, afs_int32 index,
+	   afs_int32 volid, afs_int32 taskid)
 {
     afs_int32 i;
     int report = 1;
@@ -1472,10 +1490,9 @@ SkipVolume(Restore, size, index, volid, taskid)
     return 0;
 }
 
-xbsaRestoreVolume(taskId, restoreInfo, rparamsPtr)
-     afs_uint32 taskId;
-     struct tc_restoreDesc *restoreInfo;
-     struct restoreParams *rparamsPtr;
+int
+xbsaRestoreVolume(afs_uint32 taskId, struct tc_restoreDesc *restoreInfo,
+		  struct restoreParams *rparamsPtr)
 {
     afs_int32 code = 0;
 #ifdef xbsa
@@ -1664,10 +1681,9 @@ xbsaRestoreVolume(taskId, restoreInfo, rparamsPtr)
     return (code);
 }
 
-restoreVolume(taskId, restoreInfo, rparamsPtr)
-     afs_uint32 taskId;
-     struct tc_restoreDesc *restoreInfo;
-     struct restoreParams *rparamsPtr;
+int
+restoreVolume(afs_uint32 taskId, struct tc_restoreDesc *restoreInfo,
+	      struct restoreParams *rparamsPtr)
 {
     afs_int32 code = 0, rc;
     afs_int32 newServer, newPart, newVolId;
@@ -1832,7 +1848,7 @@ Restorer(void *param) {
 
 	/* restoreVolume function takes care of all the related fragments
 	 * spanning various tapes. On return the complete volume has been
-	 * restored 
+	 * restored
 	 */
 	if (CONF_XBSA) {
 	    tcode = xbsaRestoreVolume(taskId, RestoreDesc, &rparams);
@@ -1918,16 +1934,14 @@ Restorer(void *param) {
 
     FreeNode(taskId);
     LeaveDeviceQueue(deviceLatch);
-    return (void *)(code);
+    return (void *)(intptr_t)(code);
 }
 
 /* this is just scaffolding, creates new tape label with name <tapeName> */
 
 void
-GetNewLabel(tapeInfoPtr, pName, AFSName, tapeLabel)
-     struct butm_tapeInfo *tapeInfoPtr;
-     char *pName, *AFSName;
-     struct butm_tapeLabel *tapeLabel;
+GetNewLabel(struct butm_tapeInfo *tapeInfoPtr, char *pName, char *AFSName,
+	    struct butm_tapeLabel *tapeLabel)
 {
     struct timeval tp;
     struct timezone tzp;
@@ -1959,13 +1973,11 @@ GetNewLabel(tapeInfoPtr, pName, AFSName, tapeLabel)
     strcpy(tapeLabel->creator.cell, globalCellName);
 }
 
-/* extracts trailer out of buffer, nbytes is set to total data in buffer - trailer size */
+/* extracts trailer out of buffer, nbytes is set to total data in
+ * buffer - trailer size */
 afs_int32
-ExtractTrailer(buffer, size, nbytes, volTrailerPtr)
-     char *buffer;
-     afs_int32 *nbytes;
-     afs_int32 size;
-     struct volumeHeader *volTrailerPtr;
+ExtractTrailer(char *buffer, afs_int32 size, afs_int32 *nbytes,
+	       struct volumeHeader *volTrailerPtr)
 {
     afs_int32 code = 0;
     afs_int32 startPos;
@@ -1993,11 +2005,8 @@ ExtractTrailer(buffer, size, nbytes, volTrailerPtr)
 }
 
 int
-FindVolTrailer(buffer, size, dSize, volTrailerPtr)
-     char *buffer;
-     afs_int32 size;
-     struct volumeHeader *volTrailerPtr;
-     afs_int32 *dSize;		/* dataSize */
+FindVolTrailer(char *buffer, afs_int32 size, afs_int32 *dSize,
+	       struct volumeHeader *volTrailerPtr)
 {
     afs_int32 offset, s;
     int found;
@@ -2017,15 +2026,9 @@ FindVolTrailer(buffer, size, dSize, volTrailerPtr)
 }
 
 int
-FindVolTrailer2(buffera, sizea, dataSizea, bufferb, sizeb, dataSizeb,
-		volTrailerPtr)
-     char *buffera;
-     afs_int32 sizea;
-     afs_int32 *dataSizea;
-     char *bufferb;
-     afs_int32 sizeb;
-     afs_int32 *dataSizeb;
-     struct volumeHeader *volTrailerPtr;
+FindVolTrailer2(char *buffera, afs_int32 sizea, afs_int32 *dataSizea,
+		char *bufferb, afs_int32 sizeb, afs_int32 *dataSizeb,
+		struct volumeHeader *volTrailerPtr)
 {
     afs_int32 offset, s;
     afs_int32 headB, tailB;
@@ -2070,11 +2073,9 @@ FindVolTrailer2(buffera, sizea, dataSizea, bufferb, sizeb, dataSizeb,
     return found;
 }
 
-/* Returns true or false depending on whether the tape is expired or not */
 
 Date
-ExpirationDate(dumpid)
-     afs_int32 dumpid;
+ExpirationDate(afs_int32 dumpid)
 {
     afs_int32 code;
     Date expiration = 0;
@@ -2084,9 +2085,10 @@ ExpirationDate(dumpid)
 
     if (dumpid) {
 	/*
-	 * Get the expiration date from DB if its there. The expiration of any tape
-	 * will be the most future expiration of any dump in the set. Can't use
-	 * bcdb_FindTape because dumpid here pertains to the initial dump id.
+	 * Get the expiration date from DB if its there. The expiration of
+	 * any tape will be the most future expiration of any dump in the
+	 * set. Can't use bcdb_FindTape because dumpid here pertains to the
+	 * initial dump id.
 	 */
 	code = bcdb_FindLastTape(dumpid, &dumpEntry, &tapeEntry, &volEntry);
 	if (!code)
@@ -2095,9 +2097,10 @@ ExpirationDate(dumpid)
     return (expiration);
 }
 
+/* Returns true or false depending on whether the tape is expired or not */
+
 int
-tapeExpired(tapeLabelPtr)
-     struct butm_tapeLabel *tapeLabelPtr;
+tapeExpired(struct butm_tapeLabel *tapeLabelPtr)
 {
     Date expiration;
     struct timeval tp;
@@ -2113,16 +2116,16 @@ tapeExpired(tapeLabelPtr)
 
 /* updateTapeLabel
  *	given the label on the tape, delete any old information from the
- *	database. 
+ *	database.
 
  * Deletes all entries that match the volset.dumpnode
  *	and the dump path.
  */
 
-updateTapeLabel(labelIfPtr, tapeInfoPtr, newLabelPtr)
-     struct labelTapeIf *labelIfPtr;
-     struct butm_tapeInfo *tapeInfoPtr;
-     struct butm_tapeLabel *newLabelPtr;
+int
+updateTapeLabel(struct labelTapeIf *labelIfPtr,
+		struct butm_tapeInfo *tapeInfoPtr,
+		struct butm_tapeLabel *newLabelPtr)
 {
     struct butm_tapeLabel oldLabel;
     afs_int32 i, code = 0;
@@ -2158,7 +2161,7 @@ updateTapeLabel(labelIfPtr, tapeInfoPtr, newLabelPtr)
 
 	    if ((strcmp(newLabelPtr->AFSName, "") != 0)
 		&& (strcmp(oldLabel.pName, "") != 0)) {
-		/* We are setting the AFS name, yet tape 
+		/* We are setting the AFS name, yet tape
 		 * has a permanent name (not allowed).
 		 */
 		TLog(taskId, "Can't label. Tape has permanent label '%s'\n",
@@ -2280,7 +2283,7 @@ Labeller(void *param)
 
     free(labelIfPtr);
     LeaveDeviceQueue(deviceLatch);
-    return (void *)(code);
+    return (void *)(intptr_t)(code);
 }
 
 /* PrintTapeLabel
@@ -2288,8 +2291,7 @@ Labeller(void *param)
  */
 
 void
-PrintTapeLabel(labelptr)
-     struct butm_tapeLabel *labelptr;
+PrintTapeLabel(struct butm_tapeLabel *labelptr)
 {
     char tapeName[BU_MAXTAPELEN + 32];
     time_t t;
@@ -2317,14 +2319,14 @@ PrintTapeLabel(labelptr)
     printf("-- End of tape label --\n\n");
 }
 
-/* ReadLabel 
+/* ReadLabel
  * 	Read the label from a tape.
  *	Currently prints out a "detailed" summary of the label but passes
  *	back only selected fields.
  */
 
-ReadLabel(label)
-     struct tc_tapeLabel *label;
+int
+ReadLabel(struct tc_tapeLabel *label)
 {
     struct butm_tapeLabel newTapeLabel;
     struct butm_tapeInfo tapeInfo;
@@ -2413,14 +2415,9 @@ ReadLabel(label)
    into consideration, different word alignment rules.
 */
 afs_int32
-readVolumeHeader(buffer, bufloc, header)
-     /*in */
-     char *buffer;		/* buffer to read header from */
-				/*in */ afs_int32 bufloc;
-				/* header's location in buffer */
-						/*out */ struct volumeHeader *header;
-						/* header structure */
-
+readVolumeHeader(char *buffer,		/* in - buffer to read header from */
+		 afs_int32 bufloc,      /* in - header's location in buffer */
+		 struct volumeHeader *header) /* out -header structure */
 {
     struct volumeHeader vhptr, *tempvhptr;
     afs_int32 firstSplice = (afs_int32) ((char*)& vhptr.pad - (char*) & vhptr);
@@ -2428,7 +2425,7 @@ readVolumeHeader(buffer, bufloc, header)
     int nextSplice = sizeof(struct volumeHeader) - firstSplice - padLen;
 
     /* Four cases are to be handled
-     * 
+     *
      * Volume Header (byte alignment)
      * -----------------------
      * Tape   In Core
@@ -2438,7 +2435,7 @@ readVolumeHeader(buffer, bufloc, header)
      * Case 3:  1       1
      * Case 4:  1       4
      * -----------------------
-     * 
+     *
      * Case 2 and Case 3 are identical cases and handled the same way.
      * Case 1 and Case 4 are separate cases. In one case the pad needs
      * to be removed and in the other, it needs to be spliced in. The
@@ -2461,7 +2458,9 @@ readVolumeHeader(buffer, bufloc, header)
 	/* Handle Case 1 */
 	memset(&vhptr, 0, sizeof(struct volumeHeader));
 	memcpy(&vhptr, buffer + bufloc, firstSplice);
-	memcpy(&vhptr + firstSplice, buffer + bufloc + firstSplice + padLen,
+	/* probably GCC bug 37060; however, no guarantee on length of buffer */
+	tempvhptr = (struct volumeHeader *)(buffer + firstSplice);
+	memcpy(tempvhptr, buffer + bufloc + firstSplice + padLen,
 	       nextSplice);
 	HEADER_CHECKS(vhptr, header);
 
