@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -66,6 +66,10 @@
 #define SYS_NAME_ID_ppc64_darwin_100     513
 #define SYS_NAME_ID_x86_darwin_100       514
 #define SYS_NAME_ID_amd64_darwin_100     515
+#define SYS_NAME_ID_arm_darwin_100       516
+#define SYS_NAME_ID_x86_darwin_110       517
+#define SYS_NAME_ID_amd64_darwin_110     518
+#define SYS_NAME_ID_arm_darwin_110       519
 
 #define SYS_NAME_ID_next_mach20		 601
 #define SYS_NAME_ID_next_mach30		 602
@@ -192,7 +196,13 @@
 #define SYS_NAME_ID_i386_fbsd_62        2114
 #define SYS_NAME_ID_i386_fbsd_70        2115
 #define SYS_NAME_ID_i386_fbsd_71        2116
+#define SYS_NAME_ID_i386_fbsd_72        2130
+#define SYS_NAME_ID_i386_fbsd_73        2131
+#define SYS_NAME_ID_i386_fbsd_74        2132
 #define SYS_NAME_ID_i386_fbsd_80        2117
+#define SYS_NAME_ID_i386_fbsd_81        2118
+#define SYS_NAME_ID_i386_fbsd_82        2119
+#define SYS_NAME_ID_i386_fbsd_90        2120
 
 #define SYS_NAME_ID_ia64_linux2		2200
 #define SYS_NAME_ID_ia64_linux22	2201
@@ -256,6 +266,7 @@
 #define SYS_NAME_ID_i386_obsd45         2614
 #define SYS_NAME_ID_i386_obsd46         2615
 #define SYS_NAME_ID_i386_obsd47         2616
+#define SYS_NAME_ID_i386_obsd48         2617
 
 #define SYS_NAME_ID_amd64_linux2        2700
 #define SYS_NAME_ID_amd64_linux22       2701
@@ -272,10 +283,16 @@
 #define SYS_NAME_ID_ppc64_linux24	2902
 #define SYS_NAME_ID_ppc64_linux26	2903
 
-#define SYS_NAME_ID_amd64_fbsd_53        3008
-#define SYS_NAME_ID_amd64_fbsd_70        3009
-#define SYS_NAME_ID_amd64_fbsd_71        3010
-#define SYS_NAME_ID_amd64_fbsd_80        3011
+#define SYS_NAME_ID_amd64_fbsd_53       3008
+#define SYS_NAME_ID_amd64_fbsd_70       3009
+#define SYS_NAME_ID_amd64_fbsd_71       3010
+#define SYS_NAME_ID_amd64_fbsd_72       3030
+#define SYS_NAME_ID_amd64_fbsd_73       3031
+#define SYS_NAME_ID_amd64_fbsd_74       3032
+#define SYS_NAME_ID_amd64_fbsd_80       3011
+#define SYS_NAME_ID_amd64_fbsd_81       3012
+#define SYS_NAME_ID_amd64_fbsd_82       3013
+#define SYS_NAME_ID_amd64_fbsd_90       3020
 
 #define SYS_NAME_ID_amd64_w2k           3400
 
@@ -288,9 +305,27 @@
 
 #define SYS_NAME_ID_i386_dfbsd_22        3900
 #define SYS_NAME_ID_i386_dfbsd_23        3901
+#define SYS_NAME_ID_i386_dfbsd_24        3902
+#define SYS_NAME_ID_i386_dfbsd_25        3903
+#define SYS_NAME_ID_i386_dfbsd_26        3904
+#define SYS_NAME_ID_i386_dfbsd_27        3905
+
+#define SYS_NAME_ID_amd64_obsd36        4005
+#define SYS_NAME_ID_amd64_obsd37        4006
+#define SYS_NAME_ID_amd64_obsd38        4007
+#define SYS_NAME_ID_amd64_obsd39        4008
+#define SYS_NAME_ID_amd64_obsd40        4009
+#define SYS_NAME_ID_amd64_obsd41        4010
+#define SYS_NAME_ID_amd64_obsd42        4011
+#define SYS_NAME_ID_amd64_obsd43        4012
+#define SYS_NAME_ID_amd64_obsd44        4013
+#define SYS_NAME_ID_amd64_obsd45        4014
+#define SYS_NAME_ID_amd64_obsd46        4015
+#define SYS_NAME_ID_amd64_obsd47        4016
+#define SYS_NAME_ID_amd64_obsd48        4017
 
 /*
- * Placeholder to keep system-wide standard flags since this file is included by all 
+ * Placeholder to keep system-wide standard flags since this file is included by all
  * files (i.e in afs/param.h)
  */
 #ifdef	notdef
@@ -313,5 +348,5 @@
 #define	AFS_REALM_SZ		64
 #endif
 /* Specifies the number of equivalent local realm names */
-#define AFS_NUM_LREALMS         4
+#define AFS_NUM_LREALMS 4
 #endif /* __AFS_SYSNAMES_INCL_ENV_ */
