@@ -20,16 +20,15 @@
 #include <afs/afs_sysnames.h>
 
 #define AFS_DARWIN_ENV
-#define AFS_DARWIN13_ENV
-#define AFS_DARWIN14_ENV
-#define AFS_DARWIN60_ENV
 #define AFS_DARWIN70_ENV
 #define AFS_DARWIN80_ENV
 #define AFS_DARWIN90_ENV
+#undef  AFS_NONFSTRANS
 #define AFS_NONFSTRANS
 #define AFS_SYSCALL             230
 #define AFS_NAMEI_ENV 1
 #define DARWIN_REFBASE 3
+#define AFS_WARNUSER_MARINER_ENV 1
 #define AFS_CACHE_VNODE_PATH
 
 /* File system entry (used if mount.h doesn't define MOUNT_AFS */
@@ -65,6 +64,7 @@
 
 #define AFS_GCPAGS               0
 #define RXK_LISTENER_ENV         1
+#define RXK_TIMEDSLEEP_ENV       1
 
 #ifdef KERNEL
 #undef MACRO_BEGIN
@@ -117,15 +117,14 @@
 #include <afs/afs_sysnames.h>
 #define AFS_USERSPACE_ENV
 #define AFS_USR_DARWIN_ENV
-#define AFS_USR_DARWIN13_ENV
-#define AFS_USR_DARWIN14_ENV
-#define AFS_USR_DARWIN60_ENV
 #define AFS_USR_DARWIN70_ENV
 #define AFS_USR_DARWIN80_ENV
 #define AFS_USR_DARWIN90_ENV
+#undef  AFS_NONFSTRANS
 #define AFS_NONFSTRANS
 #define AFS_SYSCALL             230
 #define DARWIN_REFBASE 0
+#define AFS_WARNUSER_MARINER_ENV 1
 
 /* File system entry (used if mount.h doesn't define MOUNT_AFS */
 #define AFS_MOUNT_AFS    "afs"
