@@ -31,7 +31,7 @@
  *
  * originally written by Steve Miller, MIT Project Athena
  */
-#include <mit-cpyright.h>
+#include "mit-cpyright.h"
 
 #include <afsconfig.h>
 #include <afs/param.h>
@@ -39,11 +39,12 @@
 
 #ifndef KERNEL
 #include <stdio.h>
+#include <string.h>
 #endif
 #ifdef AFS_PTHREAD_ENV
 #include <pthread.h>
 #endif /* AFS_PTHREAD_ENV */
-#include <des.h>
+#include "des.h"
 #include "des_internal.h"
 #include "s_table.h"
 #ifdef BIG
@@ -52,6 +53,21 @@
 
 #include "des_prototypes.h"
 
+#ifdef AFS_HPUX_ENV
+/* When compiling with '-g -O', the HP-UX compiler can get confused with
+ * including ip.c and fp.c below. The result is that the compiler spins on
+ * the CPU and never compiles the file. As a workaround, we can avoid this if
+ * R1 and L1 are declared 'register' (I'm guessing this has to do with the
+ * fact that they are used in ip.c/fp.c, and declaring them register avoids
+ * creating debugging information, and it's that debugging information that
+ * the compiler is flipping out over since they're in multiple .c files). So
+ * declare them as 'register' on HP-UX; only do it on HP-UX to minimize the
+ * scope of such a change, so we don't impact other platforms. */
+# define HPUX_REGISTER register
+#else
+# define HPUX_REGISTER
+#endif
+
 #define XPRT_DES
 
 #ifdef DEBUG
@@ -66,13 +82,13 @@
 
 afs_int32
 des_ecb_encrypt(void * clear, void * cipher,
-		register des_key_schedule schedule, int encrypt)
+		des_key_schedule schedule, int encrypt)
 {
     /* better pass 8 bytes, length not checked here */
 
-    register afs_uint32 R1 = 0;
-    register afs_uint32 L1 = 0;	/* R1 = r10, L1 = r9 */
-    register afs_uint32 R2 = 0, L2 = 0;	/* R2 = r8, L2 = r7 */
+    HPUX_REGISTER afs_uint32 R1 = 0;
+    HPUX_REGISTER afs_uint32 L1 = 0;	/* R1 = r10, L1 = r9 */
+    afs_uint32 R2 = 0, L2 = 0;	/* R2 = r8, L2 = r7 */
     afs_int32 i;
     /* one more registers left on VAX, see below P_temp_p */
 #ifdef BITS16
@@ -116,7 +132,7 @@ des_ecb_encrypt(void * clear, void * cipher,
 	abort();
     }
 #endif
-    if ((afs_uint32) clear & 3) {
+    if (afs_pointer_to_int(clear) & 3) {
 	memcpy((char *)(&L_save), (char *)clear, sizeof(L_save));
 	clear=((afs_uint32*)clear)+1;
 	memcpy((char *)(&R_save), (char *)clear, sizeof(R_save));
@@ -428,7 +444,7 @@ des_ecb_encrypt(void * clear, void * cipher,
      */
 
 #ifdef MUSTALIGN
-    if ((afs_int32) cipher & 3) {
+    if (afs_pointer_to_int(cipher) & 3) {
 	L_save = L2;		/* cant bcopy a reg */
 	R_save = R2;
 	memcpy((char *)cipher, (char *)&L_save, sizeof(L_save));
@@ -437,7 +453,7 @@ des_ecb_encrypt(void * clear, void * cipher,
     } else
 #endif
     {
-        *((afs_uint32*)cipher)= L2;	
+        *((afs_uint32*)cipher)= L2;
 	cipher = ((afs_int32 *)cipher)+1;
 	*((afs_uint32 *)cipher) = R2;
     }
