@@ -20,13 +20,14 @@
  * spm 8/85	MIT project athena
  */
 
-#include <mit-cpyright.h>
+#include "mit-cpyright.h"
 #ifndef KERNEL
 #include <stdio.h>
+#include <string.h>
 #endif
 #include <afsconfig.h>
 #include <afs/param.h>
-#include <des.h>
+#include "des.h"
 #include "des_prototypes.h"
 
 
@@ -66,18 +67,18 @@
 /*
     des_cblock *in;             * >= length bytes of input text *
     des_cblock *out;            * >= length bytes of output text *
-    register afs_int32 length;  * in bytes *
+    afs_int32 length;  * in bytes *
     int encrypt;                * 0 ==> decrypt, else encrypt *
     des_key_schedule key;       * precomputed key schedule *
     des_cblock *iv;             * 8 bytes of ivec *
 */
 afs_int32
-des_pcbc_encrypt(void * in, void * out, register afs_int32 length,
+des_pcbc_encrypt(void * in, void * out, afs_int32 length,
 		 des_key_schedule key, des_cblock * iv, int encrypt)
 {
-    register afs_uint32 *input = (afs_uint32 *) in;
-    register afs_uint32 *output = (afs_uint32 *) out;
-    register afs_uint32 *ivec = (afs_uint32 *) iv;
+    afs_uint32 *input = (afs_uint32 *) in;
+    afs_uint32 *output = (afs_uint32 *) out;
+    afs_uint32 *ivec = (afs_uint32 *) iv;
 
     afs_uint32 i, j;
     afs_uint32 t_input[2];
@@ -87,7 +88,7 @@ des_pcbc_encrypt(void * in, void * out, register afs_int32 length,
 
     if (encrypt) {
 #ifdef MUSTALIGN
-	if ((afs_int32) ivec & 3) {
+	if (afs_pointer_to_int(ivec) & 3) {
 	    memcpy((char *)&xor_0, (char *)ivec++, sizeof(xor_0));
 	    memcpy((char *)&xor_1, (char *)ivec, sizeof(xor_1));
 	} else
@@ -100,7 +101,7 @@ des_pcbc_encrypt(void * in, void * out, register afs_int32 length,
 	for (i = 0; length > 0; i++, length -= 8) {
 	    /* get input */
 #ifdef MUSTALIGN
-	    if ((afs_int32) input & 3) {
+	    if (afs_pointer_to_int(input) & 3) {
 		memcpy((char *)&t_input[0], (char *)input,
 		       sizeof(t_input[0]));
 		memcpy((char *)&t_input[1], (char *)(input + 1),
@@ -133,7 +134,7 @@ des_pcbc_encrypt(void * in, void * out, register afs_int32 length,
 	     * case both input and output are the same space.
 	     */
 #ifdef MUSTALIGN
-	    if ((afs_int32) input & 3) {
+	    if (afs_pointer_to_int(input) & 3) {
 		memcpy((char *)&xor_0, (char *)input++, sizeof(xor_0));
 		xor_0 ^= t_output[0];
 		memcpy((char *)&xor_1, (char *)input++, sizeof(xor_1));
@@ -148,7 +149,7 @@ des_pcbc_encrypt(void * in, void * out, register afs_int32 length,
 
 	    /* copy temp output and save it for cbc */
 #ifdef MUSTALIGN
-	    if ((afs_int32) output & 3) {
+	    if (afs_pointer_to_int(output) & 3) {
 		memcpy((char *)output++, (char *)&t_output[0],
 		       sizeof(t_output[0]));
 		memcpy((char *)output++, (char *)&t_output[1],
@@ -177,7 +178,7 @@ des_pcbc_encrypt(void * in, void * out, register afs_int32 length,
     else {
 	/* decrypt */
 #ifdef MUSTALIGN
-	if ((afs_int32) ivec & 3) {
+	if (afs_pointer_to_int(ivec) & 3) {
 	    memcpy((char *)&xor_0, (char *)ivec++, sizeof(xor_0));
 	    memcpy((char *)&xor_1, (char *)ivec, sizeof(xor_1));
 	} else
@@ -190,7 +191,7 @@ des_pcbc_encrypt(void * in, void * out, register afs_int32 length,
 	for (i = 0; length > 0; i++, length -= 8) {
 	    /* get input */
 #ifdef MUSTALIGN
-	    if ((afs_int32) input & 3) {
+	    if (afs_pointer_to_int(input) & 3) {
 		memcpy((char *)&t_input[0], (char *)input++,
 		       sizeof(t_input[0]));
 		memcpy((char *)&t_input[1], (char *)input++,
@@ -222,7 +223,7 @@ des_pcbc_encrypt(void * in, void * out, register afs_int32 length,
 	    t_output[1] ^= xor_1;
 	    /* copy temp output */
 #ifdef MUSTALIGN
-	    if ((afs_int32) output & 3) {
+	    if (afs_pointer_to_int(output) & 3) {
 		memcpy((char *)output++, (char *)&t_output[0],
 		       sizeof(t_output[0]));
 		memcpy((char *)output++, (char *)&t_output[1],
