@@ -18,8 +18,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/des/test/verify.c,v 1.8 2003/12/07 22:49:25 jaltman Exp $");
 
 
 extern char *errmsg();
@@ -211,7 +209,7 @@ main(argc, argv)
 flip(array)
      char *array;
 {
-    register old, new, i, j;
+    int old, new, i, j;
     /* flips the bit order within each byte from 0 lsb to 0 msb */
     for (i = 0; i <= 7; i++) {
 	old = *array;
