@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -34,22 +34,26 @@
 #include <lwp.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
+#include <rx/rxstat.h>
 #include <rx/rxkad.h>
 #include <rx/rx_globals.h>
 #include <afs/cellconfig.h>
 #include <lock.h>
 #include <afs/afsutil.h>
 #include <afs/com_err.h>
+#include <afs/audit.h>
 #include <ubik.h>
 #include <sys/stat.h>
 #include "kauth.h"
+#include "kauth_internal.h"
 #include "kautils.h"
 #include "kaserver.h"
-
+#include "kadatabase.h"
+#include "kaprocs.h"
 
 struct kadstats dynamic_statistics;
 struct ubik_dbase *KA_dbase;
-afs_int32 myHost = 0;
+afs_uint32 myHost = 0;
 afs_int32 verbose_track = 1;
 afs_int32 krb4_cross = 0;
 afs_int32 rxBind = 0;
@@ -72,8 +76,7 @@ static int debugOutput;
 
 /* check whether caller is authorized to manage RX statistics */
 int
-KA_rxstat_userok(call)
-     struct rx_call *call;
+KA_rxstat_userok(struct rx_call *call)
 {
     return afsconf_SuperUser(KA_conf, call, NULL);
 }
@@ -92,7 +95,7 @@ es_Report(char *fmt, ...)
 }
 
 static void
-initialize_dstats()
+initialize_dstats(void)
 {
     memset(&dynamic_statistics, 0, sizeof(dynamic_statistics));
     dynamic_statistics.start_time = time(0);
@@ -100,10 +103,8 @@ initialize_dstats()
 }
 
 static int
-convert_cell_to_ubik(cellinfo, myHost, serverList)
-     struct afsconf_cell *cellinfo;
-     afs_int32 *myHost;
-     afs_int32 *serverList;
+convert_cell_to_ubik(struct afsconf_cell *cellinfo, afs_uint32 *myHost,
+		     afs_uint32 *serverList)
 {
     int i;
     char hostname[64];
@@ -116,7 +117,7 @@ convert_cell_to_ubik(cellinfo, myHost, serverList)
 	ViceLog(0, ("kaserver: couldn't get address of this host.\n"));
 	exit(1);
     }
-    memcpy(myHost, th->h_addr, sizeof(afs_int32));
+    memcpy(myHost, th->h_addr, sizeof(afs_uint32));
 
     for (i = 0; i < cellinfo->numServers; i++)
 	if (cellinfo->hostAddr[i].sin_addr.s_addr != *myHost) {
@@ -128,10 +129,7 @@ convert_cell_to_ubik(cellinfo, myHost, serverList)
 }
 
 static afs_int32
-kvno_admin_key(rock, kvno, key)
-     char *rock;
-     afs_int32 kvno;
-     struct ktc_encryptionKey *key;
+kvno_admin_key(void *rock, afs_int32 kvno, struct ktc_encryptionKey *key)
 {
     return ka_LookupKvno(0, KA_ADMIN_NAME, KA_ADMIN_INST, kvno, key);
 
@@ -147,13 +145,12 @@ kvno_admin_key(rock, kvno, key)
 
 #include "AFS_component_version_number.c"
 
-main(argc, argv)
-     int argc;
-     char *argv[];
+int
+main(int argc, char *argv[])
 {
     afs_int32 code;
     char *whoami = argv[0];
-    afs_int32 serverList[MAXSERVERS];
+    afs_uint32 serverList[MAXSERVERS];
     struct afsconf_cell cellinfo;
     char *cell;
     const char *cellservdb, *dbpath, *lclpath;
@@ -166,25 +163,18 @@ main(argc, argv)
     afs_int32 i;
     char clones[MAXHOSTSPERCELL];
     afs_uint32 host = ntohl(INADDR_ANY);
+    char *auditFileName = NULL;
 
     struct rx_service *tservice;
     struct rx_securityClass *sca[1];
     struct rx_securityClass *scm[3];
 
-    extern int afsconf_ClientAuthSecure();
-    extern int afsconf_ServerAuth();
-    extern int afsconf_CheckAuth();
-
     extern int rx_stackSize;
-    extern int KAA_ExecuteRequest();
-    extern int KAT_ExecuteRequest();
-    extern int KAM_ExecuteRequest();
-    extern int RXSTATS_ExecuteRequest();
 
 #ifdef	AFS_AIX32_ENV
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
@@ -200,10 +190,10 @@ main(argc, argv)
 
     if (argc == 0) {
       usage:
-	printf("Usage: kaserver [-noAuth] [-fastKeys] [-database <dbpath>] "
-	       "[-auditlog <log path>] [-rxbind] "
-	       "[-localfiles <lclpath>] [-minhours <n>] [-servers <serverlist>] "
-	       "[-crossrealm]"
+	printf("Usage: kaserver [-noAuth] [-database <dbpath>] "
+	       "[-auditlog <log path>] [-audit-interface <file|sysvmq>] "
+	       "[-rxbind] [-localfiles <lclpath>] [-minhours <n>] "
+	       "[-servers <serverlist>] [-crossrealm] "
 	       /*" [-enable_peer_stats] [-enable_process_stats] " */
 	       "[-help]\n");
 	exit(1);
@@ -247,9 +237,16 @@ main(argc, argv)
 		lclpath = dbpath;
 	}
 	else if (strncmp(arg, "-auditlog", arglen) == 0) {
-	    char *fileName = argv[++a];
-	    
-	    osi_audit_file(fileName);
+	    auditFileName = argv[++a];
+
+	} else if (strncmp(arg, "-audit-interface", arglen) == 0) {
+	    char *interface = argv[++a];
+
+	    if (osi_audit_interface(interface)) {
+		printf("Invalid audit interface '%s'\n", interface);
+		exit(1);
+	    }
+
 	} else if (strcmp(arg, "-localfiles") == 0)
 	    lclpath = argv[++a];
 	else if (strcmp(arg, "-servers") == 0)
@@ -291,7 +288,12 @@ main(argc, argv)
 	    goto usage;
 	}
     }
-    if (code = ka_CellConfig(cellservdb))
+
+    if (auditFileName) {
+	osi_audit_file(auditFileName);
+    }
+
+    if ((code = ka_CellConfig(cellservdb)))
 	goto abort;
     cell = ka_LocalCell();
     KA_conf = afsconf_Open(cellservdb);
@@ -306,7 +308,7 @@ main(argc, argv)
 #else
     /* NT & HPUX do not have dbm package support. So we can only do some
      * text logging. So open the AuthLog file for logging and redirect
-     * stdin and stdout to it 
+     * stdin and stdout to it
      */
     OpenLog(AFSDIR_SERVER_KALOG_FILEPATH);
     SetupLogSignals();
@@ -321,9 +323,9 @@ main(argc, argv)
 
     code =
 	afsconf_GetExtendedCellInfo(KA_conf, cell, AFSCONF_KAUTHSERVICE,
-				    &cellinfo, &clones);
+				    &cellinfo, clones);
     if (servers) {
-	if (code = ubik_ParseServerList(argc, argv, &myHost, serverList)) {
+	if ((code = ubik_ParseServerList(argc, argv, &myHost, serverList))) {
 	    afs_com_err(whoami, code, "Couldn't parse server list");
 	    exit(1);
 	}
@@ -363,16 +365,14 @@ main(argc, argv)
 
     if (rxBind) {
 	afs_int32 ccode;
-#ifndef AFS_NT40_ENV
-        if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
+        if (AFSDIR_SERVER_NETRESTRICT_FILEPATH ||
             AFSDIR_SERVER_NETINFO_FILEPATH) {
             char reason[1024];
             ccode = parseNetFiles(SHostAddrs, NULL, NULL,
                                            ADDRSPERSITE, reason,
                                            AFSDIR_SERVER_NETINFO_FILEPATH,
                                            AFSDIR_SERVER_NETRESTRICT_FILEPATH);
-        } else 
-#endif	
+        } else
 	{
             ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
         }
@@ -402,7 +402,7 @@ main(argc, argv)
     rx_SetNoJumbo();
 
     tservice =
-	rx_NewServiceHost(host, 0, KA_AUTHENTICATION_SERVICE, 
+	rx_NewServiceHost(host, 0, KA_AUTHENTICATION_SERVICE,
 			  "AuthenticationService", sca, 1, KAA_ExecuteRequest);
     if (tservice == (struct rx_service *)0) {
 	ViceLog(0, ("Could not create Authentication rx service\n"));
@@ -411,7 +411,7 @@ main(argc, argv)
     rx_SetMinProcs(tservice, 1);
     rx_SetMaxProcs(tservice, 1);
 
-    
+
     tservice =
 	rx_NewServiceHost(host, 0, KA_TICKET_GRANTING_SERVICE, "TicketGrantingService",
 		      sca, 1, KAT_ExecuteRequest);
@@ -457,7 +457,7 @@ main(argc, argv)
     if (init_kaprocs(lclpath, initFlags))
 	return -1;
 
-    if (code = init_krb_udp()) {
+    if ((code = init_krb_udp())) {
 	ViceLog(0,
 		("Failed to initialize UDP interface; code = %d.\n", code));
 	ViceLog(0, ("Running without UDP access.\n"));
