@@ -18,13 +18,12 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/kauth/test/test_interim_ktc.c,v 1.7.2.1 2007/04/10 18:43:43 shadow Exp $");
 
 #include <afs/stds.h>
 #include <afs/com_err.h>
 #include <afs/cellconfig.h>
 #include <afs/auth.h>
+#include <afs/afs_consts.h>
 #include "kautils.h"
 
 
@@ -298,21 +297,20 @@ CheckAuth2(server)
 /* Stolen from the "fs" command. */
 
 #define MAXNAME 100
-#define	MAXSIZE	2048
 
 static void
 ListCellsCmd()
 {
-    register long code;
+    long code;
     long i, j;
     char *tcp;
     long clear;
-    char space[MAXSIZE];
+    char space[AFS_PIOCTL_MAXSIZE];
     struct ViceIoctl blob;
 
     for (i = 0; i < 1000; i++) {
 	char *cellname;
-	blob.out_size = MAXSIZE;
+	blob.out_size = AFS_PIOCTL_MAXSIZE;
 	blob.in_size = sizeof(long);
 	blob.in = space;
 	blob.out = space;
@@ -368,8 +366,8 @@ char *
 AclToString(acl)
      struct Acl *acl;
 {
-    static char mydata[MAXSIZE];
-    char tstring[MAXSIZE];
+    static char mydata[AFS_PIOCTL_MAXSIZE];
+    char tstring[AFS_PIOCTL_MAXSIZE];
     struct AclEntry *tp;
     sprintf(mydata, "%d\n%d\n", acl->nplus, acl->nminus);
     for (tp = acl->pluslist; tp; tp = tp->next) {
@@ -385,7 +383,7 @@ AclToString(acl)
 
 char *
 SkipLine(astr)
-     register char *astr;
+     char *astr;
 {
     while (*astr != '\n')
 	astr++;
@@ -450,7 +448,7 @@ ParseAcl(astr)
 ZapList(alist)
      struct AclEntry *alist;
 {
-    register struct AclEntry *tp, *np;
+    struct AclEntry *tp, *np;
     for (tp = alist; tp; tp = np) {
 	np = tp->next;
 	free(tp);
@@ -484,12 +482,12 @@ static int
 AddTester(pathname)
      char *pathname;
 {
-    register long code;
+    long code;
     struct ViceIoctl blob;
     struct Acl *al;
-    char space[MAXSIZE];
+    char space[AFS_PIOCTL_MAXSIZE];
 
-    blob.out_size = MAXSIZE;
+    blob.out_size = AFS_PIOCTL_MAXSIZE;
     blob.in_size = 0;
     blob.out = space;
     code = pioctl(pathname, VIOCGETAL, &blob, 1);
