@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -20,6 +20,7 @@
 #include <stdio.h>		/* sprintf */
 #include <malloc.h>
 #include <afs/kautils.h>
+#include <afs/cm.h>
 #include <afs/cm_config.h>
 #include <afs/krb.h>
 #include <afs/krb_prot.h>
@@ -105,14 +106,13 @@ ka_UserAuthenticateGeneral2(afs_int32 flags, char *name, char *instance,
     if (lifetime == 0)
 	lifetime = MAXKTCTICKETLIFETIME;
 
-    code = cm_SearchCellFile(realm, fullRealm, ka_AddHostProc, NULL);
-
-#ifdef AFS_AFSDB_ENV
+    code = cm_SearchCellRegistry(1, realm, fullRealm, NULL, ka_AddHostProc, NULL);
+    if (code && code != CM_ERROR_FORCE_DNS_LOOKUP)
+        code = cm_SearchCellFile(realm, fullRealm, ka_AddHostProc, NULL);
     if (code) {
 	code =
 	    cm_SearchCellByDNS(realm, fullRealm, &ttl, ka_AddHostProc, NULL);
     }
-#endif
     if (code) {
 	*reasonP = "specified realm is unknown";
 	return (code);
@@ -181,7 +181,7 @@ ka_UserAuthenticateGeneral2(afs_int32 flags, char *name, char *instance,
 /*
  * krb_get_in_tkt()
  *
- * This code is descended from kerberos files krb_get_in_tkt.c and 
+ * This code is descended from kerberos files krb_get_in_tkt.c and
  * send_to_kdc.c, and one.c.
  */
 
@@ -289,7 +289,7 @@ KTEXT pkt_cipher(KTEXT packet);
  */
 
 /*
- * Check_response is a support routine for krb_get_in_tkt. 
+ * Check_response is a support routine for krb_get_in_tkt.
  *
  * Check the response with the supplied key. If the key is apparently
  * wrong, return INTK_BADPW, otherwise zero.
@@ -713,7 +713,7 @@ send_to_kdc(pkt, rpkt)
     struct sockaddr_in to;
     int timeAvail, timePerIter, numIters;
 
-    memset((char *)&to, 0, sizeof(to));
+    memset(&to, 0, sizeof(to));
     if ((f = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
 	if (krb_debug)
 	    fprintf(stderr, "%s: Can't open socket\n", prog);
