@@ -17,7 +17,6 @@
 #include <string.h>
 #include <afs/afs_Admin.h>
 #include "afs_AdminInternal.h"
-#include "afs_utilAdmin.h"
 #include <afs/pthread_glock.h>
 #include <afs/cellconfig.h>
 #include <afs/dirpath.h>
@@ -28,11 +27,12 @@
 #include <afs/pterror.h>
 #include <afs/bnode.h>
 #include <afs/volser.h>
-#include <afs/afsint.h>
+#include <afs/afscbint.h>
 #include <rx/rx.h>
 #include <rx/rxstat.h>
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
+# include <winsock2.h>
+# include <afs/krb5_nt.h>
 #else
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -40,6 +40,7 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 #endif
+#include "afs_utilAdmin.h"
 
 /*
  * AIX 4.2 has PTHREAD_CREATE_UNDETACHED and not PTHREAD_CREATE_JOINABLE
@@ -79,6 +80,9 @@ init_once(void)
     initialize_AU_error_table();
     initialize_AV_error_table();
     initialize_VOLS_error_table();
+#ifdef AFS_KRB5_ERROR_ENV
+    initialize_krb5();
+#endif
     error_init_done = 1;
 }
 
@@ -103,6 +107,12 @@ util_AdminErrorCodeTranslate(afs_status_t errorCode, int langId,
 	pthread_once(&error_init_once, init_once);
     code = (afs_int32) errorCode;
     *errorTextP = afs_error_message(code);
+#ifdef AFS_KRB5_ERROR_ENV
+    if (strncmp(*errorTextP, "unknown", strlen("unknown")) == 0) {
+        const char *msg = fetch_krb5_error_message(NULL, code);
+        *errorTextP = msg ? msg : error_message(code);
+    }
+#endif
     rc = 1;
 
   fail_util_AdminErrorCodeTranslate:
@@ -1319,7 +1329,11 @@ DestroyRPCStats(void *rpc_specific, afs_status_p st)
  */
 
 int ADMINAPI
-util_RPCStatsGetBegin(struct rx_connection *conn, int (*rpc) (),
+util_RPCStatsGetBegin(struct rx_connection *conn, 
+		      int (*rpc) (struct rx_connection *,
+				  afs_uint32, afs_uint32 *,
+				  afs_uint32 *, afs_uint32 *,
+				  afs_uint32 *, struct rpcStats *),
 		      void **iterationIdP, afs_status_p st)
 {
     int rc = 0;
@@ -1513,7 +1527,9 @@ util_RPCStatsGetDone(const void *iterationId, afs_status_p st)
  */
 
 int ADMINAPI
-util_RPCStatsStateGet(struct rx_connection *conn, int (*rpc) (),
+util_RPCStatsStateGet(struct rx_connection *conn, 
+		      int (*rpc) (struct rx_connection *,
+			      	  afs_RPCStatsState_p),
 		      afs_RPCStatsState_p state, afs_status_p st)
 {
     int rc = 0;
@@ -1569,7 +1585,8 @@ util_RPCStatsStateGet(struct rx_connection *conn, int (*rpc) (),
  */
 
 int ADMINAPI
-util_RPCStatsStateEnable(struct rx_connection *conn, int (*rpc) (),
+util_RPCStatsStateEnable(struct rx_connection *conn, 
+		         int (*rpc) (struct rx_connection *),
 			 afs_status_p st)
 {
     int rc = 0;
@@ -1620,7 +1637,8 @@ util_RPCStatsStateEnable(struct rx_connection *conn, int (*rpc) (),
  */
 
 int ADMINAPI
-util_RPCStatsStateDisable(struct rx_connection *conn, int (*rpc) (),
+util_RPCStatsStateDisable(struct rx_connection *conn, 
+			  int (*rpc) (struct rx_connection *),
 			  afs_status_p st)
 {
     int rc = 0;
@@ -1674,7 +1692,9 @@ util_RPCStatsStateDisable(struct rx_connection *conn, int (*rpc) (),
  */
 
 int ADMINAPI
-util_RPCStatsClear(struct rx_connection *conn, int (*rpc) (),
+util_RPCStatsClear(struct rx_connection *conn, 
+		   int (*rpc) (struct rx_connection *,
+			       afs_RPCStatsClearFlag_t),
 		   afs_RPCStatsClearFlag_t flag, afs_status_p st)
 {
     int rc = 0;
@@ -1988,18 +2008,28 @@ ListCellsRPC(void *rpc_specific, int slot, int *last_item,
     afs_status_t tst = 0;
     cm_list_cell_get_p t = (cm_list_cell_get_p) rpc_specific;
     char *name;
+    serverList sl;
+    unsigned int n;
 
     /*
      * Get the next entry in the CellServDB.
      */
     name = t->cell[slot].cellname;
+    sl.serverList_len = 0;
+    sl.serverList_val = NULL;
+    memset(t->cell[slot].serverAddr, 0, sizeof(afs_int32)*UTIL_MAX_CELL_HOSTS);
     tst =
-	RXAFSCB_GetCellServDB(t->conn, t->index, &name,
-			      t->cell[slot].serverAddr);
+	RXAFSCB_GetCellServDB(t->conn, t->index, &name, &sl);
     if (tst) {
 	goto fail_ListCellsRPC;
     }
     strcpy(t->cell[slot].cellname, name);
+    if (sl.serverList_val) {
+        for (n=0; n<sl.serverList_len && n<UTIL_MAX_CELL_HOSTS; n++) {
+            t->cell[slot].serverAddr[n] = sl.serverList_val[n];
+        }
+        xdr_free((xdrproc_t) xdr_serverList, &sl);
+    }
 
     /*
      * See if we've processed all the entries
@@ -2280,7 +2310,7 @@ util_CMClientConfig(struct rx_connection *conn, afs_ClientConfig_p config,
 {
     int rc = 0;
     afs_status_t tst = 0;
-    afs_int32 allocbytes;
+    afs_uint32 allocbytes;
     struct cacheConfig tconfig;
 
     if (conn == NULL) {
