@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -20,6 +20,9 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
 #include <time.h>
 
 /* allocate externs here */
@@ -98,24 +101,24 @@ extern char PRE_Block;		/* from preempt.c */
     ;									\
 } while (0)
 #endif
-
+
 static void Dispatcher(void);
 static void Create_Process_Part2(void);
 static void Exit_LWP(void);
 static afs_int32 Initialize_Stack(char *stackptr, int stacksize);
-static int Stack_Used(register char *stackptr, int stacksize);
+static int Stack_Used(char *stackptr, int stacksize);
 
 static void Abort_LWP(char *msg);
 static void Overflow_Complain(void);
 static void Initialize_PCB(PROCESS temp, int priority, char *stack,
-			   int stacksize, void *(*ep)(void *), void *parm, 
+			   int stacksize, void *(*ep)(void *), void *parm,
 			   char *name);
 static void Dispose_of_Dead_PCB(PROCESS cur);
 static void Free_PCB(PROCESS pid);
 static int Internal_Signal(void *event);
 static int purge_dead_pcbs(void);
 static int LWP_MwaitProcess(int wcount, void *evlist[]);
-	
+
 
 #define MAX_PRIORITIES	(LWP_MAX_PRIORITY+1)
 
@@ -123,8 +126,8 @@ struct QUEUE {
     PROCESS head;
     int count;
 } runnable[MAX_PRIORITIES], blocked, qwaiting;
-/* Invariant for runnable queues: The head of each queue points to the 
- * currently running process if it is in that queue, or it points to the 
+/* Invariant for runnable queues: The head of each queue points to the
+ * currently running process if it is in that queue, or it points to the
  * next process in that queue that should run. */
 
 /* Offset of stack field within pcb -- used by stack checking stuff */
@@ -148,7 +151,7 @@ int lwp_nextindex;
 int lwp_MinStackSize = 0;
 
 static int
-lwp_remove(register PROCESS p, register struct QUEUE *q)
+lwp_remove(PROCESS p, struct QUEUE *q)
 {
     /* Special test for only element on queue */
     if (q->count == 1)
@@ -167,7 +170,7 @@ lwp_remove(register PROCESS p, register struct QUEUE *q)
 }
 
 static int
-insert(register PROCESS p, register struct QUEUE *q)
+insert(PROCESS p, struct QUEUE *q)
 {
     if (q->head == NULL) {	/* Queue is empty */
 	q->head = p;
@@ -195,14 +198,14 @@ move(PROCESS p, struct QUEUE *from, struct QUEUE *to)
 /* Iterator macro */
 #define for_all_elts(var, q, body)\
 	{\
-	    register PROCESS var, _NEXT_;\
-	    register int _I_;\
+	    PROCESS var, _NEXT_;\
+	    int _I_;\
 	    for (_I_=q.count, var = q.head; _I_>0; _I_--, var=_NEXT_) {\
 		_NEXT_ = var -> next;\
 		body\
 	    }\
 	}
-
+
 /*									    */
 /*****************************************************************************\
 * 									      *
@@ -237,7 +240,7 @@ static struct lwp_ctl *lwp_init = 0;
 int
 LWP_QWait(void)
 {
-    register PROCESS tp;
+    PROCESS tp;
     (tp = lwp_cpptr)->status = QWAITING;
     move(tp, &runnable[tp->priority], &qwaiting);
     Set_LWP_RC();
@@ -245,7 +248,7 @@ LWP_QWait(void)
 }
 
 int
-LWP_QSignal(register PROCESS pid)
+LWP_QSignal(PROCESS pid)
 {
     if (pid->status == QWAITING) {
 	pid->status = READY;
@@ -257,7 +260,7 @@ LWP_QSignal(register PROCESS pid)
 
 #ifdef	AFS_AIX32_ENV
 char *
-reserveFromStack(register afs_int32 size)
+reserveFromStack(afs_int32 size)
 {
     char *x;
     x = alloca(size);
@@ -311,8 +314,8 @@ LWP_CreateProcess(void *(*ep) (void *), int stacksize, int priority, void *parm,
 #ifdef	AFS_AIX32_ENV
 	if (!stackptr) {
 	    /*
-	     * The following signal action for AIX is necessary so that in case of a 
-	     * crash (i.e. core is generated) we can include the user's data section 
+	     * The following signal action for AIX is necessary so that in case of a
+	     * crash (i.e. core is generated) we can include the user's data section
 	     * in the core dump. Unfortunately, by default, only a partial core is
 	     * generated which, in many cases, isn't too useful.
 	     *
@@ -330,7 +333,7 @@ LWP_CreateProcess(void *(*ep) (void *), int stacksize, int priority, void *parm,
 	    /*
 	     * First we need to increase the default resource limits,
 	     * if necessary, so that we can guarantee that we have the
-	     * resources to create the core file, but we can't always 
+	     * resources to create the core file, but we can't always
 	     * do it as an ordinary user.
 	     */
 	    if (!geteuid()) {
@@ -373,7 +376,7 @@ LWP_CreateProcess(void *(*ep) (void *), int stacksize, int priority, void *parm,
 	Initialize_PCB(temp, priority, stackmemory, stacksize, ep, parm, name);
 	insert(temp, &runnable[priority]);
 	temp2 = lwp_cpptr;
-#ifndef AFS_ARM_LINUX20_ENV
+#if !defined(AFS_ARM_LINUX20_ENV) && !defined(AFS_ARM_DARWIN_ENV)
 	if (PRE_Block != 0)
 	    Abort_LWP("PRE_Block not 0");
 
@@ -468,7 +471,7 @@ LWP_CreateProcess2(void *(*ep) (void *), int stacksize, int priority, void *parm
 	Initialize_PCB(temp, priority, stackptr, stacksize, ep, parm, name);
 	insert(temp, &runnable[priority]);
 	temp2 = lwp_cpptr;
-#ifndef AFS_ARM_LINUX20_ENV
+#if !defined(AFS_ARM_LINUX20_ENV) && !defined(AFS_ARM_DARWIN_ENV)
 	if (PRE_Block != 0)
 	    Abort_LWP("PRE_Block not 0");
 
@@ -570,7 +573,7 @@ int
 Dump_Processes(void)
 {
     if (lwp_init) {
-	register int i;
+	int i;
 	for (i = 0; i < MAX_PRIORITIES; i++)
 	    for_all_elts(x, runnable[i], {
 			 printf("[Priority %d]\n", i);
@@ -605,7 +608,7 @@ LWP_InitializeProcessSupport(int priority, PROCESS * pid)
 {
     PROCESS temp;
     struct lwp_pcb dummy;
-    register int i;
+    int i;
     char *value;
 
     Debug(0, ("Entered LWP_InitializeProcessSupport"));
@@ -670,7 +673,7 @@ LWP_INTERNALSIGNAL(void *event, int yield)
 int
 LWP_TerminateProcessSupport(void)
 {				/* terminate all LWP support */
-    register int i;
+    int i;
 
     Debug(0, ("Entered Terminate_Process_Support"));
     if (lwp_init == NULL)
@@ -708,7 +711,7 @@ LWP_WaitProcess(void *event)
 int
 LWP_MwaitProcess(int wcount, void *evlist[])
 {				/* wait on m of n events */
-    register int ecount, i;
+    int ecount, i;
 
 
     Debug(0, ("Entered Mwait_Process [waitcnt = %d]", wcount));
@@ -767,7 +770,7 @@ LWP_StackUsed(PROCESS pid, int *maxa, int *used)
 	return LWP_NO_STACK;
     return LWP_SUCCESS;
 }
-
+
 /*
  *  The following functions are strictly
  *  INTERNAL to the LWP support package.
@@ -805,13 +808,13 @@ Create_Process_Part2(void)
 }
 
 static int
-Delete_PCB(register PROCESS pid)
+Delete_PCB(PROCESS pid)
 {				/* remove a PCB from the process list */
     Debug(4, ("Entered Delete_PCB"));
     lwp_remove(pid,
 	       (pid->blockflag || pid->status == WAITING
 		|| pid->status ==
-		DESTROYED ? &blocked : 
+		DESTROYED ? &blocked :
 		(pid->status == QWAITING) ? &qwaiting :
 		&runnable[pid->priority]));
     LWPANCHOR.processcnt--;
@@ -883,7 +886,7 @@ int LWP_TraceProcesses = 0;
 static void
 Dispatcher(void)
 {				/* Lightweight process dispatcher */
-    register int i;
+    int i;
 #ifdef DEBUG
     static int dispatch_count = 0;
 
@@ -931,9 +934,9 @@ Dispatcher(void)
 #endif
 	printf("stackcheck = %u: stack = %u \n", lwp_cpptr->stackcheck,
 	       *(int *)lwp_cpptr->stack);
-	printf("topstack = 0x%x: stackptr = 0x%x: stacksize = 0x%x\n",
-	       (unsigned int)lwp_cpptr->context.topstack, 
-	       (unsigned int)lwp_cpptr->stack,
+	printf("topstack = 0x%" AFS_PTR_FMT ": stackptr = 0x%" AFS_PTR_FMT ": stacksize = 0x%x\n",
+	       (void *)(uintptr_t)lwp_cpptr->context.topstack,
+	       (void *)(uintptr_t)lwp_cpptr->stack,
 	       lwp_cpptr->stacksize);
 
 	switch (lwp_overflowAction) {
@@ -967,14 +970,14 @@ Dispatcher(void)
 	printf("Dispatch %d [PCB at 0x%x] \"%s\"\n", ++dispatch_count,
 	       runnable[i].head, runnable[i].head->name);
 #endif
-#ifndef AFS_ARM_LINUX20_ENV
+#if !defined(AFS_ARM_LINUX20_ENV) && !defined(AFS_ARM_DARWIN_ENV)
     if (PRE_Block != 1)
 	Abort_LWP("PRE_Block not 1");
 #endif
     lwp_cpptr = runnable[i].head;
 
     returnto(&lwp_cpptr->context);
-    
+
     return; /* not reachable */
 }
 
@@ -1035,7 +1038,7 @@ static void
 Initialize_PCB(PROCESS temp, int priority, char *stack, int stacksize,
 	       void *(*ep) (void *), void *parm, char *name)
 {
-    register int i = 0;
+    int i = 0;
 
     Debug(4, ("Entered Initialize_PCB"));
     if (name != NULL)
@@ -1071,10 +1074,10 @@ Initialize_PCB(PROCESS temp, int priority, char *stack, int stacksize,
 }
 
 static int
-Internal_Signal(register void *event)
+Internal_Signal(void *event)
 {
     int rc = LWP_ENOWAIT;
-    register int i;
+    int i;
 
     Debug(0, ("Entered Internal_Signal [event id 0x%x]", event));
     if (!lwp_init)
@@ -1102,7 +1105,7 @@ Internal_Signal(register void *event)
 static afs_int32
 Initialize_Stack(char *stackptr, int stacksize)
 {
-    register int i;
+    int i;
 
     Debug(4, ("Entered Initialize_Stack"));
     if (lwp_stackUseEnabled)
@@ -1118,9 +1121,9 @@ Initialize_Stack(char *stackptr, int stacksize)
 }
 
 static int
-Stack_Used(register char *stackptr, int stacksize)
+Stack_Used(char *stackptr, int stacksize)
 {
-    register int i;
+    int i;
 
 #if defined(__hp9000s800) || defined(AFS_PARISC_LINUX24_ENV)
     if (*(afs_int32 *) (stackptr + stacksize - 4) == STACKMAGIC)
@@ -1151,15 +1154,15 @@ LWP_NewRock(int Tag, char *Value)
      * LWP_SUCCESS      Rock did not exist and a new one was used
      * LWP_EBADROCK     Rock already exists.
      * LWP_ENOROCKS     All rocks are in use.
-     * 
+     *
      * From the above semantics, you can only set a rock value once.  This is specifically
      * to prevent multiple users of the LWP package from accidentally using the same Tag
      * value and clobbering others.  You can always use one level of indirection to obtain
      * a rock whose contents can change.
      */
 {
-    register int i;
-    register struct rock *ra;	/* rock array */
+    int i;
+    struct rock *ra;	/* rock array */
 
     ra = lwp_cpptr->lwp_rlist;
 
@@ -1185,8 +1188,8 @@ LWP_GetRock(int Tag, char **Value)
      * LWP_EBADROCK     rock specified does not exist
      */
 {
-    register int i;
-    register struct rock *ra;
+    int i;
+    struct rock *ra;
 
     ra = lwp_cpptr->lwp_rlist;
 
@@ -1351,13 +1354,15 @@ static PROCESS
 lwp_alloc_process(char *name, pthread_startroutine_t ep, pthread_addr_t arg)
 {
     PROCESS lp;
-    assert(lp = (PROCESS) malloc(sizeof(*lp)));
-    memset((char *)lp, 0, sizeof(*lp));
+    lp = (PROCESS) malloc(sizeof(*lp));
+    assert(lp);
+    memset(lp, 0, sizeof(*lp));
     if (!name) {
 	char temp[100];
 	static procnum;
 	sprintf(temp, "unnamed_process_%04d", ++procnum);
-	assert(name = (char *)malloc(strlen(temp) + 1));
+	name = (char *)malloc(strlen(temp) + 1);
+	assert(name);
 	strcpy(name, temp);
     }
     lp->name = name;
@@ -1374,7 +1379,8 @@ lwp_thread_process(PROCESS lp)
 {
     lp->next = lwp_process_list;
     lwp_process_list = lp;
-    assert(!pthread_setspecific(lwp_process_key, (pthread_addr_t) lp));
+    if (pthread_setspecific(lwp_process_key, (pthread_addr_t) lp) != 0)
+	assert(0);
 }
 
 /* The top-level routine used as entry point to explicitly created LWP
@@ -1385,10 +1391,10 @@ lwp_top_level(pthread_addr_t argp)
 {
     PROCESS lp = (PROCESS) argp;
 
-    assert(!pthread_mutex_lock(&lwp_mutex));
+    MUTEX_ENTER(&lwp_mutex);
     lwp_thread_process(lp);
     (lp->ep) (lp->arg);
-    assert(!pthread_mutex_unlock(&lwp_mutex));
+    MUTEX_EXIT(&lwp_mutex);
     /* Should cleanup state */
 }
 
@@ -1405,10 +1411,13 @@ LWP_CreateProcess(pthread_startroutine_t ep, int stacksize, int priority,
     if (!cmalwp_pri_inrange(priority))
 	return LWP_EBADPRI;
 #endif
-    assert(!pthread_attr_create(&attr));
-    assert(!pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED));
+    if (pthread_attr_create(&attr))
+	assert(0);
+    if (pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED))
+	assert(0);
     if (stacksize)
-	assert(!pthread_attr_setstacksize(&attr, stacksize));
+	if (pthread_attr_setstacksize(&attr, stacksize))
+	    assert(0);
 #ifndef BDE_THREADS
     (void)pthread_attr_setinheritsched(&attr, PTHREAD_DEFAULT_SCHED);
     (void)pthread_attr_setsched(&attr, SCHED_FIFO);
@@ -1420,14 +1429,16 @@ LWP_CreateProcess(pthread_startroutine_t ep, int stacksize, int priority,
 			   (pthread_addr_t) parm);
 
     /* allow new thread to run if higher priority */
-    assert(!pthread_mutex_unlock(&lwp_mutex));
+    MUTEX_EXIT(&lwp_mutex);
     /* process is only added to active list after first time it runs (it adds itself) */
     status =
 	pthread_create(&lp->handle, attr,
 		       (pthread_startroutine_t) lwp_top_level,
 		       (pthread_addr_t) lp);
-    assert(!pthread_attr_delete(&attr));
-    assert(!pthread_mutex_lock(&lwp_mutex));
+    if (pthread_attr_delete(&attr))
+	assert(0);
+    if (pthread_mutex_lock(&lwp_mutex))
+	assert(0);
     if (status != 0) {
 	free(lp);
 	return LWP_ENOMEM;
@@ -1440,14 +1451,16 @@ PROCESS
 LWP_ActiveProcess(void)
 {				/* returns pid of current process */
     PROCESS pid;
-    assert(!pthread_getspecific(lwp_process_key, (pthread_addr_t *) & pid));
+    if (pthread_getspecific(lwp_process_key, (pthread_addr_t *) & pid))
+	assert(0);
     return pid;
 }
 
 int
 LWP_CurrentProcess(PROCESS * pid)
 {				/* get pid of current process */
-    assert(!pthread_getspecific(lwp_process_key, (pthread_addr_t *) pid));
+    if (pthread_getspecific(lwp_process_key, (pthread_addr_t *) pid))
+	assert(0);
     return LWP_SUCCESS;
 }
 
@@ -1460,9 +1473,9 @@ LWP_DestroyProcess(PROCESS pid)
 int
 LWP_DispatchProcess(void)
 {				/* explicit voluntary preemption */
-    assert(!pthread_mutex_unlock(&lwp_mutex));
+    MUTEX_EXIT(&lwp_mutex);
     pthread_yield();
-    assert(!pthread_mutex_lock(&lwp_mutex));
+    MUTEX_ENTER(&lwp_mutex);
     return LWP_SUCCESS;
 }
 
@@ -1491,8 +1504,9 @@ LWP_InitializeProcessSupport(int priority, PROCESS * pid)
 
     /* Create pthread key to associate LWP process descriptor with each
      * LWP-created thread */
-    assert(!pthread_keycreate(&lwp_process_key, (pthread_destructor_t)
-			      lwp_process_key_destructor));
+    if (pthread_keycreate(&lwp_process_key, (pthread_destructor_t)
+			  lwp_process_key_destructor))
+	assert(0);
 
     lp = lwp_alloc_process("main process", main, 0);
     lp->handle = pthread_self();
@@ -1502,8 +1516,8 @@ LWP_InitializeProcessSupport(int priority, PROCESS * pid)
 			       cmalwp_lwppri_to_cmapri(priority));
 
 #endif
-    assert(pthread_mutex_init(&lwp_mutex, MUTEX_FAST_NP) == 0);
-    assert(pthread_mutex_lock(&lwp_mutex) == 0);
+    MUTEX_INIT(&lwp_mutex, "lwp", MUTEX_DEFAULT, 0);
+    MUTEX_ENTER(&lwp_mutex);
     initialized = 1;
     *pid = lp;
     return LWP_SUCCESS;
@@ -1539,7 +1553,8 @@ getevent(void *event)
 	assert(newp);
 	newp->next = hashtable[hashcode];
 	hashtable[hashcode] = newp;
-	assert(!pthread_cond_init(&newp->cond, ((pthread_condattr_t) 0)));
+	if (pthread_cond_init(&newp->cond, ((pthread_condattr_t) 0)))
+	    assert(0);
 	newp->seq = 0;
     }
     newp->event = event;
@@ -1561,7 +1576,8 @@ LWP_WaitProcess(void *event)
     ev = getevent(event);
     seq = ev->seq;
     while (seq == ev->seq) {
-	assert(pthread_cond_wait(&ev->cond, &lwp_mutex) == 0);
+	if (pthread_cond_wait(&ev->cond, &lwp_mutex))
+	    assert(0);
     }
     debugf(("%s: Woken up (%x)\n", lwp_process_string(), event));
     relevent(ev);
@@ -1584,7 +1600,8 @@ LWP_NoYieldSignal(void *event)
     ev = getevent(event);
     if (ev->refcount > 1) {
 	ev->seq++;
-	assert(pthread_cond_broadcast(&ev->cond) == 0);
+	if (pthread_cond_broadcast(&ev->cond))
+	    assert(0);
     }
     relevent(ev);
     return LWP_SUCCESS;
@@ -1600,10 +1617,10 @@ LWP_SignalProcess(void *event)
     ev = getevent(event);
     if (ev->refcount > 1) {
 	ev->seq++;
-	assert(!pthread_mutex_unlock(&lwp_mutex));
-	assert(!pthread_cond_broadcast(&ev->cond));
+	MUTEX_EXIT(&lwp_mutex);
+	CV_BROADCAST(&ev->cond);
 	pthread_yield();
-	assert(!pthread_mutex_lock(&lwp_mutex));
+	MUTEX_ENTER(&lwp_mutex);
     }
     relevent(ev);
     return LWP_SUCCESS;
