@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -54,10 +54,12 @@
 #include "ptint.h"
 #include "pterror.h"
 #include "ptserver.h"
+#include "ptuser.h"
+#include "display.h"
 
 struct prheader cheader;
 int fd;
-char *pr_dbaseName;
+const char *pr_dbaseName;
 char *whoami = "db_verify";
 #define UBIK_HEADERSIZE 64
 
@@ -83,7 +85,7 @@ printheader(struct prheader *h)
 }
 
 static afs_int32
-pr_Read(afs_int32 pos, char *buff, afs_int32 len)
+pr_Read(afs_int32 pos, void *buff, afs_int32 len)
 {
     afs_int32 code;
 
@@ -108,7 +110,7 @@ pr_Read(afs_int32 pos, char *buff, afs_int32 len)
  */
 
 afs_int32
-ReadHeader()
+ReadHeader(void)
 {
     afs_int32 code;
 
@@ -136,14 +138,14 @@ IDHash(afs_int32 x)
 }
 
 static afs_int32
-NameHash(register unsigned char *aname)
+NameHash(char *aname)
 {
     /* returns hash bucket for aname */
-    register unsigned int hash = 0;
-    register int i;
+    unsigned int hash = 0;
+    int i;
 /* stolen directly from the HashString function in the vol package */
     for (i = strlen(aname), aname += i - 1; i--; aname--)
-	hash = (hash * 31) + (*aname - 31);
+	hash = (hash * 31) + (*(unsigned char *)aname - 31);
     return (hash % HASHSIZE);
 }
 
@@ -209,8 +211,8 @@ readUbikHeader(struct misc_data *misc)
     /* now read the info */
     r = read(fd, &uheader, sizeof(uheader));
     if (r != sizeof(uheader)) {
-	printf("error: read of %d bytes failed: %d %d\n", sizeof(uheader), r,
-	       errno);
+	printf("error: read of %" AFS_SIZET_FMT " bytes failed: %d %d\n",
+	       sizeof(uheader), r, errno);
 	return (-1);
     }
 
@@ -400,7 +402,7 @@ WalkNextChain(char map[],		/* one byte per db entry */
     int noErrors = 1;
     int length;			/* length of chain */
 #if defined(SUPERGROUPS)
-    int sgcount;		/* number of sgentrys */
+    int sgcount = 0;		/* number of sgentrys */
     afs_int32 sghead;
 #define g (((struct prentryg *)e))
 #endif
@@ -410,7 +412,6 @@ WalkNextChain(char map[],		/* one byte per db entry */
 	eid = ntohl(e->id);
 	bit = MAP_CONT;
 #if defined(SUPERGROUPS)
-	sgcount = 0;
 	sghead = ntohl(g->next);
 #endif
 	for (i = 0; i < PRSIZE; i++) {
@@ -930,12 +931,12 @@ GC(char map[], struct misc_data *misc)
 	    id = ntohl(e.id);
 #if defined(SUPERGROUPS)
 	    if ((id != ANONYMOUSID)
-		&& ((refCount = idcount(&misc->idmap, id)) != ntohl(e.count))) 
+		&& ((refCount = idcount(&misc->idmap, id)) != ntohl(e.count)))
 #else
 	    if ((id >= misc->minId) && (id <= misc->maxId)
 		&& (id != ANONYMOUSID)
 		&& ((refCount = misc->idmap[id - misc->minId]) !=
-		    ntohl(e.count))) 
+		    ntohl(e.count)))
 #endif /* SUPERGROUPS */
 	      {
 		afs_int32 na;
@@ -1039,9 +1040,9 @@ DumpRecreate(char map[], struct misc_data *misc)
 		/* check for duplicate id.  This may still lead to duplicate
 		 * names. */
 #if defined(SUPERGROUPS)
-		if (idcount(&idmap, id)) 
+		if (idcount(&idmap, id))
 #else
-		if (idmap[id - misc->minId]) 
+		if (idmap[id - misc->minId])
 #endif
 		  {
 		    fprintf(stderr, "Skipping entry with duplicate id %di\n",
@@ -1150,9 +1151,9 @@ DumpRecreate(char map[], struct misc_data *misc)
 
 	    owner = ntohl(e.owner);
 #if defined(SUPERGROUPS)
-	    if (!idcount(&idmap, owner)) 
+	    if (!idcount(&idmap, owner))
 #else
-	    if (idmap[owner - misc->minId] == 0) 
+	    if (idmap[owner - misc->minId] == 0)
 #endif
 	      {
 		fprintf(stderr,
@@ -1179,9 +1180,6 @@ DumpRecreate(char map[], struct misc_data *misc)
 	    if ((id < 0) && (flags & PRGRP)) {
 		int count = 0;
 		afs_int32 na;
-#if defined(SUPERGROUPS)
-		afs_int32 ng;
-#endif
 		int i;
 		for (i = 0; i < PRSIZE; i++) {
 		    afs_int32 uid = ntohl(e.entries[i]);
@@ -1200,43 +1198,6 @@ DumpRecreate(char map[], struct misc_data *misc)
 				id);
 #endif
 		}
-#if defined(SUPERGROUPS)
-#define g	(*((struct prentryg *)&e))
-		ng = ntohl(g.nextsg);
-		for (i = 0; i < SGSIZE; i++) {
-		    afs_int32 uid = ntohl(g.supergroup[i]);
-		    if (uid == 0)
-			break;
-		    if (uid == PRBADID)
-			continue;
-		    fprintf(rc, "au %d %d\n", uid, id);
-		    count++;
-		}
-		while (ng) {
-		    struct prentry c;
-		    code = pr_Read(ng, (char *)&c, sizeof(c));
-		    if (code)
-			return code;
-
-		    if ((id == ntohl(c.id)) && (c.flags & htonl(PRCONT))) {
-			for (i = 0; i < COSIZE; i++) {
-			    afs_int32 uid = ntohl(c.entries[i]);
-			    if (uid == 0)
-				break;
-			    if (uid == PRBADID)
-				continue;
-			    fprintf(rc, "au %d %d\n", uid, id);
-			    count++;
-			}
-		    } else {
-			fprintf(stderr, "Skipping continuation block at %d\n",
-				ng);
-			break;
-		    }
-		    ng = ntohl(c.next);
-		}
-#undef g
-#endif /* SUPERGROUPS */
 		na = ntohl(e.next);
 		while (na) {
 		    struct prentry c;
@@ -1294,7 +1255,8 @@ CheckPrDatabase(struct misc_data *misc)	/* info & statistics */
     n = eof / sizeof(struct prentry);
     if ((eof < 0) || (n * sizeof(struct prentry) != eof)) {
 	code = PRDBBAD;
-	afs_com_err(whoami, code, "eof ptr no good: eof=%d, sizeof(prentry)=%d",
+	afs_com_err(whoami, code,
+		    "eof ptr no good: eof=%d, sizeof(prentry)=%" AFS_SIZET_FMT,
 		eof, sizeof(struct prentry));
       abort:
 	return code;
@@ -1421,7 +1383,7 @@ CheckPrDatabase(struct misc_data *misc)	/* info & statistics */
     free(map);
     return code;
 }
-    
+
 #include "AFS_component_version_number.c"
 
 int
@@ -1508,7 +1470,7 @@ void
 zeromap(struct idused *idmap)
 {
     while (idmap) {
-	memset((char *)idmap->idcount, 0, sizeof idmap->idcount);
+	memset(idmap->idcount, 0, sizeof idmap->idcount);
 	idmap = idmap->idnext;
     }
 }
@@ -1522,7 +1484,7 @@ inccount(struct idused **idmapp, int id)
 	fprintf(stderr, "IDCOUNT must be power of 2!\n");
 	exit(1);
     }
-    while (idmap = *idmapp) {
+    while ((idmap = *idmapp) != NULL) {
 	if (idmap->idstart == (id & ~(IDCOUNT - 1)))
 	    break;
 	idmapp = &idmap->idnext;
@@ -1533,7 +1495,7 @@ inccount(struct idused **idmapp, int id)
 	    perror("idmap");
 	    exit(1);
 	}
-	memset((char *)idmap, 0, sizeof idmap);
+	memset(idmap, 0, sizeof idmap);
 	idmap->idstart = id & ~(IDCOUNT - 1);
 	idmap->idnext = *idmapp;
 	*idmapp = idmap;
@@ -1550,7 +1512,7 @@ idcount(struct idused **idmapp, int id)
 	fprintf(stderr, "IDCOUNT must be power of 2!\n");
 	exit(1);
     }
-    while (idmap = *idmapp) {
+    while ((idmap = *idmapp) != NULL) {
 	if (idmap->idstart == (id & ~(IDCOUNT - 1))) {
 	    return idmap->idcount[id & (IDCOUNT - 1)];
 	}
