@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -30,7 +30,9 @@
 #include <afs/com_err.h>
 #include <afs/cellconfig.h>
 #include "ptclient.h"
+#include "ptuser.h"
 #include "pterror.h"
+#include "display.h"
 #include <afs/afsutil.h>
 
 
@@ -41,6 +43,8 @@ char *whoami;
 
 #ifndef AFS_PTHREAD_ENV
 extern struct ubik_client *pruclient;
+static void skip(char **);
+static void PrintHelp(void);
 #endif
 
 static int ignoreExist = 0;
@@ -62,9 +66,7 @@ osi_audit()
 #endif /* !AFS_PTHREAD_ENV */
 
 int
-GetToken(format, l)
-     char *format;
-     afs_int32 *l;
+GetToken(char *format, afs_int32 *l)
 {
     int c;
 
@@ -85,9 +87,7 @@ GetToken(format, l)
 #define GetXInt32(l) GetToken ("%x", l)
 
 int
-GetString(s, slen)
-     char *s;
-     int slen;
+GetString(char *s, int slen)
 {
     char *beg;
     int l;
@@ -121,8 +121,7 @@ GetString(s, slen)
 }
 
 int
-CodeOk(code)
-     afs_int32 code;
+CodeOk(afs_int32 code)
 {
     if (!ignoreExist)
 	return code;
@@ -130,14 +129,13 @@ CodeOk(code)
 }
 
 int
-PrintEntry(ea, e, indent)
-     afs_int32 ea;
-     struct prentry *e;
-     int indent;
+PrintEntry(afs_int32 ea, struct prentry *e, int indent)
 {
     /* handle screwed up versions of DumpEntry */
     if (e->flags & PRCONT) {
-	afs_int32 id = *(afs_int32 *) (e->name);
+	afs_int32 id;
+
+	memcpy(&id, e->name, sizeof(id));
 	if ((id != PRBADID) && ((id > (1 << 24)) || (id < -(1 << 24)))) {
 	    /* assume server incorrectly swapped these bytes... */
 	    int i = 0;
@@ -162,16 +160,16 @@ PrintEntry(ea, e, indent)
 
 #include "AFS_component_version_number.c"
 
-main(argc, argv)
-     int argc;
-     char **argv;
+int
+main(int argc, char **argv)
 {
-    register afs_int32 code;
+    afs_int32 code;
     char op[8];
     char name[PR_MAXNAMELEN];
     afs_int32 id, oid = ANONYMOUSID, gid;
     afs_int32 pos;
-    int i;
+    unsigned int i;
+    int n;
     struct prentry entry;
     prlist alist;
     idlist lid;
@@ -185,8 +183,8 @@ main(argc, argv)
 
 #ifdef	AFS_AIX32_ENV
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
@@ -203,30 +201,30 @@ main(argc, argv)
 
     strcpy(confdir, AFSDIR_CLIENT_ETC_DIRPATH);
     cell = 0;
-    i = 1;
-    while (i < argc) {
-	int arglen = strlen(argv[i]);
+    n = 1;
+    while (n < argc) {
+	int arglen = strlen(argv[n]);
 	char arg[256];
-	lcstring(arg, argv[i], sizeof(arg));
+	lcstring(arg, argv[n], sizeof(arg));
 #define IsArg(a) (strncmp (arg,a, arglen) == 0)
 	if (IsArg("-testconfdir"))
-	    strncpy(confdir, argv[++i], sizeof(confdir));
+	    strncpy(confdir, argv[++n], sizeof(confdir));
 	else if (IsArg("client"))
 	    strncpy(confdir, AFSDIR_CLIENT_ETC_DIRPATH, sizeof(confdir));
 	else if (IsArg("server"))
 	    strncpy(confdir, AFSDIR_SERVER_ETC_DIRPATH, sizeof(confdir));
 	else if (IsArg("0") || IsArg("1") || IsArg("2"))
-	    security = atoi(argv[i]);
+	    security = atoi(argv[n]);
 	else if (IsArg("-ignoreexist"))
 	    ignoreExist++;
 	else if (IsArg("-cell"))
-	    cell = argv[++i];
+	    cell = argv[++n];
 	else {
 	    printf
 		("Usage is: 'prclient [-testconfdir <dir> | server | client] [0 | 1 | 2] [-ignoreExist] [-cell <cellname>]\n");
 	    exit(1);
 	}
-	i++;
+	n++;
     }
 
     printf("Using CellServDB file in %s\n", confdir);
@@ -309,7 +307,7 @@ main(argc, argv)
 	    if (GetInt32(&pos))
 		code = PRBADARG;
 	    else
-		code = ubik_PR_DumpEntry(pruclient, 0, pos, &entry);
+		code = ubik_PR_DumpEntry(pruclient, 0, pos, (struct prdebugentry *)&entry);
 	    if (CodeOk(code))
 		printf("%s\n", pr_ErrorMsg(code));
 	    if (code == PRSUCCESS) {
@@ -475,7 +473,7 @@ main(argc, argv)
 	    else if (!(hostinfo = gethostbyname(name)))
 		code = PRBADARG;
 	    else {
-		hostaddr = hostinfo->h_addr_list[0];
+		hostaddr = (struct in_addr *)hostinfo->h_addr_list[0];
 		id = ntohl(hostaddr->s_addr);
 		code =
 		    ubik_PR_GetHostCPS(pruclient, 0, id, &alist, &over);
@@ -711,7 +709,8 @@ main(argc, argv)
 }
 
 
-PrintHelp()
+static void
+PrintHelp(void)
 {
     printf("cr name id owner - create entry with name and id.\n");
     printf("wh id  - what is the offset into database for id?\n");
@@ -740,9 +739,8 @@ PrintHelp()
     printf("q - quit.\n?- this message.\n");
 }
 
-
-skip(s)
-     char **s;
+static void
+skip(char **s)
 {
     while (**s != ' ' && **s != '\0')
 	(*s)++;
