@@ -1,13 +1,13 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-/* 
+/*
  *                      (5) Add functions to process supergroups:
  *                          ChangeIDEntry(), RemoveFromSGEntry(),
  *                          AddToSGEntry(), GetListSG2().
@@ -40,6 +40,7 @@
 #include <afs/cellconfig.h>
 #include "ptserver.h"
 #include "pterror.h"
+#include "ptprototypes.h"
 #include <stdlib.h>
 
 /* Foreign cells are represented by the group system:authuser@cell*/
@@ -49,9 +50,11 @@ extern int restricted;
 extern struct ubik_dbase *dbase;
 extern struct afsconf_dir *prdir;
 extern int pr_noAuth;
-extern int IDCmp();
 
-extern afs_int32 AddToEntry();
+static int inRange(struct prentry *cellEntry, afs_int32 aid);
+static afs_int32 allocNextId(struct ubik_trans *, struct prentry *);
+static int AddAuthGroup(struct prentry *tentry, prlist *alist, afs_int32 *size);
+
 static char *whoami = "ptserver";
 
 int prp_user_default = PRP_USER_DEFAULT;
@@ -62,10 +65,8 @@ int prp_group_default = PRP_GROUP_DEFAULT;
 #include "map.h"
 
 afs_int32 depthsg = 5;		/* Maximum iterations used during IsAMemberOF */
-extern int IDCmp();
 afs_int32 GetListSG2(struct ubik_trans *at, afs_int32 gid, prlist * alist,
 		     afs_int32 * sizeP, afs_int32 depth);
-afs_int32 allocNextId(struct ubik_trans *, struct prentry *);
 
 struct map *sg_flagged;
 struct map *sg_found;
@@ -83,10 +84,11 @@ struct map *sg_found;
  *  so we can always tell if we're writing a group record.
  */
 
-int (*pt_save_dbase_write) ();
+int (*pt_save_dbase_write)(struct ubik_dbase *, afs_int32, void *, afs_int32,
+                           afs_int32);
 
 int
-pt_mywrite(struct ubik_dbase *tdb, afs_int32 fno, char *bp, afs_int32 pos, afs_int32 count)
+pt_mywrite(struct ubik_dbase *tdb, afs_int32 fno, void *bp, afs_int32 pos, afs_int32 count)
 {
     afs_uint32 headersize = ntohl(cheader.headerSize);
 
@@ -161,8 +163,8 @@ pt_mywrite(struct ubik_dbase *tdb, afs_int32 fno, char *bp, afs_int32 pos, afs_i
  *  just after ubik_ServerInit.
  */
 
-void 
-pt_hook_write()
+void
+pt_hook_write(void)
 {
     extern struct ubik_dbase *ubik_dbase;
     if (ubik_dbase->write != pt_mywrite) {
@@ -174,22 +176,18 @@ pt_hook_write()
 #endif /* SUPERGROUPS */
 
 /* CorrectUserName - Check to make sure a user name is OK.  It must not include
- *   either a colon (or it would look like a group) or an atsign (or it would
- *   look like a foreign user).  The length is checked as well to make sure
- *   that the user name, an atsign, and the local cell name will fit in
- *   PR_MAXNAMELEN.  This is so this user can fit in another cells database as
- *   a foreign user with our cell name tacked on.  This is a predicate, so it
- *   return one if name is OK and zero if name is bogus. */
+ *   either a colon (or it would look like a group) or a newline (which can
+ *   confuse some ptdb code, depending on the format we're reading from).
+ *   This is a predicate, so it return one if name is OK and zero if name is
+ *   bogus. */
 
 static int
 CorrectUserName(char *name)
 {
-    extern int pr_realmNameLen;
-
     /* We accept foreign names, so we will deal with '@' later */
     if (strchr(name, ':') || strchr(name, '\n'))
 	return 0;
-    if (strlen(name) >= PR_MAXNAMELEN - pr_realmNameLen - 1)
+    if (strlen(name) >= PR_MAXNAMELEN)
 	return 0;
     return 1;
 }
@@ -407,7 +405,7 @@ CreateEntry(struct ubik_trans *at, char aname[PR_MAXNAMELEN], afs_int32 *aid, af
 	}
     } else {
 	/* A foreign user: <name>@<cell>. The foreign user is added to
-	 * its representing group. It is 
+	 * its representing group. It is
 	 */
 	char *cellGroup;
 	afs_int32 pos, n;
@@ -437,7 +435,7 @@ CreateEntry(struct ubik_trans *at, char aname[PR_MAXNAMELEN], afs_int32 *aid, af
 		return PRBADARG;
 	    tentry.id = *aid;
 	} else {
-	    /* Allocate an ID special for this foreign user. It is based 
+	    /* Allocate an ID special for this foreign user. It is based
 	     * on the representing group's id and nusers count.
 	     */
 	    tentry.id = allocNextId(at, &centry);
@@ -537,7 +535,7 @@ CreateEntry(struct ubik_trans *at, char aname[PR_MAXNAMELEN], afs_int32 *aid, af
 	    }
 
 	    /* Admins don't get charged for creating a group.
-	     * If in noAuth mode, you get changed for it but you 
+	     * If in noAuth mode, you get changed for it but you
 	     * are still allowed to create as many groups as you want.
 	     */
 	    admin = ((creator == SYSADMINID)
@@ -685,9 +683,9 @@ RemoveFromEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 bid)
  * entry if appropriate */
 
 afs_int32
-ChangeIDEntry(register struct ubik_trans *at, register afs_int32 aid, afs_int32 newid, register afs_int32 bid)
+ChangeIDEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 newid, afs_int32 bid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct prentry tentry;
     struct contentry centry;
     afs_int32 temp;
@@ -753,9 +751,9 @@ ChangeIDEntry(register struct ubik_trans *at, register afs_int32 aid, afs_int32
  * continuation entry if appropriate */
 
 afs_int32
-RemoveFromSGEntry(register struct ubik_trans *at, register afs_int32 aid, register afs_int32 bid)
+RemoveFromSGEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 bid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct prentry tentry;
     struct prentryg *tentryg;
     struct contentry centry;
@@ -914,7 +912,7 @@ DeleteEntry(struct ubik_trans *at, struct prentry *tentry, afs_int32 loc)
     }
 #endif /* SUPERGROUPS */
     nptr = tentry->next;
-    while (nptr != (afs_int32) NULL) {
+    while (nptr != 0) {
 	code = pr_ReadCoEntry(at, 0, nptr, &centry);
 	if (code != 0)
 	    return PRDBFAIL;
@@ -1035,7 +1033,7 @@ AddToEntry(struct ubik_trans *tt, struct prentry *entry, afs_int32 loc, afs_int3
     }
     last = 0;
     nptr = entry->next;
-    while (nptr != (afs_int32) NULL) {
+    while (nptr != 0) {
 	code = pr_ReadCoEntry(tt, 0, nptr, &nentry);
 	if (code != 0)
 	    return code;
@@ -1117,7 +1115,7 @@ AddToEntry(struct ubik_trans *tt, struct prentry *entry, afs_int32 loc, afs_int3
 afs_int32
 AddToSGEntry(struct ubik_trans *tt, struct prentry *entry, afs_int32 loc, afs_int32 aid)
 {
-    register afs_int32 code;
+    afs_int32 code;
     afs_int32 i;
     struct contentry nentry;
     struct contentry aentry;
@@ -1125,7 +1123,7 @@ AddToSGEntry(struct ubik_trans *tt, struct prentry *entry, afs_int32 loc, afs_in
     afs_int32 nptr;
     afs_int32 last;		/* addr of last cont. block */
     afs_int32 first = 0;
-    afs_int32 cloc;
+    afs_int32 cloc = 0;
     afs_int32 slot = -1;
 
     if (entry->id == aid)
@@ -1298,8 +1296,12 @@ GetList(struct ubik_trans *at, struct prentry *tentry, prlist *alist, afs_int32
 		return code;
 #endif
 	}
-	if (count++ > 50)
-	    IOMGR_Poll(), count = 0;
+	if (count++ > 50) {
+#ifndef AFS_PTHREAD_ENV
+	    IOMGR_Poll();
+#endif
+	    count = 0;
+	}
     }
 
     if (add) {			/* this is for a CPS, so tack on appropriate stuff */
@@ -1314,8 +1316,10 @@ GetList(struct ubik_trans *at, struct prentry *tentry, prlist *alist, afs_int32
 		return code;
 	}
     }
+#ifndef AFS_PTHREAD_ENV
     if (alist->prlist_len > 100)
 	IOMGR_Poll();
+#endif
     qsort(alist->prlist_val, alist->prlist_len, sizeof(afs_int32), IDCmp);
     return PRSUCCESS;
 }
@@ -1352,7 +1356,7 @@ GetList2(struct ubik_trans *at, struct prentry *tentry, struct prentry *tentry2,
     }
 
     nptr = tentry->next;
-    while (nptr != (afs_uint32) NULL) {
+    while (nptr != 0) {
 	/* look through cont entries */
 	code = pr_ReadCoEntry(at, 0, nptr, &centry);
 	if (code != 0)
@@ -1374,8 +1378,12 @@ GetList2(struct ubik_trans *at, struct prentry *tentry, struct prentry *tentry2,
 #endif
 	}
 	nptr = centry.next;
-	if (count++ > 50)
-	    IOMGR_Poll(), count = 0;
+	if (count++ > 50) {
+#ifndef AFS_PTHREAD_ENV
+	    IOMGR_Poll();
+#endif
+	    count = 0;
+	}
     }
 
     for (i = 0; i < PRSIZE; i++) {
@@ -1390,7 +1398,7 @@ GetList2(struct ubik_trans *at, struct prentry *tentry, struct prentry *tentry2,
 
     if (!code) {
 	nptr = tentry2->next;
-	while (nptr != (afs_uint32) NULL) {
+	while (nptr != 0) {
 	    /* look through cont entries */
 	    code = pr_ReadCoEntry(at, 0, nptr, &centry);
 	    if (code != 0)
@@ -1405,8 +1413,12 @@ GetList2(struct ubik_trans *at, struct prentry *tentry, struct prentry *tentry2,
 		    break;
 	    }
 	    nptr = centry.next;
-	    if (count++ > 50)
-		IOMGR_Poll(), count = 0;
+	    if (count++ > 50) {
+#ifndef AFS_PTHREAD_ENV
+		IOMGR_Poll();
+#endif
+		count = 0;
+	    }
 	}
     }
     if (add) {			/* this is for a CPS, so tack on appropriate stuff */
@@ -1421,8 +1433,10 @@ GetList2(struct ubik_trans *at, struct prentry *tentry, struct prentry *tentry2,
 		return code;
 	}
     }
+#ifndef AFS_PTHREAD_ENV
     if (alist->prlist_len > 100)
 	IOMGR_Poll();
+#endif
     qsort(alist->prlist_val, alist->prlist_len, sizeof(afs_int32), IDCmp);
     return PRSUCCESS;
 }
@@ -1432,7 +1446,7 @@ GetList2(struct ubik_trans *at, struct prentry *tentry, struct prentry *tentry2,
 afs_int32
 GetListSG2(struct ubik_trans *at, afs_int32 gid, prlist *alist, afs_int32 *sizeP, afs_int32 depth)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct prentry tentry;
     struct prentryg *tentryg = (struct prentryg *)&tentry;
     afs_int32 i;
@@ -1527,8 +1541,12 @@ GetListSG2(struct ubik_trans *at, afs_int32 gid, prlist *alist, afs_int32 *sizeP
 		return code;
 	}
 	nptr = centry.next;
-	if (count++ > 50)
-	    IOMGR_Poll(), count = 0;
+	if (count++ > 50) {
+#ifndef AFS_PTHREAD_ENV
+	    IOMGR_Poll();
+#endif
+	    count = 0;
+	}
     }
 #if DEBUG_SG_MAP
     fprintf(stderr, "] for gid %d, done [flag=%s]\n", gid,
@@ -1548,7 +1566,7 @@ GetListSG2(struct ubik_trans *at, afs_int32 gid, prlist *alist, afs_int32 *sizeP
 afs_int32
 GetSGList(struct ubik_trans *at, struct prentry *tentry, prlist *alist)
 {
-    register afs_int32 code;
+    afs_int32 code;
     afs_int32 i;
     struct contentry centry;
     struct prentryg *tentryg;
@@ -1587,12 +1605,18 @@ GetSGList(struct ubik_trans *at, struct prentry *tentry, prlist *alist)
 		return code;
 	}
 	nptr = centry.next;
-	if (count++ > 50)
-	    IOMGR_Poll(), count = 0;
+	if (count++ > 50) {
+#ifndef AFS_PTHREAD_ENV
+	    IOMGR_Poll();
+#endif
+	    count = 0;
+	}
     }
 
+#ifndef AFS_PTHREAD_ENV
     if (alist->prlist_len > 100)
 	IOMGR_Poll();
+#endif
     qsort((char *)alist->prlist_val, (int)alist->prlist_len,
 	  sizeof(afs_int32), IDCmp);
     return PRSUCCESS;
@@ -1621,11 +1645,17 @@ GetOwnedChain(struct ubik_trans *ut, afs_int32 *next, prlist *alist)
 	}
 	if (code)
 	    return code;
-	if (count++ > 50)
-	    IOMGR_Poll(), count = 0;
+	if (count++ > 50) {
+#ifndef AFS_PTHREAD_ENV
+	    IOMGR_Poll();
+#endif
+	    count = 0;
+	}
     }
+#ifndef AFS_PTHREAD_ENV
     if (alist->prlist_len > 100)
 	IOMGR_Poll();
+#endif
     qsort(alist->prlist_val, alist->prlist_len, sizeof(afs_int32), IDCmp);
     return PRSUCCESS;
 }
@@ -1660,14 +1690,11 @@ SetMax(struct ubik_trans *at, afs_int32 id, afs_int32 flag)
     return PRSUCCESS;
 }
 
-afs_int32
-read_DbHeader(struct ubik_trans *tt)
+static afs_int32
+UpdateCache(struct ubik_trans *tt, void *rock)
 {
     afs_int32 code;
 
-    if (!ubik_CacheUpdate(tt))
-	return 0;
-
     code = pr_Read(tt, 0, 0, (char *)&cheader, sizeof(cheader));
     if (code != 0) {
 	afs_com_err(whoami, code, "Couldn't read header");
@@ -1675,15 +1702,57 @@ read_DbHeader(struct ubik_trans *tt)
     return code;
 }
 
+afs_int32
+read_DbHeader(struct ubik_trans *tt)
+{
+    return ubik_CheckCache(tt, UpdateCache, NULL);
+}
+
 int pr_noAuth;
-afs_int32 initd = 0;
+
+/**
+ * reads in db cache from ubik.
+ *
+ * @param[in] ut ubik transaction
+ * @param[out] rock  opaque pointer to an int*, which on success will be set
+ *                   to 1 if we need to build the database, or 0 if we do not
+ *
+ * @return operation status
+ *   @retval 0 success
+ */
+static afs_int32
+Initdb_check(struct ubik_trans *tt, void *rock)
+{
+    int *build_rock = rock;
+    afs_int32 code;
+    afs_int32 len;
+
+    len = sizeof(cheader);
+    code = pr_Read(tt, 0, 0, (char *)&cheader, len);
+    if (code != 0) {
+	afs_com_err(whoami, code, "couldn't read header");
+	return code;
+    }
+    if ((ntohl(cheader.version) == PRDBVERSION)
+	&& ntohl(cheader.headerSize) == sizeof(cheader)
+	&& ntohl(cheader.eofPtr) != 0
+	&& FindByID(tt, ANONYMOUSID) != 0) {
+	/* database exists, so we don't have to build it */
+	*build_rock = 0;
+	return 0;
+    }
+
+    /* else we need to build a database */
+    *build_rock = 1;
+    return 0;
+}
 
 afs_int32
-Initdb()
+Initdb(void)
 {
-    afs_int32 code;
     struct ubik_trans *tt;
-    afs_int32 len;
+    int build = 0;
+    afs_int32 code;
 
     /* init the database.  We'll try reading it, but if we're starting
      * from scratch, we'll have to do a write transaction. */
@@ -1698,52 +1767,40 @@ Initdb()
 	ubik_AbortTrans(tt);
 	return code;
     }
-    if (!initd) {
-	initd = 1;
-    } else if (!ubik_CacheUpdate(tt)) {
-	code = ubik_EndTrans(tt);
-	return code;
-    }
 
-    len = sizeof(cheader);
-    code = pr_Read(tt, 0, 0, (char *)&cheader, len);
-    if (code != 0) {
-	afs_com_err(whoami, code, "couldn't read header");
+    code = ubik_CheckCache(tt, Initdb_check, &build);
+    if (code) {
 	ubik_AbortTrans(tt);
 	return code;
     }
-    if ((ntohl(cheader.version) == PRDBVERSION)
-	&& ntohl(cheader.headerSize) == sizeof(cheader)
-	&& ntohl(cheader.eofPtr) != (afs_uint32) NULL
-	&& FindByID(tt, ANONYMOUSID) != 0) {
-	/* database exists, so we don't have to build it */
-	code = ubik_EndTrans(tt);
-	if (code)
-	    return code;
-	return PRSUCCESS;
-    }
-    /* else we need to build a database */
-    code = ubik_EndTrans(tt);
-    if (code)
-	return code;
 
-    /* Only rebuild database if the db was deleted (the header is zero) and we
-     * are running noAuth. */
-    {
+    if (build) {
+	/* Only rebuild database if the db was deleted (the header is zero) and we
+	 * are running noAuth. */
 	char *bp = (char *)&cheader;
 	int i;
-	for (i = 0; i < sizeof(cheader); i++)
+	for (i = 0; i < sizeof(cheader); i++) {
 	    if (bp[i]) {
 		code = PRDBBAD;
 		afs_com_err(whoami, code,
 			"Can't rebuild database because it is not empty");
-		return code;
+		break;
 	    }
+	}
+	if (!pr_noAuth) {
+	    code = PRDBBAD;
+	    afs_com_err(whoami, code,
+	                "Can't rebuild database because not running NoAuth");
+	}
     }
-    if (!pr_noAuth) {
-	code = PRDBBAD;
-	afs_com_err(whoami, code,
-		"Can't rebuild database because not running NoAuth");
+
+    if (code) {
+	ubik_EndTrans(tt);
+    } else {
+	code = ubik_EndTrans(tt);
+    }
+    if (code || !build) {
+	/* either we encountered an error, or we don't need to build the db */
 	return code;
     }
 
@@ -1762,9 +1819,15 @@ Initdb()
      * actually have been a good database out there.  Now that we have a
      * real write transaction, make sure things are still bad.
      */
+    code = pr_Read(tt, 0, 0, (char *)&cheader, sizeof(cheader));
+    if (code != 0) {
+	afs_com_err(whoami, code, "couldn't read header");
+	ubik_AbortTrans(tt);
+	return code;
+    }
     if ((ntohl(cheader.version) == PRDBVERSION)
 	&& ntohl(cheader.headerSize) == sizeof(cheader)
-	&& ntohl(cheader.eofPtr) != (afs_uint32) NULL
+	&& ntohl(cheader.eofPtr) != 0
 	&& FindByID(tt, ANONYMOUSID) != 0) {
 	/* database exists, so we don't have to build it */
 	code = ubik_EndTrans(tt);
@@ -1820,11 +1883,13 @@ afs_int32
 ChangeEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 cid, char *name, afs_int32 oid, afs_int32 newid)
 {
     afs_int32 code;
-    afs_int32 i, nptr, pos;
+    afs_int32 i, pos;
 #if defined(SUPERGROUPS)
     afs_int32 nextpos;
-#endif
+#else
+    afs_int32 nptr;
     struct contentry centry;
+#endif
     struct prentry tentry, tent;
     afs_int32 loc;
     afs_int32 oldowner;
@@ -1841,7 +1906,7 @@ ChangeEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 cid, char *name, afs
     code = pr_ReadEntry(at, 0, loc, &tentry);
     if (code)
 	return PRDBFAIL;
-    if (restricted && !IsAMemberOf(at, cid, SYSADMINID)) 
+    if (restricted && !IsAMemberOf(at, cid, SYSADMINID))
 	return PRPERM;
     if (tentry.owner != cid && !IsAMemberOf(at, cid, SYSADMINID)
 	&& !IsAMemberOf(at, cid, tentry.owner) && !pr_noAuth)
@@ -1969,7 +2034,7 @@ ChangeEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 cid, char *name, afs
 		return code;
 	}
 	/* Look through cont entries too. This needs to be broken into
-	 * seperate transaction so that no one transaction becomes too 
+	 * seperate transaction so that no one transaction becomes too
 	 * large to complete.
 	 */
 	for (nptr = tentry.next; nptr; nptr = centry.next) {
@@ -2011,7 +2076,7 @@ ChangeEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 cid, char *name, afs
 	tentry.owner = oid;
 	/* The entry must be written through first so Remove and Add routines
 	 * can operate on disk data */
-	code = pr_WriteEntry(at, 0, loc, (char *)&tentry);
+	code = pr_WriteEntry(at, 0, loc, &tentry);
 	if (code)
 	    return PRDBFAIL;
 
@@ -2078,7 +2143,7 @@ ChangeEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 cid, char *name, afs
 	if (code != PRSUCCESS)
 	    return code;
 	strncpy(tentry.name, name, PR_MAXNAMELEN);
-	code = pr_WriteEntry(at, 0, loc, (char *)&tentry);
+	code = pr_WriteEntry(at, 0, loc, &tentry);
 	if (code)
 	    return PRDBFAIL;
 	code = AddToNameHash(at, tentry.name, loc);
@@ -2090,7 +2155,7 @@ ChangeEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 cid, char *name, afs
 }
 
 
-afs_int32
+static afs_int32
 allocNextId(struct ubik_trans * at, struct prentry * cellEntry)
 {
     /* Id's for foreign cell entries are constructed as follows:
@@ -2108,20 +2173,20 @@ allocNextId(struct ubik_trans * at, struct prentry * cellEntry)
     }
 
     cellEntry->nusers = htonl(id);
-    /* use the field nusers to keep 
+    /* use the field nusers to keep
      * the next available id in that
      * foreign cell's group. Note :
      * It would seem more appropriate
      * to use ngroup for that and nusers
      * to enforce the quota, however pts
-     * does not have an option to change 
+     * does not have an option to change
      * foreign users quota yet  */
 
     id = (id << 16) | cellid;
     return id;
 }
 
-int
+static int
 inRange(struct prentry *cellEntry, afs_int32 aid)
 {
     afs_uint32 id, cellid, groupid;
@@ -2130,7 +2195,7 @@ inRange(struct prentry *cellEntry, afs_int32 aid)
     /*
      * The only thing that we want to make sure here is that
      * the id is in the legal range of this group. If it is
-     * a duplicate we don't care since it will get caught 
+     * a duplicate we don't care since it will get caught
      * in a different check.
      */
 
@@ -2139,9 +2204,9 @@ inRange(struct prentry *cellEntry, afs_int32 aid)
     if (cellid != groupid)
 	return 0;		/* not in range */
 
-    /* 
+    /*
      * if we got here we're ok but we need to update the nusers
-     * field in order to get the id correct the next time that 
+     * field in order to get the id correct the next time that
      * we try to allocate it automatically
      */
 
@@ -2152,7 +2217,7 @@ inRange(struct prentry *cellEntry, afs_int32 aid)
 
 }
 
-int
+static int
 AddAuthGroup(struct prentry *tentry, prlist *alist, afs_int32 *size)
 {
     if (!(strchr(tentry->name, '@')))
