@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -18,8 +18,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/uss/uss.c,v 1.8.2.3 2007/11/26 21:21:56 shadow Exp $");
 
 #ifdef	AFS_AIX32_ENV
 #include <signal.h>
@@ -31,11 +29,18 @@ RCSID
 #include "uss_procs.h"		/*Main uss operations */
 #include "uss_kauth.h"		/*AuthServer routines */
 #include "uss_fs.h"		/*CacheManager ops */
+#include "uss_ptserver.h"
+#include "uss_vol.h"
+#include "uss_acl.h"
 #include <afs/cmd.h>		/*Command line parsing */
 #include <afs/cellconfig.h>	/*Cell config defs */
 #include <afs/kautils.h>	/*MAXKTCREALMLEN & MAXKTCNAMELEN */
+#include <afs/pterror.h>
+#include <afs/vlserver.h>
 #include <ubik.h>
 
+extern int yylex(void);
+extern int yyparse (void);
 /*
  * Label certain things which will be activated at a later time,
  * as well as certain semi-implemented features/switches which need
@@ -101,7 +106,7 @@ extern int doUnlog;
 int uss_BulkExpires = 0;
 int local_Cell = 1;
 
-static int DoAdd();
+static int DoAdd(void);
 
 /*-----------------------------------------------------------------------
  * static GetCommon
@@ -124,13 +129,13 @@ static int DoAdd();
  *------------------------------------------------------------------------*/
 
 static int
-GetCommon(register struct cmd_syndesc *a_as, void *arock)
+GetCommon(struct cmd_syndesc *a_as, void *arock)
 {				/*GetCommon */
 
     int code;			/*Result of ka_LocalCell */
 
     if (strcmp(a_as->name, "help") == 0)
-	return;
+	return 0;
     if (a_as->parms[AUSS_TEMPLATE].items)
 	strcpy(Template, a_as->parms[AUSS_TEMPLATE].items->data);
     if (a_as->parms[AUSS_VERBOSE].items)
@@ -208,11 +213,12 @@ GetCommon(register struct cmd_syndesc *a_as, void *arock)
  *------------------------------------------------------------------------*/
 
 static int
-SaveRestoreInfo()
+SaveRestoreInfo(void)
 {				/*SaveRestoreInfo */
-
+#ifdef USS_DB
     static char rn[] = "uss:SaveRestoreInfo";	/*Routine name */
-    register afs_int32 code;	/*Return code */
+#endif
+    afs_int32 code;	/*Return code */
     afs_int32 deletedUid;	/*Uid to be nuked */
 
     /*
@@ -266,7 +272,7 @@ SaveRestoreInfo()
  *------------------------------------------------------------------------*/
 
 static int
-DoDelete()
+DoDelete(void)
 {				/*DoDelete */
 
     int code;			/*Return code */
@@ -438,7 +444,8 @@ DelUser(struct cmd_syndesc *a_as, void *a_rock)
 
 }				/*DelUser */
 
-
+#if USS_FUTURE_FEATURES
+#if USS_DONT_HIDE_SOME_FEATURES
 /*-----------------------------------------------------------------------
  * static PurgeVolumes
  *
@@ -502,6 +509,8 @@ RestoreUser(struct cmd_syndesc *a_as, void *a_rock)
 
 }				/*RestoreUser */
 
+#endif
+#endif
 
 /*-----------------------------------------------------------------------
  * static DoBulkAddLine
@@ -527,14 +536,13 @@ RestoreUser(struct cmd_syndesc *a_as, void *a_rock)
  *------------------------------------------------------------------------*/
 
 static int
-DoBulkAddLine(a_buf, a_tp)
-     char *a_buf;
-     char *a_tp;
-
+DoBulkAddLine(char *a_buf,  char *a_tp)
 {				/*DoBulkAddLine */
 
-    register int i;		/*Loop variable */
+    int i;		/*Loop variable */
+#ifdef USS_DB
     static char rn[] = "DoBulkAddLine";	/*Routine name */
+#endif
     int overflow;		/*Overflow in field copy? */
 
 #ifdef USS_DB
@@ -748,10 +756,7 @@ DoBulkAddLine(a_buf, a_tp)
  *------------------------------------------------------------------------*/
 
 static int
-DoBulkDeleteLine(a_buf, a_tp)
-     char *a_buf;
-     char *a_tp;
-
+DoBulkDeleteLine(char *a_buf, char *a_tp)
 {				/*DoBulkDeleteLine */
 
     char volField[32];		/*Value of optional vol disposition field */
@@ -908,13 +913,10 @@ DoBulkDeleteLine(a_buf, a_tp)
  *------------------------------------------------------------------------*/
 
 static int
-DoBulkPurgeVolumeLine(a_buf, a_tp)
-     char *a_buf;
-     char *a_tp;
-
+DoBulkPurgeVolumeLine(char *a_buf, char *a_tp)
 {				/*DoBulkPurgeVolumeLine */
 
-    register int i;		/*Loop variable */
+    int i;		/*Loop variable */
     int overflow;		/*Did a field copy overflow happen? */
 
     /*
@@ -1089,13 +1091,10 @@ DoBulkPurgeVolumeLine(a_buf, a_tp)
  *------------------------------------------------------------------------*/
 
 static int
-DoBulkRestoreLine(a_buf, a_tp)
-     char *a_buf;
-     char *a_tp;
-
+DoBulkRestoreLine(char *a_buf, char *a_tp)
 {				/*DoBulkRestoreLine */
 
-    register int i;		/*Loop variable */
+    int i;		/*Loop variable */
     int overflow;		/*Overflow occur on field copy? */
 
     /*
@@ -1266,13 +1265,10 @@ DoBulkRestoreLine(a_buf, a_tp)
  *------------------------------------------------------------------------*/
 
 static int
-DoBulkExecLine(a_buf, a_tp)
-     char *a_buf;
-     char *a_tp;
-
+DoBulkExecLine(char *a_buf, char *a_tp)
 {				/*DoBulkExecLine */
 
-    register afs_int32 code;	/*Return code */
+    afs_int32 code;	/*Return code */
 
     /*
      * Really, uss_procs_Exec does all the work for us!
@@ -1306,7 +1302,7 @@ DoBulkExecLine(a_buf, a_tp)
  *------------------------------------------------------------------------*/
 extern int Pipe;
 static int
-HandleBulk(register struct cmd_syndesc *a_as, void *a_rock)
+HandleBulk(struct cmd_syndesc *a_as, void *a_rock)
 {				/*HandleBulk */
 
 #define USS_BULK_CMD_CHARS	 128
@@ -1319,7 +1315,7 @@ HandleBulk(register struct cmd_syndesc *a_as, void *a_rock)
     int code;
 
     int line_no = 0;
-    int error;
+    int error = 0;
     char tbuf[USS_BULK_BUF_CHARS];
 
     /*
@@ -1515,11 +1511,11 @@ HandleBulk(register struct cmd_syndesc *a_as, void *a_rock)
  *------------------------------------------------------------------------*/
 
 static int
-AddUser(register struct cmd_syndesc *a_as, void *a_rock)
+AddUser(struct cmd_syndesc *a_as, void *a_rock)
 {				/*AddUser */
 
     int i;
-    register struct cmd_item *ti;
+    struct cmd_item *ti;
     int code;
 
     /*
@@ -1639,7 +1635,7 @@ AddUser(register struct cmd_syndesc *a_as, void *a_rock)
  *------------------------------------------------------------------------*/
 
 static int
-DoAdd()
+DoAdd(void)
 {				/*DoAdd */
 
     int code;			/*Return code */
@@ -1730,7 +1726,7 @@ DoAdd()
  *------------------------------------------------------------------------*/
 
 static int
-DoRestore()
+DoRestore(void)
 {				/*DoRestore */
 
     return (0);
@@ -1760,7 +1756,7 @@ DoRestore()
  *------------------------------------------------------------------------*/
 
 void
-InitETTables()
+InitETTables(void)
 {				/*InitETTables */
 
 
@@ -1781,7 +1777,7 @@ InitETTables()
 
 
 int
-osi_audit()
+osi_audit(void)
 {
 /* this sucks but it works for now.
 */
@@ -1790,19 +1786,17 @@ osi_audit()
 
 #include "AFS_component_version_number.c"
 
-main(argc, argv)
-     int argc;
-     char *argv[];
-
+int
+main(int argc, char *argv[])
 {				/*Main routine */
 
-    register struct cmd_syndesc *cs;	/*Command line syntax descriptor */
-    register afs_int32 code;	/*Return code */
+    struct cmd_syndesc *cs;	/*Command line syntax descriptor */
+    afs_int32 code;	/*Return code */
 
 #ifdef	AFS_AIX32_ENV
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
