@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -22,6 +22,7 @@
 #include "uss_ptserver.h"	/*Module interface */
 #include <afs/ptclient.h>	/*Protection Server client interface */
 #include <afs/pterror.h>	/*Protection Server error codes */
+#include <afs/ptuser.h>
 #include <afs/com_err.h>	/*Error code xlation */
 
 
@@ -64,11 +65,13 @@ static int initDone = 0;	/*Module initialized? */
  *------------------------------------------------------------------------*/
 
 static afs_int32
-InitThisModule()
+InitThisModule(void)
 {				/*InitThisModule */
 
+#ifdef USS_PTSERVER_DB
     static char rn[] = "uss_ptserver:InitThisModule";	/*Routine name */
-    register afs_int32 code;	/*Return code */
+#endif
+    afs_int32 code;	/*Return code */
 
     /*
      * Only once, guys.
@@ -111,10 +114,7 @@ InitThisModule()
  *------------------------------------------------------------------------*/
 
 afs_int32
-uss_ptserver_AddUser(a_user, a_uid)
-     char *a_user;
-     char *a_uid;
-
+uss_ptserver_AddUser(char *a_user, char *a_uid)
 {				/*uss_ptserver_AddUser */
 
     afs_int32 code;		/*Various return codes */
@@ -173,7 +173,7 @@ uss_ptserver_AddUser(a_user, a_uid)
 	     * ID known for the user name.
 	     */
 	    mappedUserID = id;
-	    if (code = pr_SNameToId(a_user, &mappedUserID)) {
+	    if ((code = pr_SNameToId(a_user, &mappedUserID))) {
 		afs_com_err(uss_whoami, code,
 			"while getting uid from Protection Server");
 		return (code);
@@ -224,9 +224,7 @@ uss_ptserver_AddUser(a_user, a_uid)
  *------------------------------------------------------------------------*/
 
 afs_int32
-uss_ptserver_DelUser(a_name)
-     char *a_name;
-
+uss_ptserver_DelUser(char *a_name)
 {				/*uss_ptserver_DelUser */
 
     afs_int32 code;		/*Various return codes */
@@ -292,14 +290,12 @@ uss_ptserver_DelUser(a_name)
  *------------------------------------------------------------------------*/
 
 afs_int32
-uss_ptserver_XlateUser(a_user, a_uidP)
-     char *a_user;
-     afs_int32 *a_uidP;
-
+uss_ptserver_XlateUser(char *a_user, afs_int32 *a_uidP)
 {				/*uss_ptserver_XlateUser */
-
+#ifdef USS_PTSERVER_DB
     static char rn[] = "uss_ptserver_XlateUser";	/*Routine name */
-    register afs_int32 code;	/*Various return codes */
+#endif
+    afs_int32 code;	/*Various return codes */
 
     if (uss_verbose)
 	fprintf(stderr, "Translating user '%s' via the Protection DB\n",
