@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -13,22 +13,22 @@
  * Copyright (c) 2002 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- * 
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 
+ *
  * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -49,8 +49,6 @@
 #include <afs/param.h>
 #endif
 
-RCSID
-    ("$Header: /cvs/openafs/src/util/uuid.c,v 1.16.2.2 2007/10/30 15:24:09 shadow Exp $");
 
 #ifdef KERNEL
 #include "afs/sysincludes.h"
@@ -62,6 +60,7 @@ RCSID
 #include <errno.h>
 #include <string.h>
 #ifdef AFS_NT40_ENV
+#include <rpc.h>
 #include <winsock2.h>
 #include <process.h>
 #else
@@ -81,7 +80,7 @@ RCSID
 #endif
 #include <sys/stat.h>
 #include <fcntl.h>
-#if !defined(AFS_NT40_ENV) && !defined(AFS_LINUX20_ENV)
+#if !defined(AFS_NT40_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_ARM_DARWIN_ENV)
 #include <netinet/if_ether.h>
 #endif
 #include "afsutil.h"
@@ -245,12 +244,15 @@ afsUUID_to_string(const afsUUID * uuid, char *str, size_t strsz)
 afs_int32
 afs_uuid_create(afsUUID * uuid)
 {
+#ifdef AFS_NT40_ENV
+    UuidCreate((UUID *) uuid);
+#else /* AFS_NT40_ENV */
     uuid_address_t eaddr;
     afs_int32 got_no_time = 0, code;
 
     if (!uuid_init_done) {
 	uuid_time_t t;
-	u_short *seedp, seed = 0;
+	u_short seedp[4], seed = 0;
 	rand_m = 971;;
 	rand_ia = 11113;
 	rand_ib = 104322;
@@ -267,13 +269,15 @@ afs_uuid_create(afsUUID * uuid)
 	 * are multiple processes creating UUID's on a system, we add in the PID.
 	 */
 	uuid__get_os_time(&t);
-	seedp = (u_short *) (&t);
-	seed ^= *seedp++;
-	seed ^= *seedp++;
-	seed ^= *seedp++;
-	seed ^= *seedp++;
+	memcpy(&seedp, &t, sizeof(seedp));
+	seed ^= seedp[0];
+	seed ^= seedp[1];
+	seed ^= seedp[2];
+	seed ^= seedp[3];
 #if defined(KERNEL) && defined(AFS_XBSD_ENV)
 	rand_irand += seed + (afs_uint32) curproc->p_pid;
+#elif defined(UKERNEL)
+	rand_irand += seed + (afs_uint32) osi_getpid();
 #else
 	rand_irand += seed + (afs_uint32) getpid();
 #endif
@@ -333,6 +337,7 @@ afs_uuid_create(afsUUID * uuid)
     uuid->clock_seq_hi_and_reserved = (clock_seq & 0x3f00) >> 8;
     uuid->clock_seq_hi_and_reserved |= 0x80;
     uuid_memcpy((void *)uuid->node, (void *)&eaddr, sizeof(uuid_address_t));
+#endif /* AFS_NT40_ENV */
     return 0;
 }
 
