@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -28,7 +28,7 @@
 #include <unistd.h>
 #endif
 #include <afs/nfs.h>
-#include <afs/assert.h>
+#include <afs/afs_assert.h>
 #include <lwp.h>
 #include <lock.h>
 #include <time.h>
@@ -50,37 +50,29 @@ int
 ReallyRead(DirHandle * file, int block, char *data)
 {
     int code;
+    ssize_t rdlen;
     FdHandle_t *fdP;
+    afs_ino_str_t stmp;
 
     fdP = IH_OPEN(file->dirh_handle);
     if (fdP == NULL) {
 	code = errno;
 	ViceLog(0,
 		("ReallyRead(): open failed device %X inode %s errno %d\n",
-		 file->dirh_handle->ih_dev, PrintInode(NULL,
+		 file->dirh_handle->ih_dev, PrintInode(stmp,
 						       file->dirh_handle->
 						       ih_ino), code));
 	return code;
     }
-    if (FDH_SEEK(fdP, block * PAGESIZE, SEEK_SET) < 0) {
-	code = errno;
-	ViceLog(0,
-		("ReallyRead(): lseek failed device %X inode %s errno %d\n",
-		 file->dirh_handle->ih_dev, PrintInode(NULL,
-						       file->dirh_handle->
-						       ih_ino), code));
-	FDH_REALLYCLOSE(fdP);
-	return code;
-    }
-    code = FDH_READ(fdP, data, PAGESIZE);
-    if (code != PAGESIZE) {
-	if (code < 0)
+    rdlen = FDH_PREAD(fdP, data, PAGESIZE, ((afs_foff_t)block) * PAGESIZE);
+    if (rdlen != PAGESIZE) {
+	if (rdlen < 0)
 	    code = errno;
 	else
 	    code = EIO;
 	ViceLog(0,
 		("ReallyRead(): read failed device %X inode %s errno %d\n",
-		 file->dirh_handle->ih_dev, PrintInode(NULL,
+		 file->dirh_handle->ih_dev, PrintInode(stmp,
 						       file->dirh_handle->
 						       ih_ino), code));
 	FDH_REALLYCLOSE(fdP);
@@ -95,33 +87,24 @@ ReallyRead(DirHandle * file, int block, char *data)
 int
 ReallyWrite(DirHandle * file, int block, char *data)
 {
-    afs_int32 count;
+    ssize_t count;
     FdHandle_t *fdP;
+    afs_ino_str_t stmp;
 
     fdP = IH_OPEN(file->dirh_handle);
     if (fdP == NULL) {
 	ViceLog(0,
 		("ReallyWrite(): open failed device %X inode %s errno %d\n",
-		 file->dirh_handle->ih_dev, PrintInode(NULL,
+		 file->dirh_handle->ih_dev, PrintInode(stmp,
 						       file->dirh_handle->
 						       ih_ino), errno));
 	lpErrno = errno;
 	return 0;
     }
-    if (FDH_SEEK(fdP, block * PAGESIZE, SEEK_SET) < 0) {
-	ViceLog(0,
-		("ReallyWrite(): lseek failed device %X inode %s errno %d\n",
-		 file->dirh_handle->ih_dev, PrintInode(NULL,
-						       file->dirh_handle->
-						       ih_ino), errno));
-	lpErrno = errno;
-	FDH_REALLYCLOSE(fdP);
-	return 0;
-    }
-    if ((count = FDH_WRITE(fdP, data, PAGESIZE)) != PAGESIZE) {
+    if ((count = FDH_PWRITE(fdP, data, PAGESIZE, ((afs_foff_t)block) * PAGESIZE)) != PAGESIZE) {
 	ViceLog(0,
 		("ReallyWrite(): write failed device %X inode %s errno %d\n",
-		 file->dirh_handle->ih_dev, PrintInode(NULL,
+		 file->dirh_handle->ih_dev, PrintInode(stmp,
 						       file->dirh_handle->
 						       ih_ino), errno));
 	lpCount = count;
@@ -135,10 +118,10 @@ ReallyWrite(DirHandle * file, int block, char *data)
 
 
 void
-SetDirHandle(register DirHandle * dir, register Vnode * vnode)
+SetDirHandle(DirHandle * dir, Vnode * vnode)
 {
-    register Volume *vp = vnode->volumePtr;
-    register IHandle_t *h;
+    Volume *vp = vnode->volumePtr;
+    IHandle_t *h;
     IH_COPY(h, vnode->handle);
     dir->dirh_ino = h->ih_ino;
     dir->dirh_dev = h->ih_dev;
@@ -153,13 +136,13 @@ void
 FidZap(DirHandle * file)
 {
     IH_RELEASE(file->dirh_handle);
-    memset((char *)file, 0, sizeof(DirHandle));
+    memset(file, 0, sizeof(DirHandle));
 }
 
 void
 FidZero(DirHandle * file)
 {
-    memset((char *)file, 0, sizeof(DirHandle));
+    memset(file, 0, sizeof(DirHandle));
 }
 
 int
