@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -11,7 +11,7 @@
    System:		VICE-TWO
    Module:		vol-info.c
    Institution:	The Information Technology Center, Carnegie-Mellon University
-   
+
    */
 
 #include <afsconfig.h>
@@ -33,6 +33,7 @@
 #include <sys/time.h>
 #endif
 #include <afs/cmd.h>
+#include <afs/dir.h>
 
 #include <rx/xdr.h>
 #include <afs/afsint.h>
@@ -46,8 +47,8 @@
 #include "volume.h"
 #include "partition.h"
 #include "viceinode.h"
-#include "volinodes.h"
 #include <afs/afssyscalls.h>
+#include <afs/afsutil.h>
 
 #ifdef _AIX
 #include <time.h>
@@ -83,18 +84,18 @@ int fixheader = 0, saveinodes = 0, orphaned = 0;
 int VolumeChanged;
 
 /* Forward Declarations */
-void PrintHeader(register Volume * vp);
+void PrintHeader(Volume * vp);
 void HandleAllPart(void);
 void HandlePart(struct DiskPartition64 *partP);
 void HandleVolume(struct DiskPartition64 *partP, char *name);
 struct DiskPartition64 *FindCurrentPartition(void);
 Volume *AttachVolume(struct DiskPartition64 *dp, char *volname,
-		     register struct VolumeHeader *header);
+		     struct VolumeHeader *header);
 #if defined(AFS_NAMEI_ENV)
-void PrintVnode(int offset, VnodeDiskObject * vnode, VnodeId vnodeNumber,
+void PrintVnode(afs_foff_t offset, VnodeDiskObject * vnode, VnodeId vnodeNumber,
 		Inode ino, Volume * vp);
 #else
-void PrintVnode(int offset, VnodeDiskObject * vnode, VnodeId vnodeNumber,
+void PrintVnode(afs_foff_t offset, VnodeDiskObject * vnode, VnodeId vnodeNumber,
 		Inode ino);
 #endif
 void PrintVnodes(Volume * vp, VnodeClass class);
@@ -104,7 +105,7 @@ date(time_t date)
 {
 #define MAX_DATE_RESULT	100
     static char results[8][MAX_DATE_RESULT];
-    static next;
+    static int next;
     struct tm *tm = localtime(&date);
     char buf[32];
 
@@ -168,9 +169,9 @@ ReadHdr1(IHandle_t * ih, char *to, int size, u_int magic, u_int version)
 
 Volume *
 AttachVolume(struct DiskPartition64 * dp, char *volname,
-	     register struct VolumeHeader * header)
+	     struct VolumeHeader * header)
 {
-    register Volume *vp;
+    Volume *vp;
     afs_int32 ec = 0;
 
     vp = (Volume *) calloc(1, sizeof(Volume));
@@ -217,9 +218,9 @@ AttachVolume(struct DiskPartition64 * dp, char *volname,
 static int
 handleit(struct cmd_syndesc *as, void *arock)
 {
-    register struct cmd_item *ti;
+    struct cmd_item *ti;
     int err = 0;
-    int volumeId = 0;
+    afs_uint32 volumeId = 0;
     char *partName = 0;
     struct DiskPartition64 *partP = NULL;
 
@@ -279,13 +280,14 @@ handleit(struct cmd_syndesc *as, void *arock)
 	orphaned = 1;
 	DumpVnodes = 1;
     } else
+	orphaned = 0;
 #if defined(AFS_NAMEI_ENV)
     if (as->parms[12].items) {
 	PrintFileNames = 1;
 	DumpVnodes = 1;
     } else
+	PrintFileNames = 0;
 #endif
-	orphaned = 0;
 
     DInit(10);
 
@@ -320,7 +322,7 @@ handleit(struct cmd_syndesc *as, void *arock)
 	    }
 	}
 	(void)afs_snprintf(name1, sizeof name1, VFORMAT,
-			   afs_cast_uint32(volumeId));
+			   afs_printable_uint32_lu(volumeId));
 	if (dsizeOnly && !saveinodes)
 	    printf
 		("Volume-Id\t  Volsize  Auxsize Inodesize  AVolsize SizeDiff                (VolName)\n");
@@ -332,7 +334,7 @@ handleit(struct cmd_syndesc *as, void *arock)
 #ifdef AFS_NT40_ENV
 #include <direct.h>
 struct DiskPartition64 *
-FindCurrentPartition()
+FindCurrentPartition(void)
 {
     int dr = _getdrive();
     struct DiskPartition64 *dp;
@@ -349,7 +351,7 @@ FindCurrentPartition()
 }
 #else
 struct DiskPartition64 *
-FindCurrentPartition()
+FindCurrentPartition(void)
 {
     char partName[1024];
     char tmp = '\0';
@@ -360,7 +362,7 @@ FindCurrentPartition()
 	perror("pwd");
 	exit(1);
     }
-    p = strchr(&partName[1], '/');
+    p = strchr(&partName[1], OS_DIRSEPC);
     if (p) {
 	tmp = *p;
 	*p = '\0';
@@ -444,7 +446,7 @@ HandleVolume(struct DiskPartition64 *dp, char *name)
     struct VolumeHeader header;
     struct VolumeDiskHeader diskHeader;
     struct afs_stat status, stat;
-    register int fd;
+    int fd;
     Volume *vp;
     IHandle_t *ih;
     char headerName[1024];
@@ -455,7 +457,7 @@ HandleVolume(struct DiskPartition64 *dp, char *name)
     } else {
 	afs_int32 n;
 
-	(void)afs_snprintf(headerName, sizeof headerName, "%s/%s",
+	(void)afs_snprintf(headerName, sizeof headerName, "%s" OS_DIRSEP "%s",
 			   VPartitionPath(dp), name);
 	if ((fd = afs_open(headerName, O_RDONLY)) == -1
 	    || afs_fstat(fd, &status) == -1) {
@@ -480,15 +482,16 @@ HandleVolume(struct DiskPartition64 *dp, char *name)
 
 	if (dheader) {
 	    FdHandle_t *fdP;
-	    int size = 0;
-	    int code;
+	    afs_sfsize_t size = 0;
+	    afs_sfsize_t code;
 
 	    if (afs_fstat(fd, &stat) == -1) {
 		perror("stat");
 		exit(1);
 	    }
 	    if (!dsizeOnly && !saveinodes) {
-		printf("Volume header (size = %d):\n", size = stat.st_size);
+		size = stat.st_size;
+		printf("Volume header (size = %d):\n", (int)size);
 		printf("\tstamp\t= 0x%x\n", header.stamp.version);
 		printf("\tVolId\t= %u\n", header.id);
 	    }
@@ -510,7 +513,7 @@ HandleVolume(struct DiskPartition64 *dp, char *name)
 	    if (!dsizeOnly && !saveinodes) {
 		printf("\tparent\t= %u\n", header.parent);
 		printf("\tInfo inode\t= %s (size = %d)\n",
-		       PrintInode(NULL, header.volumeInfo), code);
+		       PrintInode(NULL, header.volumeInfo), (int)code);
 	    }
 
 	    IH_INIT(ih, dp->device, header.parent, header.smallVnodeIndex);
@@ -529,7 +532,7 @@ HandleVolume(struct DiskPartition64 *dp, char *name)
 	    size += code;
 	    if (!dsizeOnly && !saveinodes) {
 		printf("\tSmall inode\t= %s (size = %d)\n",
-		       PrintInode(NULL, header.smallVnodeIndex), code);
+		       PrintInode(NULL, header.smallVnodeIndex), (int)code);
 	    }
 
 	    IH_INIT(ih, dp->device, header.parent, header.largeVnodeIndex);
@@ -548,9 +551,9 @@ HandleVolume(struct DiskPartition64 *dp, char *name)
 	    size += code;
 	    if (!dsizeOnly && !saveinodes) {
 		printf("\tLarge inode\t= %s (size = %d)\n",
-		       PrintInode(NULL, header.largeVnodeIndex), code);
+		       PrintInode(NULL, header.largeVnodeIndex), (int)code);
 #ifndef AFS_NT40_ENV
-		printf("Total aux volume size = %d\n\n", size);
+		printf("Total aux volume size = %d\n\n", (int)size);
 #endif
 	    }
 #ifdef AFS_NAMEI_ENV
@@ -570,8 +573,8 @@ HandleVolume(struct DiskPartition64 *dp, char *name)
 	    size += code;
 	    if (!dsizeOnly && !saveinodes) {
 		printf("\tLink inode\t= %s (size = %d)\n",
-		       PrintInode(NULL, header.linkTable), code);
-		printf("Total aux volume size = %d\n\n", size);
+		       PrintInode(NULL, header.linkTable), (int)code);
+		printf("Total aux volume size = %d\n\n", (int)size);
 	    }
 #endif
 	    Vauxsize = size;
@@ -613,7 +616,7 @@ HandleVolume(struct DiskPartition64 *dp, char *name)
 int
 main(int argc, char **argv)
 {
-    register struct cmd_syndesc *ts;
+    struct cmd_syndesc *ts;
     afs_int32 code;
 
     ts = cmd_CreateSyntax(NULL, handleit, NULL, "Dump volume's internal state");
@@ -649,7 +652,7 @@ main(int argc, char **argv)
 #define typestring(type) (type == RWVOL? "read/write": type == ROVOL? "readonly": type == BACKVOL? "backup": "unknown")
 
 void
-PrintHeader(register Volume * vp)
+PrintHeader(Volume * vp)
 {
     Vdiskused = V_diskused(vp);
     if (dsizeOnly || saveinodes)
@@ -686,6 +689,7 @@ PrintHeader(register Volume * vp)
 	 V_dayUse(vp), V_weekUse(vp)[0], V_weekUse(vp)[1], V_weekUse(vp)[2],
 	 V_weekUse(vp)[3], V_weekUse(vp)[4], V_weekUse(vp)[5],
 	 V_weekUse(vp)[6], date(V_dayUseDate(vp)));
+    printf("volUpdateCounter = %u\n", V_volUpCounter(vp));
 }
 
 /* GetFileInfo
@@ -745,14 +749,17 @@ PrintVnodes(Volume * vp, VnodeClass class)
     char buf[SIZEOF_LARGEDISKVNODE];
     struct VnodeDiskObject *vnode = (struct VnodeDiskObject *)buf;
     StreamHandle_t *file;
-    register int vnodeIndex, nVnodes, offset = 0;
+    int vnodeIndex, nVnodes;
+    afs_foff_t offset = 0;
     Inode ino;
     IHandle_t *ih = vp->vnodeIndex[class].handle;
     FdHandle_t *fdP;
     int size;
     char *ctime, *atime, *mtime;
     char nfile[50], buffer[256];
-    int total, ofd, len, code, bad = 0;
+    int ofd, bad = 0;
+    afs_foff_t total;
+    ssize_t len;
 
     fdP = IH_OPEN(ih);
     if (fdP == NULL) {
@@ -774,7 +781,7 @@ PrintVnodes(Volume * vp, VnodeClass class)
 
     nVnodes = (size / diskSize) - 1;
     if (nVnodes > 0) {
-	STREAM_SEEK(file, diskSize, 0);
+	STREAM_ASEEK(file, diskSize);
     } else
 	nVnodes = 0;
 
@@ -804,7 +811,8 @@ PrintVnodes(Volume * vp, VnodeClass class)
 		}
 		total = bad = 0;
 		while (1) {
-		    len = FDH_READ(fdP1, buffer, sizeof(buffer));
+		    ssize_t nBytes;
+		    len = FDH_PREAD(fdP1, buffer, sizeof(buffer), total);
 		    if (len < 0) {
 			FDH_REALLYCLOSE(fdP1);
 			IH_RELEASE(ih1);
@@ -818,8 +826,8 @@ PrintVnodes(Volume * vp, VnodeClass class)
 		    }
 		    if (len == 0)
 			break;	/* No more input */
-		    code = write(ofd, buffer, len);
-		    if (code != len) {
+		    nBytes = write(ofd, buffer, len);
+		    if (nBytes != len) {
 			FDH_REALLYCLOSE(fdP1);
 			IH_RELEASE(ih1);
 			close(ofd);
@@ -837,8 +845,8 @@ PrintVnodes(Volume * vp, VnodeClass class)
 		FDH_REALLYCLOSE(fdP1);
 		IH_RELEASE(ih1);
 		close(ofd);
-		printf("... Copied inode %s to file %s (%d bytes)\n",
-		       PrintInode(NULL, ino), nfile, total);
+		printf("... Copied inode %s to file %s (%lu bytes)\n",
+		       PrintInode(NULL, ino), nfile, (unsigned long)total);
 	    }
 	} else {
 #if defined(AFS_NAMEI_ENV)
@@ -856,21 +864,17 @@ PrintVnodes(Volume * vp, VnodeClass class)
 
 #if defined(AFS_NAMEI_ENV)
 void
-PrintVnode(int offset, VnodeDiskObject * vnode, VnodeId vnodeNumber,
+PrintVnode(afs_foff_t offset, VnodeDiskObject * vnode, VnodeId vnodeNumber,
 	   Inode ino, Volume * vp)
 #else
 void
-PrintVnode(int offset, VnodeDiskObject * vnode, VnodeId vnodeNumber,
+PrintVnode(afs_foff_t offset, VnodeDiskObject * vnode, VnodeId vnodeNumber,
 	   Inode ino)
 #endif
 {
 #if defined(AFS_NAMEI_ENV)
     IHandle_t *ihtmpp;
-#if !defined(AFS_NT40_ENV)
     namei_t filename;
-#else
-    char filename[MAX_PATH];
-#endif
 #endif
     afs_fsize_t fileLength;
 
@@ -882,8 +886,8 @@ PrintVnode(int offset, VnodeDiskObject * vnode, VnodeId vnodeNumber,
     if (orphaned && (fileLength == 0 || vnode->parent || !offset))
 	return;
     printf
-	("%10d Vnode %u.%u.%u cloned: %u, length: %llu linkCount: %d parent: %u",
-	 offset, vnodeNumber, vnode->uniquifier, vnode->dataVersion,
+	("%10lld Vnode %u.%u.%u cloned: %u, length: %llu linkCount: %d parent: %u",
+	 (long long)offset, vnodeNumber, vnode->uniquifier, vnode->dataVersion,
 	 vnode->cloned, (afs_uintmax_t) fileLength, vnode->linkCount,
 	 vnode->parent);
     if (DumpInodeNumber)
@@ -893,12 +897,11 @@ PrintVnode(int offset, VnodeDiskObject * vnode, VnodeId vnodeNumber,
 #if defined(AFS_NAMEI_ENV)
     if (PrintFileNames) {
 	IH_INIT(ihtmpp, V_device(vp), V_parentId(vp), ino);
-#if !defined(AFS_NT40_ENV)
 	namei_HandleToName(&filename, ihtmpp);
+#if !defined(AFS_NT40_ENV)
 	printf(" UFS-Filename: %s", filename.n_path);
 #else
-	nt_HandleToName(filename, ihtmpp);
-	printf(" NTFS-Filename: %s", filename);
+	printf(" NTFS-Filename: %s", filename.n_path);
 #endif
     }
 #endif
