@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -26,11 +26,7 @@
 #include <unistd.h>
 #endif
 #include <sys/stat.h>
-#ifdef AFS_PTHREAD_ENV
-#include <assert.h>
-#else /* AFS_PTHREAD_ENV */
-#include <afs/assert.h>
-#endif /* AFS_PTHREAD_ENV */
+#include <afs/afs_assert.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <afs/afsint.h>
@@ -43,10 +39,14 @@
 #include <afs/volume.h>
 #include <afs/partition.h>
 #include "dump.h"
+#include <afs/daemon_com.h>
 #include <afs/fssync.h>
 #include <afs/acl.h>
+#include <afs/com_err.h>
+#include <afs/vol_prototypes.h>
 #include "volser.h"
 #include "volint.h"
+#include "dumpstuff.h"
 
 #ifndef AFS_NT40_ENV
 #ifdef O_LARGEFILE
@@ -62,58 +62,38 @@
 
 extern int DoLogging;
 
-/* This iod stuff is a silly little package to emulate the old qi_in stuff, which
-   emulated the stdio stuff.  There is a big assumption here, that the
-   rx_Read will never be called directly, by a routine like readFile, when
-   there is an old character that was pushed back with iod_ungetc.  This
-   is really bletchy, and is here for compatibility only.  Eventually,
-   we should define a volume format that doesn't require
-   the pushing back of characters (i.e. characters should not double both
-   as an end marker and a begin marker) */
-struct iod {
-    struct rx_call *call;	/* call to which to write, might be an array */
-    int device;			/* dump device ID for volume */
-    int parentId;		/* dump parent ID for volume */
-    struct DiskPartition64 *dumpPartition;	/* Dump partition. */
-    struct rx_call **calls;	/* array of pointers to calls */
-    int ncalls;			/* how many calls/codes in array */
-    int *codes;			/* one return code for each call */
-    char haveOldChar;		/* state for pushing back a character */
-    char oldChar;
-};
-
 
 /* Forward Declarations */
-static int DumpDumpHeader(register struct iod *iodp, register Volume * vp,
+static int DumpDumpHeader(struct iod *iodp, Volume * vp,
 			  afs_int32 fromtime);
-static int DumpPartial(register struct iod *iodp, register Volume * vp,
+static int DumpPartial(struct iod *iodp, Volume * vp,
 		       afs_int32 fromtime, int dumpAllDirs);
-static int DumpVnodeIndex(register struct iod *iodp, Volume * vp,
+static int DumpVnodeIndex(struct iod *iodp, Volume * vp,
 			  VnodeClass class, afs_int32 fromtime,
 			  int forcedump);
-static int DumpVnode(register struct iod *iodp, struct VnodeDiskObject *v,
+static int DumpVnode(struct iod *iodp, struct VnodeDiskObject *v,
 		     int volid, int vnodeNumber, int dumpEverything);
-static int ReadDumpHeader(register struct iod *iodp, struct DumpHeader *hp);
-static int ReadVnodes(register struct iod *iodp, Volume * vp, int incremental,
-		      afs_int32 * Lbuf, afs_int32 s1, afs_int32 * Sbuf,
+static int ReadDumpHeader(struct iod *iodp, struct DumpHeader *hp);
+static int ReadVnodes(struct iod *iodp, Volume * vp, int incremental,
+		      afs_foff_t * Lbuf, afs_int32 s1, afs_foff_t * Sbuf,
 		      afs_int32 s2, afs_int32 delo);
 static afs_fsize_t volser_WriteFile(int vn, struct iod *iodp,
 				    FdHandle_t * handleP, int tag,
 				    Error * status);
 
-static int SizeDumpDumpHeader(register struct iod *iodp, register Volume * vp,
+static int SizeDumpDumpHeader(struct iod *iodp, Volume * vp,
 			      afs_int32 fromtime,
-			      register struct volintSize *size);
-static int SizeDumpPartial(register struct iod *iodp, register Volume * vp,
+			      struct volintSize *size);
+static int SizeDumpPartial(struct iod *iodp, Volume * vp,
 			   afs_int32 fromtime, int dumpAllDirs,
-			   register struct volintSize *size);
-static int SizeDumpVnodeIndex(register struct iod *iodp, Volume * vp,
+			   struct volintSize *size);
+static int SizeDumpVnodeIndex(struct iod *iodp, Volume * vp,
 			      VnodeClass class, afs_int32 fromtime,
 			      int forcedump,
-			      register struct volintSize *size);
-static int SizeDumpVnode(register struct iod *iodp, struct VnodeDiskObject *v,
+			      struct volintSize *size);
+static int SizeDumpVnode(struct iod *iodp, struct VnodeDiskObject *v,
 			 int volid, int vnodeNumber, int dumpEverything,
-			 register struct volintSize *size);
+			 struct volintSize *size);
 
 #define MAX_SECTIONS    3
 #define MIN_TLV_TAG     5
@@ -131,7 +111,7 @@ RegisterTag(afs_int32 section, unsigned char tag)
 }
 
 static void
-initNonStandardTags()
+initNonStandardTags(void)
 {
     RegisterTag(0, 'n');                /* volume name */
     RegisterTag(0, 't');                /* fromtime, V_backupDate */
@@ -158,7 +138,7 @@ initNonStandardTags()
 }
 
 static void
-iod_Init(register struct iod *iodp, register struct rx_call *call)
+iod_Init(struct iod *iodp, struct rx_call *call)
 {
     iodp->call = call;
     iodp->haveOldChar = 0;
@@ -184,8 +164,8 @@ iod_InitMulti(struct iod *iodp, struct rx_call **calls, int ncalls,
 /* For the single dump case, it's ok to just return the "bytes written"
  * that rx_Write returns, since all the callers of iod_Write abort when
  * the returned value is less than they expect.  For the multi dump case,
- * I don't think we want half the replicas to go bad just because one 
- * connection timed out, but if they all time out, then we should give up. 
+ * I don't think we want half the replicas to go bad just because one
+ * connection timed out, but if they all time out, then we should give up.
  */
 static int
 iod_Write(struct iod *iodp, char *buf, int nbytes)
@@ -193,7 +173,7 @@ iod_Write(struct iod *iodp, char *buf, int nbytes)
     int code, i;
     int one_success = 0;
 
-    assert((iodp->call && iodp->ncalls == 1 && !iodp->calls)
+    osi_Assert((iodp->call && iodp->ncalls == 1 && !iodp->calls)
 	   || (!iodp->call && iodp->ncalls >= 1 && iodp->calls));
 
     if (iodp->call) {
@@ -227,7 +207,7 @@ iod_ungetc(struct iod *iodp, int achar)
 }
 
 static int
-iod_getc(register struct iod *iodp)
+iod_getc(struct iod *iodp)
 {
     unsigned char t;
 
@@ -235,15 +215,15 @@ iod_getc(register struct iod *iodp)
 	iodp->haveOldChar = 0;
 	return iodp->oldChar;
     }
-    if (iod_Read(iodp, &t, 1) == 1)
+    if (iod_Read(iodp, (char *) &t, 1) == 1)
 	return t;
     return EOF;
 }
 
 static int
-ReadShort(register struct iod *iodp, register unsigned short *sp)
+ReadShort(struct iod *iodp, unsigned short *sp)
 {
-    register b1, b0;
+    int b1, b0;
     b1 = iod_getc(iodp);
     if (b1 == EOF)
 	return 0;
@@ -255,9 +235,9 @@ ReadShort(register struct iod *iodp, register unsigned short *sp)
 }
 
 static int
-ReadInt32(register struct iod *iodp, afs_uint32 * lp)
+ReadInt32(struct iod *iodp, afs_uint32 * lp)
 {
-    afs_uint32 register b3, b2, b1, b0;
+    afs_uint32 b3, b2, b1, b0;
     b3 = iod_getc(iodp);
     if (b3 == EOF)
 	return 0;
@@ -275,10 +255,9 @@ ReadInt32(register struct iod *iodp, afs_uint32 * lp)
 }
 
 static void
-ReadString(register struct iod *iodp, register char *to, register int maxa)
+ReadString(struct iod *iodp, char *to, int maxa)
 {
-    register int c;
-    int first = 1;
+    int c;
 
     *to = '\0';
     if (maxa == 0)
@@ -295,8 +274,8 @@ ReadString(register struct iod *iodp, register char *to, register int maxa)
 }
 
 static void
-ReadByteString(register struct iod *iodp, register byte * to,
-	       register int size)
+ReadByteString(struct iod *iodp, byte * to,
+	       int size)
 {
     while (size--)
 	*to++ = iod_getc(iodp);
@@ -306,7 +285,7 @@ ReadByteString(register struct iod *iodp, register byte * to,
  * returns 1 on success and 0 otherwise
  */
 static afs_int32
-ReadStandardTagLen(register struct iod *iodp, unsigned char tag, afs_int32 section,
+ReadStandardTagLen(struct iod *iodp, unsigned char tag, afs_int32 section,
                         afs_size_t *length)
 {
     afs_int32 code, i;
@@ -330,7 +309,7 @@ ReadStandardTagLen(register struct iod *iodp, unsigned char tag, afs_int32 secti
             *length = len;
         else {
             len &= 0x7f;
-            if (code = iod_Read(iodp, buf, len) != len)
+            if ((code = iod_Read(iodp, (char *)buf, len)) != len)
                 return VOLSERDUMPERROR;
             *length = 0;
             p = (unsigned char *)&buf;
@@ -348,14 +327,14 @@ ReadStandardTagLen(register struct iod *iodp, unsigned char tag, afs_int32 secti
 static char skipbuf[256];
 
 static afs_int32
-SkipData(register struct iod *iodp, afs_size_t length)
+SkipData(struct iod *iodp, afs_size_t length)
 {
     while (length > 256) {
-        if (iod_Read(iodp, &skipbuf, 256) != 256)
+        if (iod_Read(iodp, (char *)&skipbuf, 256) != 256)
             return 0;
         length -= 256;
     }
-    if (iod_Read(iodp, &skipbuf, length) != length)
+    if (iod_Read(iodp, (char *)&skipbuf, length) != length)
         return 0;
     return 1;
 }
@@ -363,10 +342,10 @@ SkipData(register struct iod *iodp, afs_size_t length)
 static char *secname[3] = {"ReadDumpHeader", "ReadVolumeHeader", "ReadVnodes"};
 
 static int
-HandleUnknownTag(struct iod *iodp, register tag, afs_int32 section, 
+HandleUnknownTag(struct iod *iodp, int tag, afs_int32 section,
 		afs_int32 critical)
 {
-    afs_size_t taglen;
+    afs_size_t taglen = 0;
     afs_uint32 trash;
 
     if (critical) {
@@ -403,9 +382,9 @@ HandleUnknownTag(struct iod *iodp, register tag, afs_int32 section,
 }
 
 static int
-ReadVolumeHeader(register struct iod *iodp, VolumeDiskData * vol)
+ReadVolumeHeader(struct iod *iodp, VolumeDiskData * vol)
 {
-    register tag;
+    int tag;
     afs_uint32 trash;
     afs_int32 critical = 0;
     memset(vol, 0, sizeof(*vol));
@@ -527,7 +506,7 @@ ReadVolumeHeader(register struct iod *iodp, VolumeDiskData * vol)
 		return VOLSERREAD_DUMPERROR;
 	    break;
 	case 'V':
-	    if (!ReadInt32(iodp, &trash/*volUpdateCounter*/))
+	    if (!ReadInt32(iodp, (afs_uint32 *) &trash/*volUpdateCounter*/))
 		return VOLSERREAD_DUMPERROR;
 	    break;
         case 0x7e:
@@ -543,7 +522,7 @@ ReadVolumeHeader(register struct iod *iodp, VolumeDiskData * vol)
 }
 
 static int
-DumpTag(register struct iod *iodp, register int tag)
+DumpTag(struct iod *iodp, int tag)
 {
     char p;
 
@@ -553,10 +532,10 @@ DumpTag(register struct iod *iodp, register int tag)
 }
 
 static int
-DumpByte(register struct iod *iodp, char tag, byte value)
+DumpByte(struct iod *iodp, char tag, byte value)
 {
     char tbuffer[2];
-    register byte *p = (unsigned char *)tbuffer;
+    byte *p = (unsigned char *)tbuffer;
     *p++ = tag;
     *p = value;
     return ((iod_Write(iodp, tbuffer, 2) == 2) ? 0 : VOLSERDUMPERROR);
@@ -566,11 +545,11 @@ DumpByte(register struct iod *iodp, char tag, byte value)
 #define putshort(p, v) *p++ = v>>8, *p++ = v
 
 static int
-DumpDouble(register struct iod *iodp, char tag, register afs_uint32 value1,
-	   register afs_uint32 value2)
+DumpDouble(struct iod *iodp, char tag, afs_uint32 value1,
+	   afs_uint32 value2)
 {
     char tbuffer[9];
-    register byte *p = (unsigned char *)tbuffer;
+    byte *p = (unsigned char *)tbuffer;
     *p++ = tag;
     putint32(p, value1);
     putint32(p, value2);
@@ -578,23 +557,23 @@ DumpDouble(register struct iod *iodp, char tag, register afs_uint32 value1,
 }
 
 static int
-DumpInt32(register struct iod *iodp, char tag, register afs_uint32 value)
+DumpInt32(struct iod *iodp, char tag, afs_uint32 value)
 {
     char tbuffer[5];
-    register byte *p = (unsigned char *)tbuffer;
+    byte *p = (unsigned char *)tbuffer;
     *p++ = tag;
     putint32(p, value);
     return ((iod_Write(iodp, tbuffer, 5) == 5) ? 0 : VOLSERDUMPERROR);
 }
 
 static int
-DumpArrayInt32(register struct iod *iodp, char tag,
-	       register afs_uint32 * array, register int nelem)
+DumpArrayInt32(struct iod *iodp, char tag,
+	       afs_uint32 * array, int nelem)
 {
     char tbuffer[4];
-    register afs_uint32 v;
+    afs_uint32 v;
     int code = 0;
-    register byte *p = (unsigned char *)tbuffer;
+    byte *p = (unsigned char *)tbuffer;
     *p++ = tag;
     putshort(p, nelem);
     code = iod_Write(iodp, tbuffer, 3);
@@ -613,10 +592,10 @@ DumpArrayInt32(register struct iod *iodp, char tag,
 }
 
 static int
-DumpShort(register struct iod *iodp, char tag, unsigned int value)
+DumpShort(struct iod *iodp, char tag, unsigned int value)
 {
     char tbuffer[3];
-    register byte *p = (unsigned char *)tbuffer;
+    byte *p = (unsigned char *)tbuffer;
     *p++ = tag;
     *p++ = value >> 8;
     *p = value;
@@ -624,19 +603,19 @@ DumpShort(register struct iod *iodp, char tag, unsigned int value)
 }
 
 static int
-DumpBool(register struct iod *iodp, char tag, unsigned int value)
+DumpBool(struct iod *iodp, char tag, unsigned int value)
 {
     char tbuffer[2];
-    register byte *p = (unsigned char *)tbuffer;
+    byte *p = (unsigned char *)tbuffer;
     *p++ = tag;
     *p = value;
     return ((iod_Write(iodp, tbuffer, 2) == 2) ? 0 : VOLSERDUMPERROR);
 }
 
 static int
-DumpString(register struct iod *iodp, char tag, register char *s)
+DumpString(struct iod *iodp, char tag, char *s)
 {
-    register n;
+    int n;
     int code = 0;
     code = iod_Write(iodp, &tag, 1);
     if (code != 1)
@@ -649,8 +628,8 @@ DumpString(register struct iod *iodp, char tag, register char *s)
 }
 
 static int
-DumpByteString(register struct iod *iodp, char tag, register byte * bs,
-	       register int nbytes)
+DumpByteString(struct iod *iodp, char tag, byte * bs,
+	       int nbytes)
 {
     int code = 0;
 
@@ -664,7 +643,7 @@ DumpByteString(register struct iod *iodp, char tag, register byte * bs,
 }
 
 static afs_int32
-DumpStandardTag(register struct iod *iodp, char tag, afs_uint32 section)
+DumpStandardTag(struct iod *iodp, char tag, afs_uint32 section)
 {
     afs_int32 code;
     afs_uint32 off = tag >> 5;
@@ -681,11 +660,14 @@ DumpStandardTag(register struct iod *iodp, char tag, afs_uint32 section)
         return VOLSERDUMPERROR;
     }
     code = iod_Write(iodp, &tag, 1);
+    if (code != 1)
+	return VOLSERDUMPERROR;
     return 0;
 }
 
+AFS_UNUSED
 static afs_int32
-DumpStandardTagLen(register struct iod *iodp, char tag, afs_uint32 section,
+DumpStandardTagLen(struct iod *iodp, char tag, afs_uint32 section,
                         afs_size_t length)
 {
     char buf[10];
@@ -722,21 +704,26 @@ static int
 DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
 {
     int code = 0, error = 0;
-    afs_int32 pad = 0, offset;
-    afs_sfsize_t n, nbytes, howMany, howBig;
-    afs_foff_t lcode = 0;
+    afs_int32 pad = 0;
+    afs_foff_t offset = 0;
+    afs_sfsize_t nbytes, howBig;
+    ssize_t n;
+    size_t howMany;
+    afs_foff_t howFar = 0;
     byte *p;
+    afs_uint32 hi, lo;
+    afs_ino_str_t stmp;
 #ifndef AFS_NT40_ENV
     struct afs_stat status;
 #endif
     afs_sfsize_t size;
 #ifdef	AFS_AIX_ENV
 #include <sys/statfs.h>
-#if defined(AFS_AIX52_ENV) && defined(AFS_LARGEFILE_ENV)
+#if defined(AFS_AIX52_ENV)
     struct statfs64 tstatfs;
-#else /* !AFS_AIX52_ENV || !AFS_LARGEFILE_ENV */
+#else /* !AFS_AIX52_ENV */
     struct statfs tstatfs;
-#endif /* !AFS_AIX52_ENV || !AFS_LARGEFILE_ENV */
+#endif /* !AFS_AIX52_ENV */
     int statfs_code;
 #endif
 
@@ -749,19 +736,19 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
     howBig = status.st_size;
 
 #ifdef	AFS_AIX_ENV
-    /* Unfortunately in AIX valuable fields such as st_blksize are 
+    /* Unfortunately in AIX valuable fields such as st_blksize are
      * gone from the stat structure.
      */
-#if defined(AFS_AIX52_ENV) && defined(AFS_LARGEFILE_ENV)
+#if defined(AFS_AIX52_ENV)
     statfs_code = fstatfs64(handleP->fd_fd, &tstatfs);
-#else /* !AFS_AIX52_ENV || !AFS_LARGEFILE_ENV */
+#else /* !AFS_AIX52_ENV */
     statfs_code = fstatfs(handleP->fd_fd, &tstatfs);
-#endif /* !AFS_AIX52_ENV || !AFS_LARGEFILE_ENV */
+#endif /* !AFS_AIX52_ENV */
     if (statfs_code != 0) {
         Log("DumpFile: fstatfs returned error code %d on descriptor %d\n", errno, handleP->fd_fd);
 	return VOLSERDUMPERROR;
     }
-    howMany = (afs_sfsize_t) tstatfs.f_bsize;
+    howMany = tstatfs.f_bsize;
 #else
     howMany = status.st_blksize;
 #endif /* AFS_AIX_ENV */
@@ -769,26 +756,19 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
 
 
     size = FDH_SIZE(handleP);
-#ifdef AFS_LARGEFILE_ENV
-    {
-	afs_uint32 hi, lo;
-	SplitInt64(size, hi, lo);
-	if (hi == 0L) {
-	    code = DumpInt32(iodp, 'f', lo);
-	} else {
-	    code = DumpDouble(iodp, 'h', hi, lo);
-	}
+    SplitInt64(size, hi, lo);
+    if (hi == 0L) {
+	code = DumpInt32(iodp, 'f', lo);
+    } else {
+	code = DumpDouble(iodp, 'h', hi, lo);
     }
-#else /* !AFS_LARGEFILE_ENV */
-    code = DumpInt32(iodp, 'f', size);
-#endif /* !AFS_LARGEFILE_ENV */
     if (code) {
 	return VOLSERDUMPERROR;
     }
 
-    p = (unsigned char *)malloc((size_t)howMany);
+    p = malloc(howMany);
     if (!p) {
-	Log("1 Volser: DumpFile: not enough memory to allocate %u bytes\n", howMany);
+	Log("1 Volser: DumpFile: not enough memory to allocate %u bytes\n", (unsigned)howMany);
 	return VOLSERDUMPERROR;
     }
 
@@ -796,14 +776,15 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
 	if (nbytes < howMany)
 	    howMany = nbytes;
 
-	/* Read the data - unless we know we can't */
-	n = (lcode ? 0 : FDH_READ(handleP, p, (size_t)howMany));
+	/* Read the data */
+	n = FDH_PREAD(handleP, p, howMany, howFar);
+	howFar += n;
 
 	/* If read any good data and we null padded previously, log the
 	 * amount that we had null padded.
 	 */
 	if ((n > 0) && pad) {
-	    Log("1 Volser: DumpFile: Null padding file %d bytes at offset %u\n", pad, offset);
+	    Log("1 Volser: DumpFile: Null padding file %d bytes at offset %lld\n", pad, (long long)offset);
 	    pad = 0;
 	}
 
@@ -815,12 +796,12 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
 	    /* Record the read error */
 	    if (n < 0) {
 		n = 0;
-		Log("1 Volser: DumpFile: Error %d reading inode %s for vnode %d\n", errno, PrintInode(NULL, handleP->fd_ih->ih_ino), vnode);
+		Log("1 Volser: DumpFile: Error reading inode %s for vnode %d: %s\n", PrintInode(stmp, handleP->fd_ih->ih_ino), vnode, afs_error_message(errno));
 	    } else if (!pad) {
-		Log("1 Volser: DumpFile: Error reading inode %s for vnode %d\n", PrintInode(NULL, handleP->fd_ih->ih_ino), vnode);
+		Log("1 Volser: DumpFile: Error reading inode %s for vnode %d\n", PrintInode(stmp, handleP->fd_ih->ih_ino), vnode);
 	    }
 
-	    /* Pad the rest of the buffer with zeros. Remember offset we started 
+	    /* Pad the rest of the buffer with zeros. Remember offset we started
 	     * padding. Keep total tally of padding.
 	     */
 	    memset(p + n, 0, howMany - n);
@@ -831,21 +812,11 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
 	    /* Now seek over the data we could not get. An error here means we
 	     * can't do the next read.
 	     */
-	    lcode = FDH_SEEK(handleP, (size_t)((size - nbytes) + howMany), SEEK_SET);
-	    if (lcode != ((size - nbytes) + howMany)) {
-		if (lcode < 0) {
-		    Log("1 Volser: DumpFile: Error %d seeking in inode %s for vnode %d\n", errno, PrintInode(NULL, handleP->fd_ih->ih_ino), vnode);
-		} else {
-		    Log("1 Volser: DumpFile: Error seeking in inode %s for vnode %d\n", PrintInode(NULL, handleP->fd_ih->ih_ino), vnode);
-		    lcode = -1;
-		}
-	    } else {
-		lcode = 0;
-	    }
+	    howFar = (size_t)((size - nbytes) + howMany);
 	}
 
 	/* Now write the data out */
-	if (iod_Write(iodp, (char *)p, (size_t)howMany) != howMany)
+	if (iod_Write(iodp, (char *)p, howMany) != howMany)
 	    error = VOLSERDUMPERROR;
 #ifndef AFS_PTHREAD_ENV
 	IOMGR_Poll();
@@ -853,8 +824,8 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
     }
 
     if (pad) {			/* Any padding we hadn't reported yet */
-	Log("1 Volser: DumpFile: Null padding file: %d bytes at offset %u\n",
-	    pad, offset);
+	Log("1 Volser: DumpFile: Null padding file: %d bytes at offset %lld\n",
+	    pad, (long long)offset);
     }
 
     free(p);
@@ -862,7 +833,7 @@ DumpFile(struct iod *iodp, int vnode, FdHandle_t * handleP)
 }
 
 static int
-DumpVolumeHeader(register struct iod *iodp, register Volume * vp)
+DumpVolumeHeader(struct iod *iodp, Volume * vp)
 {
     int code = 0;
     static char nullString[1] = "";	/*The ``contents'' of motd */
@@ -933,7 +904,7 @@ DumpVolumeHeader(register struct iod *iodp, register Volume * vp)
 }
 
 static int
-DumpEnd(register struct iod *iodp)
+DumpEnd(struct iod *iodp)
 {
     return (DumpInt32(iodp, D_DUMPEND, DUMPENDMAGIC));
 }
@@ -942,12 +913,12 @@ DumpEnd(register struct iod *iodp)
 
 /* Dump a whole volume */
 int
-DumpVolume(register struct rx_call *call, register Volume * vp,
+DumpVolume(struct rx_call *call, Volume * vp,
 	   afs_int32 fromtime, int dumpAllDirs)
 {
     struct iod iod;
     int code = 0;
-    register struct iod *iodp = &iod;
+    struct iod *iodp = &iod;
     iod_Init(iodp, call);
 
     if (!code)
@@ -988,7 +959,7 @@ DumpVolMulti(struct rx_call **calls, int ncalls, Volume * vp,
 
 /* A partial dump (no dump header) */
 static int
-DumpPartial(register struct iod *iodp, register Volume * vp,
+DumpPartial(struct iod *iodp, Volume * vp,
 	    afs_int32 fromtime, int dumpAllDirs)
 {
     int code = 0;
@@ -1002,29 +973,29 @@ DumpPartial(register struct iod *iodp, register Volume * vp,
 }
 
 static int
-DumpVnodeIndex(register struct iod *iodp, Volume * vp, VnodeClass class,
+DumpVnodeIndex(struct iod *iodp, Volume * vp, VnodeClass class,
 	       afs_int32 fromtime, int forcedump)
 {
-    register int code = 0;
-    register struct VnodeClassInfo *vcp = &VnodeClassInfo[class];
+    int code = 0;
+    struct VnodeClassInfo *vcp = &VnodeClassInfo[class];
     char buf[SIZEOF_LARGEDISKVNODE];
     struct VnodeDiskObject *vnode = (struct VnodeDiskObject *)buf;
     StreamHandle_t *file;
     FdHandle_t *fdP;
-    int size;
+    afs_sfsize_t size, nVnodes;
     int flag;
-    register int vnodeIndex, nVnodes;
+    int vnodeIndex;
 
     fdP = IH_OPEN(vp->vnodeIndex[class].handle);
-    assert(fdP != NULL);
+    osi_Assert(fdP != NULL);
     file = FDH_FDOPEN(fdP, "r+");
-    assert(file != NULL);
+    osi_Assert(file != NULL);
     size = OS_SIZE(fdP->fd_fd);
-    assert(size != -1);
+    osi_Assert(size != -1);
     nVnodes = (size / vcp->diskSize) - 1;
     if (nVnodes > 0) {
-	assert((nVnodes + 1) * vcp->diskSize == size);
-	assert(STREAM_SEEK(file, vcp->diskSize, 0) == 0);
+	osi_Assert((nVnodes + 1) * vcp->diskSize == size);
+	osi_Assert(STREAM_ASEEK(file, vcp->diskSize) == 0);
     } else
 	nVnodes = 0;
     for (vnodeIndex = 0;
@@ -1049,7 +1020,7 @@ DumpVnodeIndex(register struct iod *iodp, Volume * vp, VnodeClass class,
 }
 
 static int
-DumpDumpHeader(register struct iod *iodp, register Volume * vp,
+DumpDumpHeader(struct iod *iodp, Volume * vp,
 	       afs_int32 fromtime)
 {
     int code = 0;
@@ -1074,7 +1045,7 @@ DumpDumpHeader(register struct iod *iodp, register Volume * vp,
 }
 
 static int
-DumpVnode(register struct iod *iodp, struct VnodeDiskObject *v, int volid,
+DumpVnode(struct iod *iodp, struct VnodeDiskObject *v, int volid,
 	  int vnodeNumber, int dumpEverything)
 {
     int code = 0;
@@ -1131,18 +1102,19 @@ DumpVnode(register struct iod *iodp, struct VnodeDiskObject *v, int volid,
 
 
 int
-ProcessIndex(Volume * vp, VnodeClass class, afs_int32 ** Bufp, int *sizep,
+ProcessIndex(Volume * vp, VnodeClass class, afs_foff_t ** Bufp, int *sizep,
 	     int del)
 {
-    int i, nVnodes, offset, code, index = 0;
-    afs_int32 *Buf;
+    int i, nVnodes, code;
+    afs_foff_t offset;
+    afs_foff_t *Buf;
     int cnt = 0;
-    int size;
+    afs_sfsize_t size;
     StreamHandle_t *afile;
     FdHandle_t *fdP;
     struct VnodeClassInfo *vcp = &VnodeClassInfo[class];
     char buf[SIZEOF_LARGEDISKVNODE], zero[SIZEOF_LARGEDISKVNODE];
-    register struct VnodeDiskObject *vnode = (struct VnodeDiskObject *)buf;
+    struct VnodeDiskObject *vnode = (struct VnodeDiskObject *)buf;
 
     memset(zero, 0, sizeof(zero));	/* zero out our proto-vnode */
     fdP = IH_OPEN(vp->vnodeIndex[class].handle);
@@ -1155,21 +1127,21 @@ ProcessIndex(Volume * vp, VnodeClass class, afs_int32 ** Bufp, int *sizep,
 	for (i = 0; i < *sizep; i++) {
 	    if (Buf[i]) {
 		cnt++;
-		STREAM_SEEK(afile, Buf[i], 0);
+		STREAM_ASEEK(afile, Buf[i]);
 		code = STREAM_READ(vnode, vcp->diskSize, 1, afile);
 		if (code == 1) {
 		    if (vnode->type != vNull && VNDISK_GET_INO(vnode)) {
 			cnt1++;
 			if (DoLogging) {
-			    Log("RestoreVolume %u Cleanup: Removing old vnode=%u inode=%llu size=unknown\n", 
-                     V_id(vp), bitNumberToVnodeNumber(i, class), 
+			    Log("RestoreVolume %u Cleanup: Removing old vnode=%u inode=%llu size=unknown\n",
+                     V_id(vp), bitNumberToVnodeNumber(i, class),
                      (afs_uintmax_t) VNDISK_GET_INO(vnode));
 			}
 			IH_DEC(V_linkHandle(vp), VNDISK_GET_INO(vnode),
 			       V_parentId(vp));
 			DOPOLL;
 		    }
-		    STREAM_SEEK(afile, Buf[i], 0);
+		    STREAM_ASEEK(afile, Buf[i]);
 		    (void)STREAM_WRITE(zero, vcp->diskSize, 1, afile);	/* Zero it out */
 		}
 		Buf[i] = 0;
@@ -1183,19 +1155,23 @@ ProcessIndex(Volume * vp, VnodeClass class, afs_int32 ** Bufp, int *sizep,
 	OS_SYNC(afile->str_fd);
     } else {
 	size = OS_SIZE(fdP->fd_fd);
-	assert(size != -1);
+	osi_Assert(size != -1);
 	nVnodes =
 	    (size <=
 	     vcp->diskSize ? 0 : size - vcp->diskSize) >> vcp->logSize;
 	if (nVnodes > 0) {
-	    Buf = (afs_int32 *) malloc(nVnodes * sizeof(afs_int32));
+	    if (DoLogging) {
+		Log("RestoreVolume ProcessIndex: Set up %d inodes for volume %d\n",
+		    nVnodes, V_id(vp));
+	    }
+	    Buf = malloc(nVnodes * sizeof(afs_foff_t));
 	    if (Buf == NULL) {
 		STREAM_CLOSE(afile);
 		FDH_CLOSE(fdP);
 		return -1;
 	    }
-	    memset((char *)Buf, 0, nVnodes * sizeof(afs_int32));
-	    STREAM_SEEK(afile, offset = vcp->diskSize, 0);
+	    memset(Buf, 0, nVnodes * sizeof(afs_foff_t));
+	    STREAM_ASEEK(afile, offset = vcp->diskSize);
 	    while (1) {
 		code = STREAM_READ(vnode, vcp->diskSize, 1, afile);
 		if (code != 1) {
@@ -1207,6 +1183,9 @@ ProcessIndex(Volume * vp, VnodeClass class, afs_int32 ** Bufp, int *sizep,
 		}
 		offset += vcp->diskSize;
 	    }
+	    if (DoLogging) {
+		Log("RestoreVolume ProcessIndex: found %d inodes\n", cnt);
+	    }
 	    *Bufp = Buf;
 	    *sizep = nVnodes;
 	}
@@ -1218,17 +1197,17 @@ ProcessIndex(Volume * vp, VnodeClass class, afs_int32 ** Bufp, int *sizep,
 
 
 int
-RestoreVolume(register struct rx_call *call, Volume * avp, int incremental,
+RestoreVolume(struct rx_call *call, Volume * avp, int incremental,
 	      struct restoreCookie *cookie)
 {
     VolumeDiskData vol;
     struct DumpHeader header;
     afs_uint32 endMagic;
     Error error = 0, vupdate;
-    register Volume *vp;
+    Volume *vp;
     struct iod iod;
-    register struct iod *iodp = &iod;
-    afs_int32 *b1 = NULL, *b2 = NULL;
+    struct iod *iodp = &iod;
+    afs_foff_t *b1 = NULL, *b2 = NULL;
     int s1 = 0, s2 = 0, delo = 0, tdelo;
     int tag;
 
@@ -1321,13 +1300,13 @@ RestoreVolume(register struct rx_call *call, Volume * avp, int incremental,
 }
 
 static int
-ReadVnodes(register struct iod *iodp, Volume * vp, int incremental,
-	   afs_int32 * Lbuf, afs_int32 s1, afs_int32 * Sbuf, afs_int32 s2,
+ReadVnodes(struct iod *iodp, Volume * vp, int incremental,
+	   afs_foff_t * Lbuf, afs_int32 s1, afs_foff_t * Sbuf, afs_int32 s2,
 	   afs_int32 delo)
 {
     afs_int32 vnodeNumber;
     char buf[SIZEOF_LARGEDISKVNODE];
-    register tag;
+    int tag;
     struct VnodeDiskObject *vnode = (struct VnodeDiskObject *)buf;
     struct VnodeDiskObject oldvnode;
     int idx;
@@ -1342,12 +1321,17 @@ ReadVnodes(register struct iod *iodp, Volume * vp, int incremental,
     V_pref(vp, nearInode);
     while (tag == D_VNODE) {
 	int haveStuff = 0;
+	int saw_f = 0;
 	memset(buf, 0, sizeof(buf));
 	if (!ReadInt32(iodp, (afs_uint32 *) & vnodeNumber))
 	    break;
 
 	if (!ReadInt32(iodp, &vnode->uniquifier))
 	    return VOLSERREAD_DUMPERROR;
+
+	if (DoLogging) {
+	    Log("ReadVnodes: setup %d/%d\n", vnodeNumber, vnode->uniquifier);
+	}
 	while ((tag = iod_getc(iodp)) > D_MAX && tag != EOF) {
 	    haveStuff = 1;
             if (critical)
@@ -1404,14 +1388,21 @@ ReadVnodes(register struct iod *iodp, Volume * vp, int incremental,
 			       VAclDiskSize(vnode));
 		acl_NtohACL(VVnodeDiskACL(vnode));
 		break;
-#ifdef AFS_LARGEFILE_ENV
 	    case 'h':
-#endif
 	    case 'f':{
 		    Inode ino;
 		    Error error;
 		    afs_fsize_t vnodeLength;
 
+		    if (saw_f) {
+			Log("Volser: ReadVnodes: warning: ignoring duplicate "
+			    "file entries for vnode %lu in dump\n",
+			    (unsigned long)vnodeNumber);
+			volser_WriteFile(vnodeNumber, iodp, NULL, tag, &error);
+			break;
+		    }
+		    saw_f = 1;
+
 		    ino =
 			IH_CREATE(V_linkHandle(vp), V_device(vp),
 				  VPartitionPath(V_partition(vp)), nearInode,
@@ -1472,16 +1463,11 @@ ReadVnodes(register struct iod *iodp, Volume * vp, int incremental,
 	if (haveStuff) {
 	    FdHandle_t *fdP = IH_OPEN(vp->vnodeIndex[class].handle);
 	    if (fdP == NULL) {
-		Log("1 Volser: ReadVnodes: Error opening vnode index; restore aborted\n");
+		Log("1 Volser: ReadVnodes: Error opening vnode index: %s; restore aborted\n",
+		    afs_error_message(errno));
 		return VOLSERREAD_DUMPERROR;
 	    }
-	    if (FDH_SEEK(fdP, vnodeIndexOffset(vcp, vnodeNumber), SEEK_SET) <
-		0) {
-		Log("1 Volser: ReadVnodes: Error seeking into vnode index; restore aborted\n");
-		FDH_REALLYCLOSE(fdP);
-		return VOLSERREAD_DUMPERROR;
-	    }
-	    if (FDH_READ(fdP, &oldvnode, sizeof(oldvnode)) ==
+	    if (FDH_PREAD(fdP, &oldvnode, sizeof(oldvnode), vnodeIndexOffset(vcp, vnodeNumber)) ==
 		sizeof(oldvnode)) {
 		if (oldvnode.type != vNull && VNDISK_GET_INO(&oldvnode)) {
 		    IH_DEC(V_linkHandle(vp), VNDISK_GET_INO(&oldvnode),
@@ -1489,14 +1475,9 @@ ReadVnodes(register struct iod *iodp, Volume * vp, int incremental,
 		}
 	    }
 	    vnode->vnodeMagic = vcp->magic;
-	    if (FDH_SEEK(fdP, vnodeIndexOffset(vcp, vnodeNumber), SEEK_SET) <
-		0) {
-		Log("1 Volser: ReadVnodes: Error seeking into vnode index; restore aborted\n");
-		FDH_REALLYCLOSE(fdP);
-		return VOLSERREAD_DUMPERROR;
-	    }
-	    if (FDH_WRITE(fdP, vnode, vcp->diskSize) != vcp->diskSize) {
-		Log("1 Volser: ReadVnodes: Error writing vnode index; restore aborted\n");
+	    if (FDH_PWRITE(fdP, vnode, vcp->diskSize, vnodeIndexOffset(vcp, vnodeNumber)) != vcp->diskSize) {
+		Log("1 Volser: ReadVnodes: Error writing vnode index: %s; restore aborted\n",
+		    afs_error_message(errno));
 		FDH_REALLYCLOSE(fdP);
 		return VOLSERREAD_DUMPERROR;
 	    }
@@ -1512,17 +1493,20 @@ ReadVnodes(register struct iod *iodp, Volume * vp, int incremental,
 /* called with disk file only.  Note that we don't have to worry about rx_Read
  * needing to read an ungetc'd character, since the ReadInt32 will have read
  * it instead.
+ *
+ * if handleP == NULL, don't write the file anywhere; just read and discard
+ * the file contents
  */
 static afs_fsize_t
 volser_WriteFile(int vn, struct iod *iodp, FdHandle_t * handleP, int tag,
 		 Error * status)
 {
     afs_int32 code;
-    afs_sfsize_t lcode;
+    ssize_t nBytes;
     afs_fsize_t filesize;
     afs_fsize_t written = 0;
-    register afs_uint32 size = 8192;
-    register afs_fsize_t nbytes;
+    size_t size = 8192;
+    afs_fsize_t nbytes;
     unsigned char *p;
 
 
@@ -1530,14 +1514,12 @@ volser_WriteFile(int vn, struct iod *iodp, FdHandle_t * handleP, int tag,
 #ifdef AFS_64BIT_ENV
     {
 	afs_uint32 filesize_high = 0L, filesize_low = 0L;
-#ifdef AFS_LARGEFILE_ENV
 	if (tag == 'h') {
 	    if (!ReadInt32(iodp, &filesize_high)) {
 		*status = 1;
 		return 0;
 	    }
 	}
-#endif /* !AFS_LARGEFILE_ENV */
 	if (!ReadInt32(iodp, &filesize_low)) {
 	    *status = 1;
 	    return 0;
@@ -1559,18 +1541,20 @@ volser_WriteFile(int vn, struct iod *iodp, FdHandle_t * handleP, int tag,
 	if (nbytes < size)
 	    size = nbytes;
 
-	if ((code = iod_Read(iodp, p, size)) != size) {
-	    Log("1 Volser: WriteFile: Error reading dump file %d size=%llu nbytes=%u (%d of %u); restore aborted\n", vn, (afs_uintmax_t) filesize, nbytes, code, size);
+	if ((code = iod_Read(iodp, (char *) p, size)) != size) {
+	    Log("1 Volser: WriteFile: Error reading dump file %d size=%llu nbytes=%u (%d of %u): %s; restore aborted\n", vn, (afs_uintmax_t) filesize, nbytes, code, (unsigned)size, afs_error_message(errno));
 	    *status = 3;
 	    break;
 	}
-	lcode = FDH_WRITE(handleP, p, size);
-	if (lcode > 0)
-	    written += lcode;
-	if (lcode != size) {
-	    Log("1 Volser: WriteFile: Error writing (%d,%u) bytes to vnode %d; restore aborted\n", (int)(lcode>>32), (int)(lcode & 0xffffffff), vn);
-	    *status = 4;
-	    break;
+	if (handleP) {
+	    nBytes = FDH_PWRITE(handleP, p, size, written);
+	    if (nBytes > 0)
+		written += nBytes;
+	    if (nBytes != size) {
+		Log("1 Volser: WriteFile: Error writing (%u) bytes to vnode %d; %s; restore aborted\n", (int)(nBytes & 0xffffffff), vn, afs_error_message(errno));
+		*status = 4;
+		break;
+	    }
 	}
     }
     free(p);
@@ -1578,9 +1562,9 @@ volser_WriteFile(int vn, struct iod *iodp, FdHandle_t * handleP, int tag,
 }
 
 static int
-ReadDumpHeader(register struct iod *iodp, struct DumpHeader *hp)
+ReadDumpHeader(struct iod *iodp, struct DumpHeader *hp)
 {
-    register tag;
+    int tag;
     afs_uint32 beginMagic;
     afs_int32 critical = 0;
     if (iod_getc(iodp) != D_DUMPHEADER || !ReadInt32(iodp, &beginMagic)
@@ -1591,7 +1575,7 @@ ReadDumpHeader(register struct iod *iodp, struct DumpHeader *hp)
     hp->nDumpTimes = 0;
     while ((tag = iod_getc(iodp)) > D_MAX) {
 	unsigned short arrayLength;
-	register int i;
+	int i;
 	if (critical)
 	    critical--;
 	switch (tag) {
@@ -1630,8 +1614,8 @@ ReadDumpHeader(register struct iod *iodp, struct DumpHeader *hp)
 /* ----- Below are the calls that calculate dump size ----- */
 
 static int
-SizeDumpVolumeHeader(register struct iod *iodp, register Volume * vp,
-		     register struct volintSize *v_size)
+SizeDumpVolumeHeader(struct iod *iodp, Volume * vp,
+		     struct volintSize *v_size)
 {
     int code = 0;
     static char nullString[1] = "";	/*The ``contents'' of motd */
@@ -1725,7 +1709,7 @@ SizeDumpVolumeHeader(register struct iod *iodp, register Volume * vp,
 }
 
 static int
-SizeDumpEnd(register struct iod *iodp, register struct volintSize *v_size)
+SizeDumpEnd(struct iod *iodp, struct volintSize *v_size)
 {
     int code = 0;
     afs_uint64 addvar;
@@ -1735,12 +1719,12 @@ SizeDumpEnd(register struct iod *iodp, register struct volintSize *v_size)
 }
 
 int
-SizeDumpVolume(register struct rx_call *call, register Volume * vp,
+SizeDumpVolume(struct rx_call *call, Volume * vp,
 	       afs_int32 fromtime, int dumpAllDirs,
-	       register struct volintSize *v_size)
+	       struct volintSize *v_size)
 {
     int code = 0;
-    register struct iod *iodp = (struct iod *)0;
+    struct iod *iodp = (struct iod *)0;
 /*    iod_Init(iodp, call); */
 
     if (!code)
@@ -1754,11 +1738,11 @@ SizeDumpVolume(register struct rx_call *call, register Volume * vp,
 }
 
 static int
-SizeDumpDumpHeader(register struct iod *iodp, register Volume * vp,
-		   afs_int32 fromtime, register struct volintSize *v_size)
+SizeDumpDumpHeader(struct iod *iodp, Volume * vp,
+		   afs_int32 fromtime, struct volintSize *v_size)
 {
     int code = 0;
-    int UseLatestReadOnlyClone = 1;
+/*    int UseLatestReadOnlyClone = 1; */
 /*    afs_int32 dumpTimes[2]; */
     afs_uint64 addvar;
 /*    iodp->device = vp->device; */
@@ -1784,9 +1768,9 @@ SizeDumpDumpHeader(register struct iod *iodp, register Volume * vp,
 }
 
 static int
-SizeDumpVnode(register struct iod *iodp, struct VnodeDiskObject *v, int volid,
+SizeDumpVnode(struct iod *iodp, struct VnodeDiskObject *v, int volid,
 	      int vnodeNumber, int dumpEverything,
-	      register struct volintSize *v_size)
+	      struct volintSize *v_size)
 {
     int code = 0;
     afs_uint64 addvar;
@@ -1846,9 +1830,9 @@ SizeDumpVnode(register struct iod *iodp, struct VnodeDiskObject *v, int volid,
 
 /* A partial dump (no dump header) */
 static int
-SizeDumpPartial(register struct iod *iodp, register Volume * vp,
+SizeDumpPartial(struct iod *iodp, Volume * vp,
 		afs_int32 fromtime, int dumpAllDirs,
-		register struct volintSize *v_size)
+		struct volintSize *v_size)
 {
     int code = 0;
     if (!code)
@@ -1863,30 +1847,30 @@ SizeDumpPartial(register struct iod *iodp, register Volume * vp,
 }
 
 static int
-SizeDumpVnodeIndex(register struct iod *iodp, Volume * vp, VnodeClass class,
+SizeDumpVnodeIndex(struct iod *iodp, Volume * vp, VnodeClass class,
 		   afs_int32 fromtime, int forcedump,
-		   register struct volintSize *v_size)
+		   struct volintSize *v_size)
 {
-    register int code = 0;
-    register struct VnodeClassInfo *vcp = &VnodeClassInfo[class];
+    int code = 0;
+    struct VnodeClassInfo *vcp = &VnodeClassInfo[class];
     char buf[SIZEOF_LARGEDISKVNODE];
     struct VnodeDiskObject *vnode = (struct VnodeDiskObject *)buf;
     StreamHandle_t *file;
     FdHandle_t *fdP;
-    int size;
+    afs_sfsize_t size, nVnodes;
     int flag;
-    register int vnodeIndex, nVnodes;
+    int vnodeIndex;
 
     fdP = IH_OPEN(vp->vnodeIndex[class].handle);
-    assert(fdP != NULL);
+    osi_Assert(fdP != NULL);
     file = FDH_FDOPEN(fdP, "r+");
-    assert(file != NULL);
+    osi_Assert(file != NULL);
     size = OS_SIZE(fdP->fd_fd);
-    assert(size != -1);
+    osi_Assert(size != -1);
     nVnodes = (size / vcp->diskSize) - 1;
     if (nVnodes > 0) {
-	assert((nVnodes + 1) * vcp->diskSize == size);
-	assert(STREAM_SEEK(file, vcp->diskSize, 0) == 0);
+	osi_Assert((nVnodes + 1) * vcp->diskSize == size);
+	osi_Assert(STREAM_ASEEK(file, vcp->diskSize) == 0);
     } else
 	nVnodes = 0;
     for (vnodeIndex = 0;
